@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2016-2020 NVIDIA Corporation
+    Copyright (c) 2016-2024 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -86,7 +86,7 @@
 // The size of the VA used for mapping uvm_mem_t allocations
 // 128 GBs should be plenty for internal allocations and fits easily on all
 // supported architectures.
-#define UVM_MEM_VA_SIZE (128ull * 1024 * 1024 * 1024)
+#define UVM_MEM_VA_SIZE (128 * UVM_SIZE_1GB)
 
 typedef struct
 {
@@ -126,7 +126,7 @@ typedef struct
     //
     // CPU mappings will always use PAGE_SIZE, so the physical allocation chunk
     // has to be aligned to PAGE_SIZE.
-    NvU32 page_size;
+    NvU64 page_size;
 
     // If true, the allocation is zeroed (scrubbed).
     bool zero;
@@ -142,10 +142,10 @@ typedef struct
 typedef struct
 {
     // Mask of processors the memory is virtually mapped on
-    uvm_global_processor_mask_t mapped_on;
+    uvm_processor_mask_t mapped_on;
 
     // Page table ranges for all GPUs
-    uvm_page_table_range_vec_t *range_vecs[UVM_GLOBAL_ID_MAX_GPUS];
+    uvm_page_table_range_vec_t *range_vecs[UVM_ID_MAX_GPUS];
 
     uvm_va_space_t *va_space;
 
@@ -161,6 +161,8 @@ struct uvm_mem_struct
     // lifetime of the GPU. For CPU allocations there is no lifetime limitation.
     uvm_gpu_t *backing_gpu;
 
+    // In Confidential Computing, the accessing GPU needs to be known at alloc
+    // time for sysmem allocations.
     uvm_gpu_t *dma_owner;
 
     union
@@ -178,13 +180,13 @@ struct uvm_mem_struct
             // GPU can physical access the memory
             //
             // TODO: Bug 3723779: Share DMA mappings within a single parent GPU
-            uvm_global_processor_mask_t mapped_on_phys;
+            uvm_processor_mask_t mapped_on_phys;
 
             struct page **pages;
             void **va;
 
             // Per GPU IOMMU mappings of the pages
-            NvU64 *dma_addrs[UVM_GLOBAL_ID_MAX_GPUS];
+            NvU64 *dma_addrs[UVM_ID_MAX_GPUS];
         } sysmem;
     };
 
@@ -192,7 +194,7 @@ struct uvm_mem_struct
     size_t chunks_count;
 
     // Size of each physical chunk (vidmem) or CPU page (sysmem)
-    NvU32 chunk_size;
+    NvU64 chunk_size;
 
     // Size of the allocation
     NvU64 size;
@@ -203,10 +205,10 @@ struct uvm_mem_struct
     struct
     {
         // Mask of processors the memory is virtually mapped on
-        uvm_global_processor_mask_t mapped_on;
+        uvm_processor_mask_t mapped_on;
 
         // Page table ranges for all GPUs
-        uvm_page_table_range_vec_t *range_vecs[UVM_GLOBAL_ID_MAX_GPUS];
+        uvm_page_table_range_vec_t *range_vecs[UVM_ID_MAX_GPUS];
 
         // Range allocation for the GPU VA
         uvm_range_allocation_t range_alloc;
@@ -254,6 +256,8 @@ bool uvm_mem_is_physically_contiguous(uvm_mem_t *mem, NvU64 offset, NvU64 size);
 //
 // Unless a specific page size is needed, or the physical pages need to be
 // zeroed, the caller can use the appropriate uvm_mem_alloc* helper instead.
+//
+// If an error is returned, mem_out is guaranteed to not have been modified.
 NV_STATUS uvm_mem_alloc(const uvm_mem_alloc_params_t *params, uvm_mem_t **mem_out);
 
 // Clear all mappings and free the memory
@@ -322,8 +326,7 @@ uvm_gpu_phys_address_t uvm_mem_gpu_physical(uvm_mem_t *mem, uvm_gpu_t *gpu, NvU6
 uvm_gpu_address_t uvm_mem_gpu_address_physical(uvm_mem_t *mem, uvm_gpu_t *gpu, NvU64 offset, NvU64 size);
 
 // Helper to get an address suitable for accessing_gpu (which may be the backing
-// GPU) to access with CE. Note that mappings for indirect peers are not
-// created automatically.
+// GPU) to access with CE.
 uvm_gpu_address_t uvm_mem_gpu_address_copy(uvm_mem_t *mem, uvm_gpu_t *accessing_gpu, NvU64 offset, NvU64 size);
 
 static bool uvm_mem_is_sysmem(uvm_mem_t *mem)
@@ -434,6 +437,6 @@ static NV_STATUS uvm_mem_alloc_sysmem_dma_and_map_cpu_kernel(NvU64 size,
 }
 
 // Helper to map an allocation on the specified processors in the UVM VA space.
-NV_STATUS uvm_mem_map_kernel(uvm_mem_t *mem, const uvm_global_processor_mask_t *mask);
+NV_STATUS uvm_mem_map_kernel(uvm_mem_t *mem, const uvm_processor_mask_t *mask);
 
 #endif // __UVM_MEM_H__
