@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015-2025 NVidia Corporation
+    Copyright (c) 2015-2024 NVidia Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -1058,6 +1058,14 @@ typedef struct
     NV_STATUS                       rmStatus;                                           // Out
 } UVM_TEST_SET_PAGE_THRASHING_POLICY_PARAMS;
 
+#define UVM_TEST_PMM_SYSMEM                              UVM_TEST_IOCTL_BASE(64)
+typedef struct
+{
+    NvU64                           range_address1                   NV_ALIGN_BYTES(8); // In
+    NvU64                           range_address2                   NV_ALIGN_BYTES(8); // In
+    NV_STATUS                       rmStatus;                                           // Out
+} UVM_TEST_PMM_SYSMEM_PARAMS;
+
 #define UVM_TEST_PMM_REVERSE_MAP                         UVM_TEST_IOCTL_BASE(65)
 typedef struct
 {
@@ -1522,6 +1530,26 @@ typedef struct
     NV_STATUS       rmStatus;                           // Out
 } UVM_TEST_INJECT_NVLINK_ERROR_PARAMS;
 
+#define UVM_TEST_FILE_INITIALIZE                         UVM_TEST_IOCTL_BASE(107)
+typedef struct
+{
+    NV_STATUS               rmStatus;                    // Out
+} UVM_TEST_FILE_INITIALIZE_PARAMS;
+
+#define UVM_TEST_FILE_UNMAP                              UVM_TEST_IOCTL_BASE(108)
+typedef struct
+{
+    // File offset at which to start unmapping. Note that this will unmap file
+    // offsets, not virtual addresses, so all virtual addresses mapping these
+    // offsets will be unmapped.
+    NvU64 offset        NV_ALIGN_BYTES(8);  // In
+
+    // Bytes to unmap
+    NvU64 length        NV_ALIGN_BYTES(8);  // In
+
+    NV_STATUS rmStatus;                     // Out
+} UVM_TEST_FILE_UNMAP_PARAMS;
+
 #define UVM_TEST_QUERY_ACCESS_COUNTERS                   UVM_TEST_IOCTL_BASE(109)
 typedef struct
 {
