@@ -168,9 +168,12 @@ namespace DisplayPort
             bool bIgnoreDscCap;           // Ignore DSC even if sink reports DSC capability
             bool bDisableDownspread;
             bool bForceHeadShutdown;
+            bool bDP2XPreferNonDSCForLowPClk;
             bool bDisableDscMaxBppLimit;
+            bool bForceHeadShutdownOnModeTransition;
             bool bSkipCableIdCheck;
             bool bAllocateManualTimeslots;
+            bool bSkipResetMSTMBeforeLt;
         }_WARFlags;
 
         _WARFlags WARFlags;
