@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2008-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2008-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -152,6 +152,7 @@ typedef NvU8 FLCN_STATUS;
 #define FLCN_ERR_VPR_APP_DISP_FALCON_IS_NOT_IN_LS_MODE                     (0x61U)
 #define FLCN_ERR_VPR_APP_VPR_IS_ALREADY_ENABLED                            (0x62U)
 #define FLCN_ERR_VPR_APP_UNEXPECTEDLY_RUNNING_ON_RISCV                     (0x63U)
+#define FLCN_ERR_INVALID_SR_CONFIGURATION                                  (0x64U)
 
 // Clocks Errors
 #define FLCN_ERR_CYCLE_DETECTED                                            (0x70U)
@@ -160,6 +161,7 @@ typedef NvU8 FLCN_STATUS;
 #define FLCN_ERR_FREQ_NOT_SUPPORTED                                        (0x73U)
 #define FLCN_ERR_INVALID_SOURCE                                            (0x74U)
 #define FLCN_ERR_NOT_INITIALIZED                                           (0x75U)
+#define FLCN_ERR_CLOCK_NOT_SETTLED                                         (0x76U)
 
 // HDCP2.2 Errors
 #define FLCN_ERR_HDCP22_GETDKEY_FAILED                                     (0x80U)
@@ -182,6 +184,9 @@ typedef NvU8 FLCN_STATUS;
 #define FLCN_ERR_NVRNG_INIT_CRYPTO_DEVICE_FAILED                           (0x95U)
 #define FLCN_ERR_NVRNG_SELECT_ENGINE_FAILED                                (0x96U)
 #define FLCN_ERR_NVRNG_GENERATE_FAILED                                     (0x97U)
+#define FLCN_ERR_NVPKA_ECDSA_INIT_FAILED                                   (0x98U)
+#define FLCN_ERR_NVPKA_ECDSA_SIGN_FAILED                                   (0x99U)
+#define FLCN_ERR_NVPKA_GET_ECC_KEY_PAIR_FAILED                             (0x9AU)
 
 // Heavy Secure Errors
 #define FLCN_ERR_HS_CHK_INVALID_INPUT                                      (0xA0U)
@@ -235,7 +240,8 @@ typedef NvU8 FLCN_STATUS;
 #define FLCN_ERR_HS_APM_SCRATCH_INIT_INVALID                               (0xD0U)
 #define FLCN_ERR_HS_INVALID_KEY                                            (0xD1U)
 #define FLCN_ERR_HS_SWKG_INVALID_SIGNATURE                                 (0xD2U)
-
+#define FLCN_ERR_HS_SCP_LIB_DMHASH_FAILED                                  (0xD3U)
+#define FLCN_ERR_HS_SCP_LIB_AESKDF_FAILED                                  (0xD4U)
 
 //
 // BAR0/CSB Priv Read/Write Error Handling Defines
@@ -262,6 +268,7 @@ typedef NvU8 FLCN_STATUS;
 // Warnings.
 #define FLCN_WARN_NOTHING_TO_DO                                            (0xD0U)
 #define FLCN_WARN_NOT_QUERIED                                              (0xD1U)
+#define FLCN_WARN_DATA_NOT_AVAILABLE                                       (0xD2U)
 
 // Queue handling Errors
 #define FLCN_ERR_QUEUE_MGMT_INVALID_UNIT_ID                                (0xE0U)
@@ -269,6 +276,7 @@ typedef NvU8 FLCN_STATUS;
 #define FLCN_ERR_QUEUE_TASK_INVALID_EVENT_TYPE                             (0xE2U)
 #define FLCN_ERR_QUEUE_TASK_INVALID_UNIT_ID                                (0xE3U)
 #define FLCN_ERR_QUEUE_TASK_INVALID_CMD_TYPE                               (0xE4U)
+#define FLCN_ERR_QUEUE_TASK_SLOT_NOT_AVAILABLE                             (0xE5U)
 
 // Posted write errors
 #define FLCN_ERR_POSTED_WRITE_FAILURE                                      (0xF0U)
@@ -289,4 +297,7 @@ typedef NvU8 FLCN_STATUS;
 // Arithmetic errors
 #define FLCN_ERR_ARITHMETIC_OVERFLOW                                       (0xFAU)
 
+// Pri errros
+#define FLCN_ERR_OUTSTANDING_PRI_ERROR                                     (0xFBU)
+
 #endif // FLCNRETVAL_H
