@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2021-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2021-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -25,11 +25,10 @@
 #include "kernel/gpu/fifo/kernel_channel.h"
 #include "kernel/gpu/fifo/kernel_channel_group.h"
 #include "kernel/gpu/fifo/kernel_channel_group_api.h"
-#include "kernel/gpu/fifo/kernel_sched_mgr.h"
 #include "gpu/mem_mgr/mem_mgr.h"
 #include "gpu/mmu/kern_gmmu.h"
 
-#include "nvRmReg.h"
+#include "nvrm_registry.h"
 
 #include "vgpu/rpc.h"
 #include "gpu/bus/kern_bus.h"
@@ -50,7 +49,6 @@ kfifoConstructHal_GM107
     KernelFifo *pKernelFifo
 )
 {
-    NV_STATUS status;
     PREALLOCATED_USERD_INFO *pUserdInfo = &pKernelFifo->userdInfo;
 
     if (FLD_TEST_DRF(_REG_STR_RM, _INST_VPR, _INSTBLK, _TRUE, pGpu->instVprOverrides))
@@ -63,11 +61,7 @@ kfifoConstructHal_GM107
     {
         default:
         case NV_REG_STR_RM_INST_LOC_INSTBLK_DEFAULT:
-            if (kfifoIsMixedInstmemApertureDefAllowed(pKernelFifo))
-                pKernelFifo->pInstAllocList  = ADDRLIST_FBMEM_PREFERRED;
-            else
-                pKernelFifo->pInstAllocList  = ADDRLIST_FBMEM_ONLY;
-
+            pKernelFifo->pInstAllocList  = ADDRLIST_FBMEM_PREFERRED;
             pKernelFifo->InstAttr        = NV_MEMORY_UNCACHED;
             break;
         case NV_REG_STR_RM_INST_LOC_INSTBLK_VID:
@@ -92,19 +86,6 @@ kfifoConstructHal_GM107
                            &pUserdInfo->userdAperture,
                            &pUserdInfo->userdAttr);
 
-    // Create child object KernelSchedMgr
-    if (kfifoIsSchedSupported(pKernelFifo))
-    {
-        pKernelFifo->pKernelSchedMgr = NULL;
-        status = objCreate(&pKernelFifo->pKernelSchedMgr, pKernelFifo, KernelSchedMgr);
-        if (status != NV_OK)
-        {
-            pKernelFifo->pKernelSchedMgr = NULL;
-            return status;
-        }
-        kschedmgrConstructPolicy(pKernelFifo->pKernelSchedMgr, pGpu);
-    }
-
     return NV_OK;
 }
 
@@ -146,7 +127,8 @@ _kfifoAllocDummyPage
         return status;
     }
 
-    status = memdescAllocList(pKernelFifo->pDummyPageMemDesc, pKernelFifo->pInstAllocList);
+    memdescTagAllocList(status, NV_FB_ALLOC_RM_INTERNAL_OWNER_UNNAMED_TAG_80, 
+                    pKernelFifo->pDummyPageMemDesc, pKernelFifo->pInstAllocList);
     if (status !=  NV_OK)
     {
         NV_PRINTF(LEVEL_ERROR, "Could not allocate dummy page\n");
@@ -636,7 +618,7 @@ kfifoConvertInstToKernelChannel_GM107
 
     memdescDescribe(&instMemDesc, instAperture, pInst->address, NV_RAMIN_ALLOC_SIZE);
 
-    kfifoGetChannelIterator(pGpu, pKernelFifo, &chanIt);
+    kfifoGetChannelIterator(pGpu, pKernelFifo, &chanIt, INVALID_RUNLIST_ID);
     while (kfifoGetNextKernelChannel(pGpu, pKernelFifo, &chanIt, &pKernelChannel) == NV_OK)
     {
         NV_ASSERT_OR_ELSE(pKernelChannel != NULL, continue);
@@ -1244,7 +1226,8 @@ kfifoPreAllocUserD_GM107
     }
     else
     {
-        status = memdescAlloc(pUserdInfo->userdPhysDesc[currentGpuInst]);
+        memdescTagAlloc(status, NV_FB_ALLOC_RM_INTERNAL_OWNER_UNNAMED_TAG_81, 
+                        pUserdInfo->userdPhysDesc[currentGpuInst]);
         if (status != NV_OK)
         {
             NV_PRINTF(LEVEL_ERROR,
@@ -1267,7 +1250,7 @@ kfifoPreAllocUserD_GM107
             (memdescGetAddressSpace(pUserdInfo->userdPhysDesc[currentGpuInst]) == ADDR_FBMEM))
         {
 
-            NV_PRINTF(LEVEL_INFO, "Mapping USERD with coherent link.\n");
+            NV_PRINTF(LEVEL_INFO, "Mapping USERD with coherent link (USERD in FBMEM).\n");
             NV_ASSERT(pGpu->getProperty(pGpu, PDB_PROP_GPU_ATS_SUPPORTED));
             NV_ASSERT(pUserdInfo->userdPhysDesc[currentGpuInst]->_flags & MEMDESC_FLAGS_PHYSICALLY_CONTIGUOUS);
 
@@ -1277,6 +1260,22 @@ kfifoPreAllocUserD_GM107
                                                        pUserdInfo->userdPhysDesc[currentGpuInst]->PteAdjust;
             }
         }
+        //
+        // get sysmem mapping for USERD if USERD is in sysmem and reflected BAR access is not allowed
+        //
+        else if ((bCoherentCpuMapping &&
+                 memdescGetAddressSpace(pUserdInfo->userdPhysDesc[currentGpuInst]) == ADDR_SYSMEM &&
+                 !kbusIsReflectedMappingAccessAllowed(pKernelBus)) ||
+                 kbusIsBar1Disabled(pKernelBus))
+        {
+            NV_PRINTF(LEVEL_INFO, "Mapping USERD with coherent link (USERD in SYSMEM).\n");
+
+            if (bFifoFirstInit)
+            {
+                pUserdInfo->userdBar1MapStartOffset =
+                        memdescGetPhysAddr(pUserdInfo->userdPhysDesc[currentGpuInst], AT_CPU, 0);
+            }
+        }
         else
         {
             // vGpu may boot with partitioning enabled but that's not true for host RM
@@ -1288,20 +1287,20 @@ kfifoPreAllocUserD_GM107
                 goto fail;
             }
             // Now BAR1 map it
-            status = kbusMapFbAperture_HAL(pGpu, pKernelBus, pUserdInfo->userdPhysDesc[currentGpuInst], 0,
-                                           &pUserdInfo->userdBar1MapStartOffset,
-                                           &temp, mapFlags | BUS_MAP_FB_FLAGS_PRE_INIT, NV01_NULL_OBJECT);
-        }
+            status = kbusMapFbApertureSingle(pGpu, pKernelBus, pUserdInfo->userdPhysDesc[currentGpuInst], 0,
+                                             &pUserdInfo->userdBar1MapStartOffset,
+                                             &temp, mapFlags | BUS_MAP_FB_FLAGS_PRE_INIT, NULL);
 
-        if (status != NV_OK)
-        {
-            NV_PRINTF(LEVEL_ERROR, "Could not map USERD to BAR1\n");
-            DBG_BREAKPOINT();
-            goto fail;
-        }
+            if (status != NV_OK)
+            {
+                NV_PRINTF(LEVEL_ERROR, "Could not map USERD to BAR1\n");
+                DBG_BREAKPOINT();
+                goto fail;
+            }
 
-        // Add current GPU to list of GPUs referencing pFifo userD bar1
-        pUserdInfo->userdBar1RefMask |= NVBIT(pGpu->gpuInstance);
+            // Add current GPU to list of GPUs referencing pFifo userD bar1
+            pUserdInfo->userdBar1RefMask |= NVBIT(pGpu->gpuInstance);
+        }
     }
 
     if (bFifoFirstInit)
@@ -1311,9 +1310,35 @@ kfifoPreAllocUserD_GM107
         if (bCoherentCpuMapping &&
             (memdescGetAddressSpace(pUserdInfo->userdPhysDesc[currentGpuInst]) == ADDR_FBMEM))
         {
-            pUserdInfo->userdBar1CpuPtr = kbusMapCoherentCpuMapping_HAL(pGpu, pKernelBus,
-                                             pUserdInfo->userdPhysDesc[currentGpuInst]);
-            status = pUserdInfo->userdBar1CpuPtr == NULL ? NV_ERR_GENERIC : NV_OK;
+            status = kbusMapCoherentCpuMapping_HAL(pGpu, pKernelBus,
+                                                   pUserdInfo->userdPhysDesc[currentGpuInst],
+                                                   0,
+                                                   pUserdInfo->userdBar1MapSize,
+                                                   NV_PROTECT_READ_WRITE,
+                                                   (void**)&pUserdInfo->userdBar1CpuPtr,
+                                                   (void**)&pUserdInfo->userdBar1Priv);
+        }
+        else if ((bCoherentCpuMapping &&
+                 memdescGetAddressSpace(pUserdInfo->userdPhysDesc[currentGpuInst]) == ADDR_SYSMEM &&
+                 !kbusIsReflectedMappingAccessAllowed(pKernelBus)) &&
+                 !kbusIsBar1Disabled(pKernelBus))
+        {
+            status = osMapPciMemoryKernelOld(pGpu,
+                                             pUserdInfo->userdBar1MapStartOffset,
+                                             pUserdInfo->userdBar1MapSize,
+                                             NV_PROTECT_READ_WRITE,
+                                             (void**)&pUserdInfo->userdBar1CpuPtr,
+                                             NV_MEMORY_CACHED);
+        }
+        else if (kbusIsBar1Disabled(pKernelBus))
+        {
+            status = memdescMap(pUserdInfo->userdPhysDesc[currentGpuInst],
+                                0,
+                                pUserdInfo->userdBar1MapSize,
+                                NV_TRUE,
+                                NV_PROTECT_READ_WRITE,
+                                (void**)&pUserdInfo->userdBar1CpuPtr,
+                                (void**)&pUserdInfo->userdBar1Priv);
         }
         else
         {
@@ -1393,7 +1418,17 @@ kfifoFreePreAllocUserD_GM107
         if (bCoherentCpuMapping)
         {
             kbusUnmapCoherentCpuMapping_HAL(pGpu, pKernelBus,
-                pUserdInfo->userdPhysDesc[currentGpuInst]);
+                pUserdInfo->userdPhysDesc[currentGpuInst],
+                pUserdInfo->userdBar1CpuPtr,
+                pUserdInfo->userdBar1Priv);
+        }
+        else if (kbusIsBar1Disabled(pKernelBus))
+        {
+            memdescUnmap(pUserdInfo->userdPhysDesc[currentGpuInst],
+                            NV_TRUE,
+                            osGetCurrentProcess(),
+                            (void*)pUserdInfo->userdBar1CpuPtr,
+                            (void*)pUserdInfo->userdBar1Priv);
         }
         else
         {
@@ -1413,11 +1448,11 @@ kfifoFreePreAllocUserD_GM107
                 // Unmap in UC for each GPU with a pKernelFifo userd
                 // reference mapped through bar1
                 //
-                kbusUnmapFbAperture_HAL(pGpu, pKernelBus,
-                                        pUserdInfo->userdPhysDesc[currentGpuInst],
-                                        pUserdInfo->userdBar1MapStartOffset,
-                                        pUserdInfo->userdBar1MapSize,
-                                        BUS_MAP_FB_FLAGS_MAP_UNICAST | BUS_MAP_FB_FLAGS_PRE_INIT);
+                kbusUnmapFbApertureSingle(pGpu, pKernelBus,
+                                          pUserdInfo->userdPhysDesc[currentGpuInst],
+                                          pUserdInfo->userdBar1MapStartOffset,
+                                          pUserdInfo->userdBar1MapSize,
+                                          BUS_MAP_FB_FLAGS_MAP_UNICAST | BUS_MAP_FB_FLAGS_PRE_INIT);
                 pUserdInfo->userdBar1RefMask &= (~NVBIT(pGpu->gpuInstance));
             }
 
