@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2018-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2018-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -42,14 +42,18 @@
 #include "kernel/gpu/bus/kern_bus.h"
 #include "kernel/gpu/mem_mgr/mem_mgr.h"
 #include "kernel/gpu/fifo/kernel_channel.h"
+#include "kernel/gpu/subdevice/subdevice.h"
+#include "kernel/gpu/timer/objtmr.h"
 #include "kernel/virtualization/hypervisor/hypervisor.h"
 #include "rmapi/client.h"
+#include "vgpu/sdk-structures.h"
 
 #include "class/cl90cdtypes.h"
 #include "ctrl/ctrl90cd.h"
 
 #define NV_FECS_TRACE_MAX_TIMESTAMPS 5
 #define NV_FECS_TRACE_MAGIC_INVALIDATED 0xdededede         // magic number for entries that have been read
+#define NV_FECS_TRACE_CALLBACK_TIME_NS 33333333            // Approximating 30Hz callback
 
 typedef struct
 {
@@ -93,6 +97,25 @@ struct KGRAPHICS_FECS_TRACE_INFO
     VGPU_FECS_TRACE_STAGING_BUFFER *pVgpuStaging;
 };
 
+/*! Private FECS event buffer data stored for the GPU as a whole */
+struct KGRMGR_FECS_GLOBAL_TRACE_INFO
+{
+    // map: { UserInfo* -> { pEventBuffer -> NV_EVENT_BUFFER_BIND_POINT* }}
+    FecsEventBufferBindMultiMap fecsEventBufferBindingsUid;
+
+    // Timer event to periodically the processing callback
+    TMR_EVENT *pFecsTimerEvent;
+
+    // Timer interval in nanoseconds
+    NvU32 fecsTimerInterval;
+
+    // Atomic for scheduling the fecs callback in timer mode
+    NvU32 fecsCallbackScheduled;
+
+    // Number of consumer clients
+    NvS16 fecsCtxswLogConsumerCount;
+};
+
 /*!
  * @brief      Function to populate client/subdevice/grRouteInfo from cached
  *             information in order to make calls into the specific MIG GPU instance
@@ -179,19 +202,21 @@ formatAndNotifyFecsRecord
     FECS_EVENT_RECORD  *pRecord
 )
 {
-    FECS_EVENT_NOTIFICATION_DATA notifRecord;
-    KernelFifo                  *pKernelFifo       = GPU_GET_KERNEL_FIFO(pGpu);
-    KernelChannel               *pKernelChannel    = NULL;
-    KernelChannel               *pKernelChannelNew = NULL;
-    MIG_INSTANCE_REF            *pChannelRef;
-    MIG_INSTANCE_REF            *pNewChannelRef;
-    INST_BLOCK_DESC              inst;
-    NvU32                        timestampId;
-    NvU64                        noisyTimestampStart = 0;
-    NvU64                        noisyTimestampRange = 0;
-    NvU32                        instSize;
-    NvU32                        instShift;
-    NV_STATUS                    status;
+    FECS_EVENT_NOTIFICATION_DATA   notifRecord;
+    KernelFifo                    *pKernelFifo       = GPU_GET_KERNEL_FIFO(pGpu);
+    KernelChannel                 *pKernelChannel    = NULL;
+    KernelChannel                 *pKernelChannelNew = NULL;
+    KernelGraphicsManager         *pKernelGraphicsManager = GPU_GET_KERNEL_GRAPHICS_MANAGER(pGpu);
+    KGRMGR_FECS_GLOBAL_TRACE_INFO *pFecsGlobalTraceInfo = kgrmgrGetFecsGlobalTraceInfo(pGpu, pKernelGraphicsManager);
+    MIG_INSTANCE_REF              *pChannelRef;
+    MIG_INSTANCE_REF              *pNewChannelRef;
+    INST_BLOCK_DESC                inst;
+    NvU32                          timestampId;
+    NvU64                          noisyTimestampStart = 0;
+    NvU64                          noisyTimestampRange = 0;
+    NvU32                          instSize;
+    NvU32                          instShift;
+    NV_STATUS                      status;
 
     if (pRecord == NULL)
     {
@@ -200,6 +225,8 @@ formatAndNotifyFecsRecord
         return;
     }
 
+    NV_ASSERT_OR_RETURN_VOID(pFecsGlobalTraceInfo != NULL);
+
     kfifoGetInstBlkSizeAlign_HAL(pKernelFifo, &instSize, &instShift);
 
     portMemSet(&notifRecord, 0, sizeof(notifRecord));
@@ -353,12 +380,12 @@ formatAndNotifyFecsRecord
             if (notifRecord.userInfo != 0)
             {
                 // Notify event buffers listening for the current UID
-                pSubmap = multimapFindSubmap(&pGpu->fecsEventBufferBindingsUid, notifRecord.userInfo);
+                pSubmap = multimapFindSubmap(&pFecsGlobalTraceInfo->fecsEventBufferBindingsUid, notifRecord.userInfo);
                 notifyEventBuffers(pGpu, pSubmap, &notifRecord);
             }
 
             // Notify event buffers listening for all UIDs
-            pSubmap = multimapFindSubmap(&pGpu->fecsEventBufferBindingsUid, 0);
+            pSubmap = multimapFindSubmap(&pFecsGlobalTraceInfo->fecsEventBufferBindingsUid, 0);
             notifyEventBuffers(pGpu, pSubmap, &notifRecord);
 
             // Clear so we don't report drops for every event in this record
@@ -441,11 +468,15 @@ notifyEventBuffers
     FECS_EVENT_NOTIFICATION_DATA const *pRecord
 )
 {
+    KernelGraphicsManager *pKernelGraphicsManager = GPU_GET_KERNEL_GRAPHICS_MANAGER(pGpu);
+    KGRMGR_FECS_GLOBAL_TRACE_INFO *pFecsGlobalTraceInfo = kgrmgrGetFecsGlobalTraceInfo(pGpu, pKernelGraphicsManager);
     NvBool bMIGInUse = IS_MIG_IN_USE(pGpu);
 
+    NV_ASSERT_OR_RETURN_VOID(pFecsGlobalTraceInfo != NULL);
+
     if (pSubmap != NULL)
     {
-        FecsEventBufferBindMultiMapIter iter = multimapSubmapIterItems(&pGpu->fecsEventBufferBindingsUid, pSubmap);
+        FecsEventBufferBindMultiMapIter iter = multimapSubmapIterItems(&pFecsGlobalTraceInfo->fecsEventBufferBindingsUid, pSubmap);
 
         while (multimapItemIterNext(&iter))
         {
@@ -634,11 +665,10 @@ fecsCtxswLoggingInit
         return NV_ERR_NO_MEMORY;
     portMemSet(pFecsTraceInfo, 0, sizeof(*pFecsTraceInfo));
 
-    *ppFecsTraceInfo = pFecsTraceInfo;
-
-    osGetCurrentTick(&seed);
+    seed = osGetCurrentTick();
     pFecsTraceInfo->pFecsLogPrng = portCryptoPseudoRandomGeneratorCreate(seed);
-    multimapInit(&pGpu->fecsEventBufferBindingsUid, portMemAllocatorGetGlobalNonPaged());
+
+    *ppFecsTraceInfo = pFecsTraceInfo;
 
     kgraphicsInitFecsRegistryOverrides_HAL(pGpu, pKernelGraphics);
 
@@ -656,13 +686,50 @@ fecsCtxswLoggingTeardown
 
     NV_ASSERT_OR_RETURN_VOID(pFecsTraceInfo != NULL);
 
-    multimapDestroy(&pGpu->fecsEventBufferBindingsUid);
-
     portCryptoPseudoRandomGeneratorDestroy(pFecsTraceInfo->pFecsLogPrng);
     pFecsTraceInfo->pFecsLogPrng = NULL;
     portMemFree(pFecsTraceInfo);
 }
 
+NV_STATUS
+fecsGlobalLoggingInit
+(
+    OBJGPU *pGpu,
+    KernelGraphicsManager *pKernelGraphicsManager,
+    KGRMGR_FECS_GLOBAL_TRACE_INFO **ppFecsGlobalTraceInfo
+)
+{
+    KGRMGR_FECS_GLOBAL_TRACE_INFO *pFecsGlobalTraceInfo;
+
+    pFecsGlobalTraceInfo = portMemAllocNonPaged(sizeof(*pFecsGlobalTraceInfo));
+    NV_ASSERT_OR_RETURN(pFecsGlobalTraceInfo != NULL, NV_ERR_NO_MEMORY);
+    portMemSet(pFecsGlobalTraceInfo, 0, sizeof(*pFecsGlobalTraceInfo));
+
+    multimapInit(&pFecsGlobalTraceInfo->fecsEventBufferBindingsUid, portMemAllocatorGetGlobalNonPaged());
+
+    *ppFecsGlobalTraceInfo = pFecsGlobalTraceInfo;
+
+    pFecsGlobalTraceInfo->fecsTimerInterval = NV_FECS_TRACE_CALLBACK_TIME_NS;
+
+    return NV_OK;
+}
+
+void
+fecsGlobalLoggingTeardown
+(
+    OBJGPU *pGpu,
+    KernelGraphicsManager *pKernelGraphicsManager
+)
+{
+    KGRMGR_FECS_GLOBAL_TRACE_INFO *pFecsGlobalTraceInfo = kgrmgrGetFecsGlobalTraceInfo(pGpu, pKernelGraphicsManager);
+
+    NV_ASSERT_OR_RETURN_VOID(pFecsGlobalTraceInfo != NULL);
+
+    multimapDestroy(&pFecsGlobalTraceInfo->fecsEventBufferBindingsUid);
+
+    portMemFree(pFecsGlobalTraceInfo);
+}
+
 /*! set num records to process per intr */
 void fecsSetRecordsPerIntr
 (
@@ -709,6 +776,176 @@ fecsBufferChanged
     return NV_FALSE;
 }
 
+static void
+_fecsClearCallbackScheduled
+(
+    KGRMGR_FECS_GLOBAL_TRACE_INFO *pFecsGlobalTraceInfo
+)
+{
+    NV_ASSERT_OR_RETURN_VOID(pFecsGlobalTraceInfo != NULL);
+    portAtomicSetU32(&pFecsGlobalTraceInfo->fecsCallbackScheduled, 0);
+}
+
+/*!
+ * @brief Atomically set fecs callback scheduled, return NV_TRUE if wasn't scheduled
+ */
+static NvBool
+_fecsSignalCallbackScheduled
+(
+    KGRMGR_FECS_GLOBAL_TRACE_INFO *pFecsGlobalTraceInfo
+)
+{
+    NV_ASSERT_OR_RETURN(pFecsGlobalTraceInfo != NULL, 0);
+    return portAtomicCompareAndSwapU32(&pFecsGlobalTraceInfo->fecsCallbackScheduled, 1, 0);
+}
+
+static void
+_fecsOsWorkItem
+(
+    NvU32 gpuInstance,
+    void *data
+)
+{
+    OBJGPU *pGpu = gpumgrGetGpu(gpuInstance);
+    KernelGraphicsManager *pKernelGraphicsManager;
+    KGRMGR_FECS_GLOBAL_TRACE_INFO *pFecsGlobalTraceInfo;
+
+    NV_CHECK_OR_RETURN_VOID(LEVEL_ERROR, pGpu != NULL);
+    pKernelGraphicsManager = GPU_GET_KERNEL_GRAPHICS_MANAGER(pGpu);
+    pFecsGlobalTraceInfo = kgrmgrGetFecsGlobalTraceInfo(pGpu, pKernelGraphicsManager);
+
+    NV_ASSERT_OR_RETURN_VOID(pFecsGlobalTraceInfo != NULL);
+
+    nvEventBufferFecsCallback(pGpu, NULL);
+    _fecsClearCallbackScheduled(pFecsGlobalTraceInfo);
+}
+
+static NV_STATUS
+_fecsTimerCallback
+(
+    OBJGPU *pGpu,
+    OBJTMR *pTmr,
+    TMR_EVENT *pTmrEvent
+)
+{
+    NV_STATUS status = NV_OK;
+    KernelGraphicsManager *pKernelGraphicsManager = GPU_GET_KERNEL_GRAPHICS_MANAGER(pGpu);
+    KGRMGR_FECS_GLOBAL_TRACE_INFO *pFecsGlobalTraceInfo = kgrmgrGetFecsGlobalTraceInfo(pGpu, pKernelGraphicsManager);
+    NvU32 i;
+    NvU32 numIter = GPU_MAX_GRS;
+
+    if (!IS_MIG_IN_USE(pGpu))
+        numIter = 1;
+
+    NV_ASSERT_OR_RETURN(pFecsGlobalTraceInfo != NULL, NV_ERR_INVALID_STATE);
+
+    // If any Kgraphics have events, schedule work item
+    for (i = 0; i < numIter; i++)
+    {
+        KernelGraphics *pKernelGraphics = GPU_GET_KERNEL_GRAPHICS(pGpu, i);
+
+        if (pKernelGraphics == NULL)
+            continue;
+
+        if (fecsBufferChanged(pGpu, pKernelGraphics) && _fecsSignalCallbackScheduled(pFecsGlobalTraceInfo))
+        {
+            NV_CHECK_OK(status, LEVEL_ERROR, osQueueWorkItemWithFlags(pGpu, _fecsOsWorkItem, NULL, OS_QUEUE_WORKITEM_FLAGS_LOCK_GPU_GROUP_DEVICE));
+
+            if (status != NV_OK)
+                _fecsClearCallbackScheduled(pFecsGlobalTraceInfo);
+
+            break;
+        }
+    }
+
+    // TMR_FLAG_RECUR does not work, so reschedule it here.
+    NV_CHECK_OK_OR_CAPTURE_FIRST_ERROR(status,
+                                       LEVEL_ERROR,
+                                       tmrEventScheduleRel(pTmr, pTmrEvent, pFecsGlobalTraceInfo->fecsTimerInterval));
+
+    return status;
+}
+
+static NV_STATUS
+_fecsTimerCreate
+(
+    OBJGPU *pGpu
+)
+{
+    OBJTMR *pTmr = GPU_GET_TIMER(pGpu);
+    KernelGraphicsManager *pKernelGraphicsManager = GPU_GET_KERNEL_GRAPHICS_MANAGER(pGpu);
+    KGRMGR_FECS_GLOBAL_TRACE_INFO *pFecsGlobalTraceInfo = kgrmgrGetFecsGlobalTraceInfo(pGpu, pKernelGraphicsManager);
+    NvU32 timerFlags = TMR_FLAG_RECUR;
+    // Unix needs to use the OS timer to avoid corrupting records, but Windows doesn't have an OS timer implementation
+    timerFlags |= TMR_FLAG_USE_OS_TIMER;
+
+    NV_ASSERT_OR_RETURN(pFecsGlobalTraceInfo != NULL, NV_ERR_INVALID_STATE);
+
+    NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
+        tmrEventCreate(pTmr, &pFecsGlobalTraceInfo->pFecsTimerEvent, _fecsTimerCallback, NULL, timerFlags));
+
+    // This won't be a true 30Hz timer as the callbacks are scheduled from the time they're called
+    NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
+        tmrEventScheduleRel(pTmr, pFecsGlobalTraceInfo->pFecsTimerEvent, pFecsGlobalTraceInfo->fecsTimerInterval));
+
+    return NV_OK;
+}
+
+static void
+_fecsTimerDestroy
+(
+    OBJGPU *pGpu
+)
+{
+    KernelGraphicsManager *pKernelGraphicsManager = GPU_GET_KERNEL_GRAPHICS_MANAGER(pGpu);
+    KGRMGR_FECS_GLOBAL_TRACE_INFO *pFecsGlobalTraceInfo = kgrmgrGetFecsGlobalTraceInfo(pGpu, pKernelGraphicsManager);
+
+    NV_ASSERT_OR_RETURN_VOID(pFecsGlobalTraceInfo != NULL);
+
+    if (pFecsGlobalTraceInfo->pFecsTimerEvent != NULL)
+    {
+        OBJTMR *pTmr = GPU_GET_TIMER(pGpu);
+
+        tmrEventDestroy(pTmr, pFecsGlobalTraceInfo->pFecsTimerEvent);
+        pFecsGlobalTraceInfo->pFecsTimerEvent = NULL;
+    }
+}
+
+NV_STATUS
+fecsHandleFecsLoggingError
+(
+    OBJGPU *pGpu,
+    NvU32 grIdx,
+    FECS_ERROR_EVENT_TYPE errorType
+)
+{
+    KernelGraphics *pKernelGraphics = GPU_GET_KERNEL_GRAPHICS(pGpu, grIdx);
+    NV_STATUS status = NV_OK;
+
+    switch (errorType)
+    {
+        case FECS_ERROR_EVENT_TYPE_BUFFER_RESET_REQUIRED:
+        {
+            fecsBufferDisableHw(pGpu, pKernelGraphics);
+            kgraphicsSetCtxswLoggingEnabled(pGpu, pKernelGraphics, NV_FALSE);
+            fecsBufferReset(pGpu, pKernelGraphics);
+            break;
+        }
+        case FECS_ERROR_EVENT_TYPE_BUFFER_FULL:
+        {
+            nvEventBufferFecsCallback(pGpu, pKernelGraphics);
+            break;
+        }
+        default:
+        {
+            status = NV_ERR_INVALID_ARGUMENT;
+            break;
+        }
+    }
+
+    return status;
+}
+
 /**
  * @brief The callback function that transfers FECS Buffer entries to an EventBuffer
  */
@@ -719,26 +956,32 @@ nvEventBufferFecsCallback
     void    *pArgs
 )
 {
-    KernelGraphics     *pKernelGraphics = (KernelGraphics*)pArgs;
-    NvU32               fecsReadOffset;
-    NvU32               fecsReadOffsetPrev;
-    NvU64               fecsBufferSize;
-    NvU32               fecsRecordSize;
-    NvU32               i, j;
-    NvU8               *pFecsBufferMapping;
-    MEMORY_DESCRIPTOR  *pFecsMemDesc = NULL;
-    FECS_EVENT_RECORD  *pPeekRecord;
-    NvU16               maxFecsRecordsPerIntr;
-    NV_STATUS           status;
-    RM_API             *pRmApi = GPU_GET_PHYSICAL_RMAPI(pGpu);
-    NvU8                numIterations = (pArgs == NULL)
-                                        ? KGRMGR_MAX_GR
-                                        : 1;
+    KernelGraphics          *pKernelGraphics = (KernelGraphics*)pArgs;
+    KernelGraphicsManager   *pKernelGraphicsManager = GPU_GET_KERNEL_GRAPHICS_MANAGER(pGpu);
+    KGRMGR_FECS_GLOBAL_TRACE_INFO *pFecsGlobalTraceInfo = kgrmgrGetFecsGlobalTraceInfo(pGpu, pKernelGraphicsManager);
+    NvU32                    fecsReadOffset;
+    NvU32                    fecsReadOffsetPrev;
+    NvU64                    fecsBufferSize;
+    NvU32                    fecsRecordSize;
+    NvU32                    i, j;
+    NvU8                    *pFecsBufferMapping;
+    MEMORY_DESCRIPTOR       *pFecsMemDesc = NULL;
+    FECS_EVENT_RECORD       *pPeekRecord;
+    NvU16                    maxFecsRecordsPerIntr;
+    NV_STATUS                status;
+    RM_API                  *pRmApi = GPU_GET_PHYSICAL_RMAPI(pGpu);
+    NvU8                     numIterations = (pArgs == NULL)
+                                             ? KGRMGR_MAX_GR
+                                             : 1;
+
+    if (!IS_MIG_IN_USE(pGpu))
+        numIterations = 1;
 
     NV_ASSERT_OR_RETURN_VOID(rmDeviceGpuLockIsOwner(pGpu->gpuInstance));
 
-    NV_ASSERT_OR_RETURN_VOID(pGpu->fecsCtxswLogConsumerCount >= 0);
-    if (pGpu->fecsCtxswLogConsumerCount <= 0)
+    NV_ASSERT_OR_RETURN_VOID(pFecsGlobalTraceInfo != NULL);
+    NV_ASSERT_OR_RETURN_VOID(pFecsGlobalTraceInfo->fecsCtxswLogConsumerCount >= 0);
+    if (pFecsGlobalTraceInfo->fecsCtxswLogConsumerCount <= 0)
         return;
 
     for (j = 0; j < numIterations; j++)
@@ -843,29 +1086,8 @@ nvEventBufferFecsCallback
 
         if (pFecsTraceInfo->fecsTraceCounter > 0)
         {
-            NvHandle hClient;
-            NvHandle hSubdevice;
-            NV2080_CTRL_INTERNAL_GR_SET_FECS_TRACE_RD_OFFSET_PARAMS params;
+            kgraphicsSetFecsTraceRdOffset_HAL(pGpu, pKernelGraphics, fecsReadOffset);
 
-            params.offset = fecsReadOffset;
-            NV_ASSERT_OK_OR_ELSE(
-                status,
-                _fecsLoadInternalRoutingInfo(pGpu,
-                                             pKernelGraphics,
-                                             &hClient,
-                                             &hSubdevice,
-                                             &params.grRouteInfo),
-                return);
-
-            NV_ASSERT_OK_OR_ELSE(
-                status,
-                pRmApi->Control(pRmApi,
-                                hClient,
-                                hSubdevice,
-                                NV2080_CTRL_CMD_INTERNAL_GR_SET_FECS_TRACE_RD_OFFSET,
-                                &params,
-                                sizeof(params)),
-                return);
             pFecsTraceInfo->fecsTraceCounter = 0;
         }
         pFecsTraceInfo->fecsTraceRdOffset = fecsReadOffset;
@@ -880,9 +1102,9 @@ NV_STATUS
 fecsAddBindpoint
 (
     OBJGPU *pGpu,
-    RsClient *pClient,
+    RmClient *pClient,
     RsResourceRef *pEventBufferRef,
-    NvHandle hNotifier,
+    Subdevice *pNotifier,
     NvBool bAllUsers,
     NV2080_CTRL_GR_FECS_BIND_EVTBUF_LOD levelOfDetail,
     NvU32 eventFilter,
@@ -891,16 +1113,17 @@ fecsAddBindpoint
 )
 {
     NV_STATUS status;
-    NvHandle hClient = pClient->hClient;
-    RmClient *pRmClient = dynamicCast(pClient, RmClient);
+    NvHandle hClient = staticCast(pClient, RsClient)->hClient;
     NvHandle hEventBuffer = pEventBufferRef->hResource;
+    NvHandle hNotifier = RES_GET_HANDLE(pNotifier);
     EventBuffer *pEventBuffer;
     NvBool bAdmin = osIsAdministrator();
     NvU32 eventMask = 0;
     NvU64 targetUser;
-    NvS32 gpuConsumerCount = pGpu->fecsCtxswLogConsumerCount;
-    NvBool bFecsBindingActive = (pGpu->fecsCtxswLogConsumerCount > 0);
-    NvBool bScheduled = NV_FALSE;
+    KernelGraphicsManager *pKernelGraphicsManager = GPU_GET_KERNEL_GRAPHICS_MANAGER(pGpu);
+    KGRMGR_FECS_GLOBAL_TRACE_INFO *pFecsGlobalTraceInfo = kgrmgrGetFecsGlobalTraceInfo(pGpu, pKernelGraphicsManager);
+    NvS32 gpuConsumerCount;
+    NvBool bFecsBindingActive;
     NvBool bIntrDriven = NV_FALSE;
     KernelMIGManager *pKernelMIGManager = GPU_GET_KERNEL_MIG_MANAGER(pGpu);
     NvBool bMIGInUse = IS_MIG_IN_USE(pGpu);
@@ -912,6 +1135,10 @@ fecsAddBindpoint
 
     CALL_CONTEXT *pCallContext = resservGetTlsCallContext();
     NV_ASSERT_OR_RETURN(pCallContext != NULL, NV_ERR_INVALID_STATE);
+    NV_ASSERT_OR_RETURN(pFecsGlobalTraceInfo != NULL, NV_ERR_INVALID_STATE);
+
+    gpuConsumerCount = pFecsGlobalTraceInfo->fecsCtxswLogConsumerCount;
+    bFecsBindingActive = (pFecsGlobalTraceInfo->fecsCtxswLogConsumerCount > 0);
 
     bKernel = pCallContext->secInfo.privLevel >= RS_PRIV_LEVEL_KERNEL;
 
@@ -921,12 +1148,26 @@ fecsAddBindpoint
     bSelectLOD = NV_TRUE;
 #endif
 
-    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmDeviceGpuLockIsOwner(pGpu->gpuInstance));
+    NV_ASSERT_OR_RETURN(rmapiLockIsOwner() && rmDeviceGpuLockIsOwner(pGpu->gpuInstance),
+        NV_ERR_INVALID_LOCK_STATE);
+
+    // Early bail-out if profiling capability is not enabled on vGPU
+    if (IS_VIRTUAL(pGpu))
+    {
+        VGPU_STATIC_INFO *pVSI = GPU_GET_STATIC_INFO(pGpu);
+        if ((pVSI == NULL) || !pVSI->vgpuStaticProperties.bProfilingTracingEnabled)
+        {
+            if (pReasonCode != NULL)
+                *pReasonCode = NV2080_CTRL_GR_FECS_BIND_REASON_CODE_NEED_CAPABILITY;
+
+            return NV_ERR_NOT_SUPPORTED;
+        }
+    }
 
     // On a hypervisor or VM: bail-out early if admin is required
     if (IS_VIRTUAL(pGpu) || hypervisorIsVgxHyper())
     {
-        if (pGpu->bRmProfilingPrivileged && !bAdmin)
+        if (pGpu->bRmProfilingPrivileged && !(bAdmin || osCheckAccess(RS_ACCESS_PERFMON)))
         {
             if (pReasonCode != NULL)
                 *pReasonCode = NV2080_CTRL_GR_FECS_BIND_REASON_CODE_NEED_ADMIN;
@@ -977,7 +1218,7 @@ fecsAddBindpoint
     }
     else
     {
-        targetUser = (NvU64)(NvUPtr)pRmClient->pUserInfo;
+        targetUser = (NvU64)(NvUPtr)pClient->pUserInfo;
 
         // Filtering UIDs is not yet implemented in legacy vGPU
         if (IS_VIRTUAL_WITHOUT_SRIOV(pGpu))
@@ -990,12 +1231,12 @@ fecsAddBindpoint
     }
 
     pEventBuffer = dynamicCast(pEventBufferRef->pResource, EventBuffer);
-    if (NULL == pEventBuffer)
+    if (pEventBuffer == NULL)
         return NV_ERR_INVALID_ARGUMENT;
 
-    if (NULL == multimapFindSubmap(&pGpu->fecsEventBufferBindingsUid, targetUser))
+    if (multimapFindSubmap(&pFecsGlobalTraceInfo->fecsEventBufferBindingsUid, targetUser) == NULL)
     {
-        if (NULL == multimapInsertSubmap(&pGpu->fecsEventBufferBindingsUid, targetUser))
+        if (multimapInsertSubmap(&pFecsGlobalTraceInfo->fecsEventBufferBindingsUid, targetUser) == NULL)
         {
             NV_PRINTF(LEVEL_ERROR, "failed to add UID binding!\n");
             return NV_ERR_INSUFFICIENT_RESOURCES;
@@ -1003,26 +1244,26 @@ fecsAddBindpoint
     }
 
     // If the binding exists already, we're done
-    if (NULL != multimapFindItem(&pGpu->fecsEventBufferBindingsUid, targetUser, (NvU64)(NvUPtr)pEventBuffer))
+    if (multimapFindItem(&pFecsGlobalTraceInfo->fecsEventBufferBindingsUid, targetUser, (NvU64)(NvUPtr)pEventBuffer) != NULL)
         return NV_OK;
 
-    pBind = multimapInsertItemNew(&pGpu->fecsEventBufferBindingsUid, targetUser, (NvU64)(NvUPtr)pEventBuffer);
+    pBind = multimapInsertItemNew(&pFecsGlobalTraceInfo->fecsEventBufferBindingsUid, targetUser, (NvU64)(NvUPtr)pEventBuffer);
     if (pBind == NULL)
         return NV_ERR_INVALID_ARGUMENT;
-    ++pGpu->fecsCtxswLogConsumerCount;
+    ++pFecsGlobalTraceInfo->fecsCtxswLogConsumerCount;
 
     pBind->hClient = hClient;
     pBind->hNotifier = hNotifier;
     pBind->hEventBuffer = hEventBuffer;
     pBind->pEventBuffer = pEventBuffer;
-    pBind->pUserInfo = (NvU64)(NvUPtr)pRmClient->pUserInfo;
+    pBind->pUserInfo = (NvU64)(NvUPtr)pClient->pUserInfo;
     pBind->bAdmin = bAdmin;
     pBind->eventMask = eventMask;
     pBind->bKernel = bKernel;
     pBind->version = version;
 
     status = registerEventNotification(&pEventBuffer->pListeners,
-                hClient,
+                staticCast(pClient, RsClient),
                 hNotifier,
                 hEventBuffer,
                 (version == 2 ?
@@ -1036,18 +1277,19 @@ fecsAddBindpoint
 
     if (bMIGInUse)
     {
-        if (kmigmgrIsClientUsingDeviceProfiling(pGpu, pKernelMIGManager, hClient))
+        if (kmigmgrIsDeviceUsingDeviceProfiling(pGpu, pKernelMIGManager, GPU_RES_GET_DEVICE(pNotifier)))
         {
             pBind->swizzId = NV2080_CTRL_GPU_PARTITION_ID_INVALID;
         }
         else
         {
             GPUInstanceSubscription *pGPUInstanceSubscription;
-            status = gisubscriptionGetGPUInstanceSubscription(pClient, hNotifier, &pGPUInstanceSubscription);
+            status = gisubscriptionGetGPUInstanceSubscription(
+                    staticCast(pClient, RsClient), hNotifier, &pGPUInstanceSubscription);
             if (status != NV_OK)
                 goto done;
 
-            if (pGPUInstanceSubscription->pKernelMIGGpuInstance == NULL)
+            if (gisubscriptionGetMIGGPUInstance(pGPUInstanceSubscription) == NULL)
             {
                 if (pReasonCode != NULL)
                     *pReasonCode = NV2080_CTRL_GR_FECS_BIND_REASON_CODE_NOT_ENABLED;
@@ -1056,7 +1298,7 @@ fecsAddBindpoint
                 goto done;
             }
 
-            pBind->swizzId = pGPUInstanceSubscription->pKernelMIGGpuInstance->swizzId;
+            pBind->swizzId = gisubscriptionGetMIGGPUInstance(pGPUInstanceSubscription)->swizzId;
         }
     }
 
@@ -1087,28 +1329,16 @@ fecsAddBindpoint
 
     if (!bFecsBindingActive && !bIntrDriven)
     {
-        status = osSchedule1SecondCallback(pGpu,
-                nvEventBufferFecsCallback,
-                NULL,
-                NV_OS_1HZ_REPEAT);
-
-        if (status != NV_OK)
-        {
-            status = NV_ERR_INSUFFICIENT_RESOURCES;
-            goto done;
-        }
-
-        bScheduled = NV_TRUE;
+        NV_CHECK_OK_OR_GOTO(status, LEVEL_ERROR, _fecsTimerCreate(pGpu), done);
     }
 
 done:
     if (status != NV_OK)
     {
-        if (gpuConsumerCount != pGpu->fecsCtxswLogConsumerCount)
+        if (gpuConsumerCount != pFecsGlobalTraceInfo->fecsCtxswLogConsumerCount)
             fecsRemoveBindpoint(pGpu, targetUser, pBind);
 
-        if (bScheduled)
-            osRemove1SecondRepeatingCallback(pGpu, nvEventBufferFecsCallback, NULL);
+        _fecsTimerDestroy(pGpu);
     }
 
     return status;
@@ -1123,8 +1353,12 @@ fecsRemoveBindpoint
 )
 {
     EventBuffer *pEventBuffer = pBind->pEventBuffer;
+    KernelGraphicsManager *pKernelGraphicsManager = GPU_GET_KERNEL_GRAPHICS_MANAGER(pGpu);
+    KGRMGR_FECS_GLOBAL_TRACE_INFO *pFecsGlobalTraceInfo = kgrmgrGetFecsGlobalTraceInfo(pGpu, pKernelGraphicsManager);
 
-    --pGpu->fecsCtxswLogConsumerCount;
+    NV_ASSERT_OR_RETURN_VOID(pFecsGlobalTraceInfo != NULL);
+
+    --pFecsGlobalTraceInfo->fecsCtxswLogConsumerCount;
 
     unregisterEventNotificationWithData(&pEventBuffer->pListeners,
             pBind->hClient,
@@ -1133,11 +1367,11 @@ fecsRemoveBindpoint
             NV_TRUE,
             pEventBuffer->producerInfo.notificationHandle);
 
-    multimapRemoveItemByKey(&pGpu->fecsEventBufferBindingsUid,
+    multimapRemoveItemByKey(&pFecsGlobalTraceInfo->fecsEventBufferBindingsUid,
             uid,
             (NvU64)(NvUPtr)pEventBuffer);
 
-    if (pGpu->fecsCtxswLogConsumerCount == 0)
+    if (pFecsGlobalTraceInfo->fecsCtxswLogConsumerCount == 0)
     {
         NvBool bMIGInUse = IS_MIG_IN_USE(pGpu);
         NvU8 grIdx;
@@ -1162,7 +1396,34 @@ fecsRemoveBindpoint
 
         if (!bIntrDriven)
         {
-            osRemove1SecondRepeatingCallback(pGpu, nvEventBufferFecsCallback, NULL);
+            _fecsTimerDestroy(pGpu);
+        }
+    }
+}
+
+void
+fecsRemoveAllBindpointsForGpu
+(
+    OBJGPU *pGpu
+)
+{
+    KernelGraphicsManager *pKernelGraphicsManager = GPU_GET_KERNEL_GRAPHICS_MANAGER(pGpu);
+    KGRMGR_FECS_GLOBAL_TRACE_INFO *pFecsGlobalTraceInfo = kgrmgrGetFecsGlobalTraceInfo(pGpu, pKernelGraphicsManager);
+    FecsEventBufferBindMultiMapSupermapIter iter;
+
+    NV_CHECK_OR_RETURN_VOID(LEVEL_SILENT, pFecsGlobalTraceInfo != NULL);
+
+    iter = multimapSubmapIterAll(&pFecsGlobalTraceInfo->fecsEventBufferBindingsUid);
+    while (multimapSubmapIterNext(&iter))
+    {
+        FecsEventBufferBindMultiMapSubmap *pSubmap = iter.pValue;
+        FecsEventBufferBindMultiMapIter subIter = multimapSubmapIterItems(&pFecsGlobalTraceInfo->fecsEventBufferBindingsUid, pSubmap);
+        NvU64 uid = mapKey_IMPL(iter.iter.pMap, pSubmap);
+
+        while (multimapItemIterNext(&subIter))
+        {
+            NV_EVENT_BUFFER_BIND_POINT_FECS *pBind = subIter.pValue;
+            fecsRemoveBindpoint(pGpu, uid, pBind);
         }
     }
 }
@@ -1176,6 +1437,7 @@ fecsRemoveAllBindpoints
     OBJGPU *pGpu = NULL;
     NvU32 gpuMask = 0;
     NvU32 gpuIndex = 0;
+    KernelGraphicsManager *pKernelGraphicsManager;
     FecsEventBufferBindMultiMapSupermapIter uidBindIter;
 
     eventBufferSetEnable(&pEventBuffer->producerInfo, NV_FALSE);
@@ -1183,14 +1445,19 @@ fecsRemoveAllBindpoints
     gpumgrGetGpuAttachInfo(NULL, &gpuMask);
     while ((pGpu = gpumgrGetNextGpu(gpuMask, &gpuIndex)) != NULL)
     {
-        uidBindIter = multimapSubmapIterAll(&pGpu->fecsEventBufferBindingsUid);
+        pKernelGraphicsManager = GPU_GET_KERNEL_GRAPHICS_MANAGER(pGpu);
+        KGRMGR_FECS_GLOBAL_TRACE_INFO *pFecsGlobalTraceInfo = kgrmgrGetFecsGlobalTraceInfo(pGpu, pKernelGraphicsManager);
+
+        NV_CHECK_OR_ELSE(LEVEL_ERROR, pFecsGlobalTraceInfo != NULL, continue;);
+
+        uidBindIter = multimapSubmapIterAll(&pFecsGlobalTraceInfo->fecsEventBufferBindingsUid);
         while (multimapSubmapIterNext(&uidBindIter))
         {
             FecsEventBufferBindMultiMapSubmap *pSubmap = uidBindIter.pValue;
             NV_EVENT_BUFFER_BIND_POINT_FECS *pBind = NULL;
             NvU64 uid = mapKey_IMPL(uidBindIter.iter.pMap, pSubmap);
 
-            while ((pBind = multimapFindItem(&pGpu->fecsEventBufferBindingsUid,
+            while ((pBind = multimapFindItem(&pFecsGlobalTraceInfo->fecsEventBufferBindingsUid,
                             uid,
                             (NvU64)(NvUPtr)pEventBuffer)) != NULL)
             {
@@ -1224,10 +1491,10 @@ fecsBufferReset
     NV_ASSERT_OK_OR_ELSE(
         status,
         _fecsLoadInternalRoutingInfo(pGpu,
-                                        pKernelGraphics,
-                                        &hClient,
-                                        &hSubdevice,
-                                        &getHwEnableParams.grRouteInfo),
+                                     pKernelGraphics,
+                                     &hClient,
+                                     &hSubdevice,
+                                     &getHwEnableParams.grRouteInfo),
         return);
 
     NV_ASSERT_OK_OR_ELSE(
@@ -1320,31 +1587,29 @@ fecsBufferDisableHw
                                      &getHwEnableParams.grRouteInfo),
         return);
 
-    NV_ASSERT_OK_OR_ELSE(
-        status,
-        pRmApi->Control(pRmApi,
-                        hClient,
-                        hSubdevice,
-                        NV2080_CTRL_CMD_INTERNAL_GR_GET_FECS_TRACE_HW_ENABLE,
-                        &getHwEnableParams,
-                        sizeof(getHwEnableParams)),
-        return);
-
+    status = pRmApi->Control(pRmApi,
+                             hClient,
+                             hSubdevice,
+                             NV2080_CTRL_CMD_INTERNAL_GR_GET_FECS_TRACE_HW_ENABLE,
+                             &getHwEnableParams,
+                             sizeof(getHwEnableParams));
+    NV_ASSERT_OR_RETURN_VOID((status == NV_OK) || (status == NV_ERR_GPU_IN_FULLCHIP_RESET));
+    if (status == NV_ERR_GPU_IN_FULLCHIP_RESET)
+        return;
+ 
     if (getHwEnableParams.bEnable)
     {
         // Copy previously loaded routing info
         setHwEnableParams.grRouteInfo = getHwEnableParams.grRouteInfo;
         setHwEnableParams.bEnable = NV_FALSE;
 
-        NV_ASSERT_OK_OR_ELSE(
-            status,
-            pRmApi->Control(pRmApi,
-                            hClient,
-                            hSubdevice,
-                            NV2080_CTRL_CMD_INTERNAL_GR_SET_FECS_TRACE_HW_ENABLE,
-                            &setHwEnableParams,
-                            sizeof(setHwEnableParams)),
-            return);
+        status = pRmApi->Control(pRmApi,
+                                 hClient,
+                                 hSubdevice,
+                                 NV2080_CTRL_CMD_INTERNAL_GR_SET_FECS_TRACE_HW_ENABLE,
+                                 &setHwEnableParams,
+                                 sizeof(setHwEnableParams));
+        NV_ASSERT_OR_RETURN_VOID((status == NV_OK) || (status == NV_ERR_GPU_IN_FULLCHIP_RESET));
     }
 }
 
@@ -1384,6 +1649,7 @@ fecsBufferMap
     NvU8 *pFecsBufferMapping = NULL;
     NV_STATUS status;
     KGRAPHICS_FECS_TRACE_INFO *pFecsTraceInfo = kgraphicsGetFecsTraceInfo(pGpu, pKernelGraphics);
+    MemoryManager *pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
 
     NV_ASSERT_OR_RETURN(pFecsTraceInfo != NULL, NV_ERR_INVALID_STATE);
 
@@ -1397,7 +1663,9 @@ fecsBufferMap
     if ((status != NV_OK) || (pFecsMemDesc == NULL))
         return NV_ERR_INVALID_STATE;
 
-    pFecsBufferMapping = kbusMapRmAperture_HAL(pGpu, pFecsMemDesc);
+    pFecsBufferMapping = memmgrMemDescBeginTransfer(pMemoryManager, pFecsMemDesc,
+                                                    TRANSFER_FLAGS_PREFER_PROCESSOR |
+                                                    TRANSFER_FLAGS_PERSISTENT_CPU_MAPPING);
     if (pFecsBufferMapping == NULL)
         return NV_ERR_INSUFFICIENT_RESOURCES;
 
@@ -1416,6 +1684,7 @@ fecsBufferUnmap
     MEMORY_DESCRIPTOR *pFecsMemDesc = NULL;
     NV_STATUS status;
     KGRAPHICS_FECS_TRACE_INFO *pFecsTraceInfo = kgraphicsGetFecsTraceInfo(pGpu, pKernelGraphics);
+    MemoryManager *pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
 
     NV_ASSERT_OR_RETURN_VOID(pFecsTraceInfo != NULL);
 
@@ -1423,10 +1692,17 @@ fecsBufferUnmap
         return;
 
     status = _getFecsMemDesc(pGpu, pKernelGraphics, &pFecsMemDesc);
-    if ((status == NV_OK) && (pFecsMemDesc != NULL) && (pFecsTraceInfo->pFecsBufferMapping != NULL))
-        kbusUnmapRmAperture_HAL(pGpu, pFecsMemDesc,
-                                &pFecsTraceInfo->pFecsBufferMapping,
-                                NV_TRUE);
+    if ((status != NV_OK) || (pFecsMemDesc == NULL))
+        return;
+
+    if (pFecsTraceInfo->pFecsBufferMapping != NULL)
+    {
+        memmgrMemDescEndTransfer(pMemoryManager, pFecsMemDesc,
+                                 TRANSFER_FLAGS_PREFER_PROCESSOR |
+                                 TRANSFER_FLAGS_PERSISTENT_CPU_MAPPING);
+
+        pFecsTraceInfo->pFecsBufferMapping = NULL;
+    }
 }
 
 /*! Atomically set intr callback pending, return NV_TRUE if wasn't pending prior */
@@ -1473,6 +1749,52 @@ NvBool fecsIsIntrPending
     return portAtomicOrU32(&pFecsTraceInfo->fecsCtxswLogIntrPending, 0) != 0;
 }
 
+NvS16
+fecsGetCtxswLogConsumerCount
+(
+    OBJGPU *pGpu,
+    KernelGraphicsManager *pKernelGraphicsManager
+)
+{
+    KGRMGR_FECS_GLOBAL_TRACE_INFO *pFecsGlobalTraceInfo = kgrmgrGetFecsGlobalTraceInfo(pGpu, pKernelGraphicsManager);
+
+    NV_CHECK_OR_RETURN(LEVEL_ERROR, pFecsGlobalTraceInfo != NULL, 0);
+
+    return pFecsGlobalTraceInfo->fecsCtxswLogConsumerCount;
+}
+
+NV_STATUS
+fecsIncrementCtxswLogConsumerCount
+(
+    OBJGPU *pGpu,
+    KernelGraphicsManager *pKernelGraphicsManager
+)
+{
+    KGRMGR_FECS_GLOBAL_TRACE_INFO *pFecsGlobalTraceInfo = kgrmgrGetFecsGlobalTraceInfo(pGpu, pKernelGraphicsManager);
+
+    NV_ASSERT_OR_RETURN(pFecsGlobalTraceInfo != NULL, NV_ERR_INVALID_STATE);
+
+    pFecsGlobalTraceInfo->fecsCtxswLogConsumerCount++;
+
+    return NV_OK;
+}
+
+NV_STATUS
+fecsDecrementCtxswLogConsumerCount
+(
+    OBJGPU *pGpu,
+    KernelGraphicsManager *pKernelGraphicsManager
+)
+{
+    KGRMGR_FECS_GLOBAL_TRACE_INFO *pFecsGlobalTraceInfo = kgrmgrGetFecsGlobalTraceInfo(pGpu, pKernelGraphicsManager);
+
+    NV_ASSERT_OR_RETURN(pFecsGlobalTraceInfo != NULL, NV_ERR_INVALID_STATE);
+
+    pFecsGlobalTraceInfo->fecsCtxswLogConsumerCount--;
+
+    return NV_OK;
+}
+
 /*! Retrieve the current VGPU staging buffer */
 VGPU_FECS_TRACE_STAGING_BUFFER *
 fecsGetVgpuStagingBuffer
@@ -1504,3 +1826,17 @@ fecsSetVgpuStagingBuffer
     pFecsTraceInfo->pVgpuStaging = pStagingBuffer;
 }
 
+FecsEventBufferBindMultiMap *
+fecsGetEventBufferBindMultiMap
+(
+    OBJGPU *pGpu,
+    KernelGraphicsManager *pKernelGraphicsManager
+)
+{
+    KGRMGR_FECS_GLOBAL_TRACE_INFO *pFecsGlobalTraceInfo = kgrmgrGetFecsGlobalTraceInfo(pGpu, pKernelGraphicsManager);
+
+    NV_ASSERT_OR_RETURN(pFecsGlobalTraceInfo != NULL, NULL);
+
+    return &pFecsGlobalTraceInfo->fecsEventBufferBindingsUid;
+}
+
