@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2023-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -34,8 +34,9 @@
 #include "core/prelude.h"
 #include "core/locks.h"
 #include "gpu/mem_mgr/ce_utils.h"
-#include "gpu/subdevice/subdevice.h"
 #include "kernel/gpu/mem_mgr/ce_utils_sizes.h"
+#include "vgpu/rpc_headers.h"
+#include "gpu/device/device.h"
 
 #include "class/clb0b5.h" // MAXWELL_DMA_COPY_A
 #include "class/clc0b5.h" // PASCAL_DMA_COPY_A
@@ -47,6 +48,47 @@
 
 #include "class/cl0080.h"
 
+NV_STATUS
+ceutilsGetFirstAsyncCe_IMPL
+(
+    CeUtils  *pCeUtils,
+    OBJGPU   *pGpu,
+    RsClient *pClient,
+    NvHandle  hDevice,
+    NvU32    *pCeInstance,
+    NvBool    forceSkipMIG
+)
+{
+    if (IS_MIG_IN_USE(pGpu) && !forceSkipMIG)
+    {
+        Device *pDevice;
+
+        NV_ASSERT_OK_OR_RETURN(
+            deviceGetByHandle(pClient, hDevice, &pDevice));
+
+        NV_ASSERT_OK_OR_RETURN(kmigmgrGetGPUInstanceScrubberCe(pGpu, GPU_GET_KERNEL_MIG_MANAGER(pGpu), pDevice, pCeInstance));
+        return NV_OK;
+    }
+    else
+    {
+        NV_CHECK_OK_OR_RETURN(LEVEL_ERROR, gpuUpdateEngineTable(pGpu));
+
+        KernelCE  *pKCe = NULL;
+
+        KCE_ITER_ALL_BEGIN(pGpu, pKCe, 0)
+            if (gpuCheckEngine_HAL(pGpu, ENG_CE(pKCe->publicID)) &&
+               !ceIsCeGrce(pGpu, RM_ENGINE_TYPE_COPY(pKCe->publicID)) &&
+               gpuCheckEngineTable(pGpu, RM_ENGINE_TYPE_COPY(pKCe->publicID)))
+            {
+                *pCeInstance = pKCe->publicID;
+                return NV_OK;
+            }
+        KCE_ITER_END
+    }
+
+    return NV_ERR_INSUFFICIENT_RESOURCES;
+}
+
 NV_STATUS
 ceutilsConstruct_IMPL
 (
@@ -58,14 +100,20 @@ ceutilsConstruct_IMPL
 {
     NV_STATUS status = NV_OK;
     NvU64 allocFlags = pAllocParams->flags;
+    pCeUtils->bForcedCeId = FLD_TEST_DRF(0050_CEUTILS, _FLAGS, _FORCE_CE_ID, _TRUE, allocFlags);
     NV_ASSERT_OR_RETURN(pGpu, NV_ERR_INVALID_STATE);
 
     NvBool bMIGInUse = IS_MIG_IN_USE(pGpu);
     MemoryManager *pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
+    pCeUtils->pGpu = pGpu;
+
+    if (FLD_TEST_DRF(0050_CEUTILS, _FLAGS, _FIFO_LITE, _TRUE, allocFlags))
+    {
+        return NV_ERR_NOT_SUPPORTED;
+    }
 
     // Allocate channel with RM internal client
     RM_API *pRmApi = rmapiGetInterface(RMAPI_GPU_LOCK_INTERNAL);
-    RmClient *pClient;
 
     OBJCHANNEL *pChannel = (OBJCHANNEL *) portMemAllocNonPaged(sizeof(OBJCHANNEL));
     if (pChannel == NULL)
@@ -85,37 +133,66 @@ ceutilsConstruct_IMPL
     }
 
     pChannel->hClient = pCeUtils->hClient;
-    pClient = serverutilGetClientUnderLock(pChannel->hClient);
-    NV_ASSERT_OR_GOTO(pClient != NULL, free_client);
 
     status = serverGetClientUnderLock(&g_resServ, pChannel->hClient, &pChannel->pRsClient);
     NV_ASSERT_OR_GOTO(status == NV_OK, free_client);
 
-    status = clientSetHandleGenerator(staticCast(pClient, RsClient), 1U, ~0U - 1U);
-    NV_ASSERT_OR_GOTO(status == NV_OK, free_client);
+    if (IS_VIRTUAL(pGpu))
+    {
+        NV_ASSERT_OK_OR_GOTO(
+            status,
+            clientSetHandleGenerator(pChannel->pRsClient, RS_UNIQUE_HANDLE_BASE,
+                                     RS_UNIQUE_HANDLE_RANGE/2 - VGPU_RESERVED_HANDLE_RANGE),
+            free_client);
+    }
+    else
+    {
+        NV_ASSERT_OK_OR_GOTO(
+            status,
+            clientSetHandleGenerator(pChannel->pRsClient, 1U, ~0U - 1U),
+            free_client);
+    }
 
     pChannel->bClientAllocated = NV_TRUE;
     pChannel->pGpu = pGpu;
-
-    pChannel->deviceId = pCeUtils->hDevice;
-    pChannel->subdeviceId = pCeUtils->hSubdevice;
-
     pChannel->pKernelMIGGpuInstance = pKernelMIGGPUInstance;
 
     // We'll allocate new VAS for now. Sharing client VAS will be added later
     pChannel->hVASpaceId = NV01_NULL_OBJECT;
     pChannel->bUseVasForCeCopy = FLD_TEST_DRF(0050_CEUTILS, _FLAGS, _VIRTUAL_MODE, _TRUE, allocFlags);
 
+    // Variable to indicate usage of either BAR1 or BAR2
+    pChannel->bUseBar1 = FLD_TEST_DRF(0050_CEUTILS, _FLAGS, _NO_BAR1_USE, _FALSE, allocFlags);
+
+    pChannel->bSecure = FLD_TEST_DRF(0050_CEUTILS, _FLAGS, _CC_SECURE, _TRUE, allocFlags);
+
     // Detect if we can enable fast scrub on this channel
     status = memmgrMemUtilsGetCopyEngineClass_HAL(pGpu, pMemoryManager, &pCeUtils->hTdCopyClass);
     NV_ASSERT_OR_GOTO(status == NV_OK, free_channel);
 
     if (((pCeUtils->hTdCopyClass == HOPPER_DMA_COPY_A)
+        || (pCeUtils->hTdCopyClass == BLACKWELL_DMA_COPY_A)
+        || (pCeUtils->hTdCopyClass == BLACKWELL_DMA_COPY_B)
         ) && !pChannel->bUseVasForCeCopy)
     {
         pChannel->type = FAST_SCRUBBER_CHANNEL;
         NV_PRINTF(LEVEL_INFO, "Enabled fast scrubber in construct.\n");
     }
+    else
+    {
+        pChannel->type = CE_SCRUBBER_CHANNEL;
+    }
+
+    // For self-hosted Hopper, we can only use VA copy or faster scrubber
+    if (pMemoryManager->bCePhysicalVidmemAccessNotSupported)
+    {
+        if (!pChannel->bUseVasForCeCopy &&
+            (pChannel->type != FAST_SCRUBBER_CHANNEL))
+        {
+            status = NV_ERR_NOT_SUPPORTED;
+            goto free_channel;
+        }
+    }
 
     // Set up various channel resources
     status = channelSetupIDs(pChannel, pGpu, pChannel->bUseVasForCeCopy, bMIGInUse);
@@ -123,6 +200,19 @@ ceutilsConstruct_IMPL
 
     channelSetupChannelBufferSizes(pChannel);
 
+    NV_ASSERT_OK_OR_GOTO(status, channelAllocSubdevice(pGpu, pChannel), free_client);
+
+    if (pCeUtils->bForcedCeId)
+    {
+        pChannel->ceId = pAllocParams->forceCeId;
+    }
+    else
+    {
+        NV_ASSERT_OK_OR_GOTO(status,
+            ceutilsGetFirstAsyncCe(pCeUtils, pGpu, pChannel->pRsClient, pChannel->deviceId, &pChannel->ceId, NV_FALSE),
+            free_client);
+    }
+
     status = memmgrMemUtilsChannelInitialize_HAL(pGpu, pMemoryManager, pChannel);
     NV_ASSERT_OR_GOTO(status == NV_OK, free_channel);
 
@@ -133,8 +223,6 @@ ceutilsConstruct_IMPL
     status = memmgrMemUtilsCopyEngineInitialize_HAL(pGpu, pMemoryManager, pChannel);
     NV_ASSERT_OR_GOTO(status == NV_OK, free_channel);
 
-    pCeUtils->pGpu = pGpu;
-
     return status;
 
 free_channel:
@@ -166,18 +254,7 @@ ceutilsDestruct_IMPL
     OBJGPU *pGpu = pCeUtils->pGpu;
     MemoryManager *pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
     RM_API *pRmApi = rmapiGetInterface(RMAPI_GPU_LOCK_INTERNAL);
-
-    // Sanity checks
-    if ((pGpu == NULL) || (pChannel == NULL))
-    {
-        NV_PRINTF(LEVEL_WARNING, "Possible double-free of CeUtils!\n");
-        return;
-    }
-    else if (pGpu != pChannel->pGpu)
-    {
-        NV_PRINTF(LEVEL_ERROR, "Bad state during ceUtils teardown!\n");
-        return;
-    }
+    NvU32 transferFlags = pChannel->bUseBar1 ? TRANSFER_FLAGS_USE_BAR1 : TRANSFER_FLAGS_NONE;
 
     if ((pChannel->bClientUserd) && (pChannel->pControlGPFifo != NULL))
     {
@@ -191,7 +268,7 @@ ceutilsDestruct_IMPL
         }
         else
         {
-            memmgrMemDescEndTransfer(pMemoryManager, pChannel->pUserdMemdesc, TRANSFER_FLAGS_USE_BAR1);
+            memmgrMemDescEndTransfer(pMemoryManager, pChannel->pUserdMemdesc, transferFlags);
             pChannel->pControlGPFifo = NULL;
         }
     }
@@ -204,7 +281,7 @@ ceutilsDestruct_IMPL
         }
         else
         {
-            memmgrMemDescEndTransfer(pMemoryManager, pChannel->pChannelBufferMemdesc, TRANSFER_FLAGS_USE_BAR1);
+            memmgrMemDescEndTransfer(pMemoryManager, pChannel->pChannelBufferMemdesc, transferFlags);
             pChannel->pbCpuVA = NULL;
         }
     }
@@ -217,18 +294,11 @@ ceutilsDestruct_IMPL
         }
         else
         {
-            memmgrMemDescEndTransfer(pMemoryManager, pChannel->pErrNotifierMemdesc, TRANSFER_FLAGS_USE_BAR1);
+            memmgrMemDescEndTransfer(pMemoryManager, pChannel->pErrNotifierMemdesc, transferFlags);
             pChannel->pTokenFromNotifier = NULL;
         }
     }
 
-    pRmApi->Free(pRmApi, pChannel->hClient, pChannel->channelId);
-    pRmApi->Free(pRmApi, pChannel->hClient, pChannel->errNotifierIdPhys);
-    pRmApi->Free(pRmApi, pChannel->hClient, pChannel->pushBufferId);
-    pRmApi->Free(pRmApi, pChannel->hClient, pChannel->errNotifierIdVirt);
-    pRmApi->Free(pRmApi, pChannel->hClient, pChannel->hVASpaceId);
-    pRmApi->Free(pRmApi, pChannel->hClient, pChannel->deviceId);
-
     // Resource server makes sure no leak can occur
     pRmApi->Free(pRmApi, pChannel->hClient, pChannel->hClient);
     portMemFree(pChannel);
@@ -246,14 +316,9 @@ ceutilsServiceInterrupts_IMPL(CeUtils *pCeUtils)
     // if the lock is held, service any interrupts on the owned CE to make progress.
     // Bug 2527660 is filed to remove this change.
     //
-    // pChannel is null when PMA scrub requests are handled in vGPU plugin.
-    // In this case vGpu plugin allocates scrubber channel in PF domain so
-    // above mention deadlock is not present here.
-    //
-    if ((pChannel != NULL) && (rmDeviceGpuLockIsOwner(pChannel->pGpu->gpuInstance)))
+    if (rmDeviceGpuLockIsOwner(pChannel->pGpu->gpuInstance))
     {
-        Intr *pIntr = GPU_GET_INTR(pChannel->pGpu);
-        intrServiceStallSingle_HAL(pChannel->pGpu, pIntr, MC_ENGINE_IDX_CE(pChannel->ceId), NV_FALSE);
+        channelServiceScrubberInterrupts(pChannel);
     }
     else
     {
@@ -265,7 +330,7 @@ ceutilsServiceInterrupts_IMPL(CeUtils *pCeUtils)
 static NvBool
 _ceUtilsFastScrubEnabled
 (
-    POBJCHANNEL      pChannel,
+    OBJCHANNEL      *pChannel,
     CHANNEL_PB_INFO *pChannelPbInfo
 )
 {
@@ -303,7 +368,7 @@ _ceUtilsFastScrubEnabled
 static NV_STATUS
 _ceutilsSubmitPushBuffer
 (
-    POBJCHANNEL       pChannel,
+    OBJCHANNEL       *pChannel,
     NvBool            bPipelined,
     NvBool            bInsertFinishPayload,
     CHANNEL_PB_INFO * pChannelPbInfo
@@ -322,9 +387,9 @@ _ceutilsSubmitPushBuffer
     // Use BAR1 if CPU access is allowed, otherwise allocate and init shadow
     // buffer for DMA access
     //
-    NvU32 transferFlags = (TRANSFER_FLAGS_USE_BAR1     | 
+    NvU32 transferFlags = (pChannel->bUseBar1 ? TRANSFER_FLAGS_USE_BAR1 : TRANSFER_FLAGS_NONE) |
                            TRANSFER_FLAGS_SHADOW_ALLOC | 
-                           TRANSFER_FLAGS_SHADOW_INIT_MEM);
+                           TRANSFER_FLAGS_SHADOW_INIT_MEM;
     NV_PRINTF(LEVEL_INFO, "Actual size of copying to be pushed: %x\n", pChannelPbInfo->size);
 
     status = channelWaitForFreeEntry(pChannel, &putIndex);
@@ -348,7 +413,13 @@ _ceutilsSubmitPushBuffer
     }
     else
     {
-        methodsLength = channelFillPb(pChannel, putIndex, bPipelined, bInsertFinishPayload, pChannelPbInfo);
+        if (pMemoryManager->bCePhysicalVidmemAccessNotSupported)
+        {
+            // Self-hosted Hopper only supports VA copy or fast scrubber
+            NV_ASSERT_OR_RETURN(pChannel->bUseVasForCeCopy, NV_ERR_NOT_SUPPORTED);
+        }
+
+        methodsLength = channelFillCePb(pChannel, putIndex, bPipelined, bInsertFinishPayload, pChannelPbInfo);
     }
 
     if (bReleaseMapping)
@@ -399,14 +470,15 @@ ceutilsMemset_IMPL
 
     NvBool bPipelined = pParams->flags & NV0050_CTRL_MEMSET_FLAGS_PIPELINED;
 
+    NV_ASSERT_OR_RETURN(!ceutilsIsSubmissionPaused(pCeUtils), NV_ERR_BUSY_RETRY);
+
     if (pMemDesc == NULL)
     {
         NV_PRINTF(LEVEL_ERROR, "Invalid memdesc for CeUtils memset.\n");
         return NV_ERR_INVALID_ARGUMENT;
     }
 
-    if ((memdescGetAddressSpace(pMemDesc) != ADDR_FBMEM) ||
-        (pMemDesc->pGpu != pCeUtils->pChannel->pGpu))
+    if (pMemDesc->pGpu != pCeUtils->pChannel->pGpu)
     {
         NV_PRINTF(LEVEL_ERROR, "Invalid memory descriptor passed.\n");
         return NV_ERR_INVALID_ARGUMENT;
@@ -443,16 +515,25 @@ ceutilsMemset_IMPL
     memsetLength = pParams->length;
     offset = pParams->offset;
 
+    //
+    // We need not just the physical address,
+    // but the physical address to be used by the engine
+    // ce2utils is only used for AT_GPU
+    //
+
     do
     {
-        NvU64 maxContigSize = bContiguous ? memsetLength : (pageGranularity - offset % pageGranularity);
+        //
+        // Use the memdesc phys addr for calculations, but the pte address for the value
+        // programmed into CE
+        //
+        NvU64 dstAddr = memdescGetPhysAddr(pMemDesc, AT_GPU, offset);
+        NvU64 maxContigSize = bContiguous ? memsetLength : (pageGranularity - dstAddr % pageGranularity);
         NvU32 memsetSizeContig = (NvU32)NV_MIN(NV_MIN(memsetLength, maxContigSize), CE_MAX_BYTES_PER_LINE);
 
-        channelPbInfo.dstAddr = memdescGetPhysAddr(pMemDesc, AT_GPU, offset);
-
-        NV_PRINTF(LEVEL_INFO, "CeUtils Memset dstAddr: %llx,  size: %x\n",
-                  channelPbInfo.dstAddr, memsetSizeContig);
+        NV_PRINTF(LEVEL_INFO, "CeUtils Memset dstAddr: %llx,  size: %x\n", dstAddr, memsetSizeContig);
 
+        channelPbInfo.dstAddr = memdescGetPtePhysAddr(pMemDesc, AT_GPU, offset);
         channelPbInfo.size = memsetSizeContig;
         status = _ceutilsSubmitPushBuffer(pChannel, bPipelined, memsetSizeContig == memsetLength, &channelPbInfo);
         if (status != NV_OK)
@@ -509,6 +590,8 @@ ceutilsMemcopy_IMPL
 
     NvBool bPipelined = pParams->flags & NV0050_CTRL_MEMCOPY_FLAGS_PIPELINED;
 
+    NV_ASSERT_OR_RETURN(!ceutilsIsSubmissionPaused(pCeUtils), NV_ERR_BUSY_RETRY);
+
     // Validate params
     if ((pSrcMemDesc == NULL) || (pDstMemDesc == NULL))
     {
@@ -516,13 +599,6 @@ ceutilsMemcopy_IMPL
         return NV_ERR_INVALID_ARGUMENT;
     }
 
-    if ((memdescGetAddressSpace(pSrcMemDesc) != ADDR_FBMEM) &&
-        (memdescGetAddressSpace(pDstMemDesc) != ADDR_FBMEM))
-    {
-        NV_PRINTF(LEVEL_ERROR, "Either Dst or Src memory should be in vidmem.\n");
-        return NV_ERR_INVALID_ARGUMENT;
-    }
-
     if ((pSrcMemDesc->pGpu != pCeUtils->pChannel->pGpu) ||
         (pDstMemDesc->pGpu != pCeUtils->pChannel->pGpu))
     {
@@ -556,6 +632,11 @@ ceutilsMemcopy_IMPL
     channelPbInfo.srcCpuCacheAttrib = pSrcMemDesc->_cpuCacheAttrib;
     channelPbInfo.dstCpuCacheAttrib = pDstMemDesc->_cpuCacheAttrib;
 
+    channelPbInfo.bSecureCopy = pParams->bSecureCopy;
+    channelPbInfo.bEncrypt = pParams->bEncrypt;
+    channelPbInfo.authTagAddr = pParams->authTagAddr;
+    channelPbInfo.encryptIvAddr = pParams->encryptIvAddr;
+
     srcPageGranularity = pSrcMemDesc->pageArrayGranularity;
     dstPageGranularity = pDstMemDesc->pageArrayGranularity;
     bSrcContig = memdescGetContiguity(pSrcMemDesc, AT_GPU);
@@ -569,16 +650,17 @@ ceutilsMemcopy_IMPL
         // This algorithm finds the maximum contig region from both src and dst
         // for each copy and iterate until we submitted the whole range to CE
         //
-        NvU64 maxContigSizeSrc = bSrcContig ? copyLength : (srcPageGranularity - srcOffset % srcPageGranularity);
-        NvU64 maxContigSizeDst = bDstContig ? copyLength : (dstPageGranularity - dstOffset % dstPageGranularity);
+        NvU64 srcAddr = memdescGetPhysAddr(pSrcMemDesc, AT_GPU, srcOffset);
+        NvU64 dstAddr = memdescGetPhysAddr(pDstMemDesc, AT_GPU, dstOffset);
+        NvU64 maxContigSizeSrc = bSrcContig ? copyLength : (srcPageGranularity - srcAddr % srcPageGranularity);
+        NvU64 maxContigSizeDst = bDstContig ? copyLength : (dstPageGranularity - dstAddr % dstPageGranularity);
         NvU32 copySizeContig = (NvU32)NV_MIN(NV_MIN(copyLength, NV_MIN(maxContigSizeSrc, maxContigSizeDst)), CE_MAX_BYTES_PER_LINE);
 
-        channelPbInfo.srcAddr = memdescGetPhysAddr(pSrcMemDesc, AT_GPU, srcOffset);
-        channelPbInfo.dstAddr = memdescGetPhysAddr(pDstMemDesc, AT_GPU, dstOffset);
-
         NV_PRINTF(LEVEL_INFO, "CeUtils Memcopy dstAddr: %llx, srcAddr: %llx, size: %x\n",
-                  channelPbInfo.dstAddr, channelPbInfo.srcAddr, copySizeContig);
+                  dstAddr, srcAddr, copySizeContig);
 
+        channelPbInfo.srcAddr = srcAddr;
+        channelPbInfo.dstAddr = dstAddr;
         channelPbInfo.size = copySizeContig;
         status = _ceutilsSubmitPushBuffer(pChannel, bPipelined, copySizeContig == copyLength, &channelPbInfo);
         if (status != NV_OK)
@@ -653,7 +735,56 @@ ceutilsUpdateProgress_IMPL
     return swLastCompletedPayload;
 }
 
-#if defined(DEBUG) || defined (DEVELOP)
+void
+ceutilsPauseSubmission_IMPL
+(
+    CeUtils *pCeUtils,
+    NvBool bWaitForWorkCompletion
+)
+{
+    if (bWaitForWorkCompletion)
+    {
+        channelWaitForFinishPayload(pCeUtils->pChannel, pCeUtils->lastSubmittedPayload);
+    }
+
+    pCeUtils->submissionPausedRefCount++;
+}
+
+void
+ceutilsResumeSubmission_IMPL
+(
+    CeUtils *pCeUtils
+)
+{
+    NV_ASSERT_OR_RETURN_VOID(ceutilsIsSubmissionPaused(pCeUtils));
+    NV_ASSERT(ceutilsUsesPreferredCe(pCeUtils));
+
+    pCeUtils->submissionPausedRefCount--;
+}
+
+NvBool
+ceutilsUsesPreferredCe_IMPL
+(
+    CeUtils *pCeUtils
+)
+{
+    OBJGPU *pGpu = pCeUtils->pGpu;
+    OBJCHANNEL *pChannel = pCeUtils->pChannel;
+    NvU32 ceId;
+
+    if (pCeUtils->bForcedCeId || pCeUtils->pLiteKernelChannel != NULL)
+    {
+        // short-lived objects, should not have to change CE
+        NV_ASSERT_OR_RETURN(0, NV_TRUE);
+    }
+
+    NV_ASSERT_OR_RETURN(
+        ceutilsGetFirstAsyncCe(pCeUtils, pGpu, pChannel->pRsClient, pChannel->deviceId, &ceId, NV_FALSE) == NV_OK,
+        NV_FALSE);
+
+    return (ceId == pChannel->ceId);
+}
+
 NV_STATUS
 ceutilsapiCtrlCmdCheckProgress_IMPL
 (
@@ -789,4 +920,3 @@ ceutilsapiCtrlCmdMemcopy_IMPL
 
     return status;
 }
-#endif // defined(DEBUG) || defined (DEVELOP)
