@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2016-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2016-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -29,7 +29,7 @@
 #include "kernel/gpu/mig_mgr/kernel_mig_manager.h"
 #include "gpu/bus/kern_bus.h"
 #include "kernel/gpu/fifo/kernel_fifo.h"
-#include "objtmr.h"
+#include "gpu/timer/objtmr.h"
 #include "gpu/mem_mgr/mem_desc.h"
 #include "kernel/gpu/intr/intr.h"
 
@@ -45,6 +45,9 @@
 #include "nvstatus.h"
 #include "rmapi/rs_utils.h"
 #include "core/locks.h"
+
+#include "gpu/conf_compute/conf_compute.h"
+
 #include "class/cl0050.h"
 #include "class/clb0b5.h"   // MAXWELL_DMA_COPY_A
 #include "class/clc0b5.h"   // PASCAL_DMA_COPY_A
@@ -58,19 +61,25 @@
 
 #include "class/clc86f.h"   // HOPPER_CHANNEL_GPFIFO_A
 
+#include "class/clc9b5.h"      // BLACKWELL_DMA_COPY_A
+
+#include "class/clcab5.h"      // BLACKWELL_DMA_COPY_B
+
 static NvU64  _scrubCheckProgress(OBJMEMSCRUB *pScrubber);
 static NvU64  _searchScrubList(OBJMEMSCRUB *pScrubber, RmPhysAddr base, NvU64 size);
 static void   _waitForPayload(OBJMEMSCRUB  *pScrubber, RmPhysAddr  base, RmPhysAddr end);
 static void   _scrubAddWorkToList(OBJMEMSCRUB  *pScrubber, RmPhysAddr  base, NvU64  size, NvU64  newId);
 static NvU32  _scrubMemory(OBJMEMSCRUB  *pScrubber, RmPhysAddr base, NvU64 size,
-                           NvU32 dstCpuCacheAttrib, NvU32 freeToken);
+                           NvU32 dstCpuCacheAttrib, NvU32 freeToken, NvU32 flags);
 static void   _scrubWaitAndSave(OBJMEMSCRUB *pScrubber, PSCRUB_NODE pList, NvLength  itemsToSave);
 static NvU64  _scrubGetFreeEntries(OBJMEMSCRUB *pScrubber);
-static NvU64  _scrubCheckAndSubmit(OBJMEMSCRUB *pScrubber, NvU64  chunkSize, NvU64  *pPages,
-                                 NvU64  pageCount, PSCRUB_NODE  pList, NvLength  pagesToScrubCheck);
+static NvU64  _scrubCheckAndSubmit(OBJMEMSCRUB *pScrubber, NvU64 pageCount, PSCRUB_NODE  pList,
+                                   PSCRUB_NODE pScrubListCopy, NvLength  pagesToScrubCheck, NvU32 flags);
 static void   _scrubCopyListItems(OBJMEMSCRUB *pScrubber, PSCRUB_NODE pList, NvLength itemsToSave);
 
 static NV_STATUS _scrubCheckLocked(OBJMEMSCRUB  *pScrubber, PSCRUB_NODE *ppList, NvU64 *pSize);
+static NV_STATUS _scrubCombinePages(NvU64 *pPages, NvU64 pageSize, NvU64 pageCount,
+                                    PSCRUB_NODE *ppScrubList, NvU64 *pSize);
 
 /**
  * Constructs the memory scrubber object and signals
@@ -158,8 +167,21 @@ scrubberConstruct
             }
             FOR_EACH_VALID_GPU_INSTANCE_END();
         }
+        ConfidentialCompute *pConfCompute = GPU_GET_CONF_COMPUTE(pGpu);
+        if ((pConfCompute != NULL) &&
+            (pConfCompute->getProperty(pCC, PDB_PROP_CONFCOMPUTE_CC_FEATURE_ENABLED)))
+        {
+            NV_ASSERT_OK_OR_GOTO(status, objCreate(&pScrubber->pSec2Utils, pHeap, Sec2Utils, pGpu, pKernelMIGGPUInstance),
+               destroyscrublist);
+            pScrubber->bIsEngineTypeSec2 = NV_TRUE;
+        }
+        else
+        {
+            NV_ASSERT_OK_OR_GOTO(status, objCreate(&pScrubber->pCeUtils, pHeap, CeUtils, pGpu, pKernelMIGGPUInstance, &ceUtilsAllocParams),
+               destroyscrublist);
 
-        NV_ASSERT_OK_OR_GOTO(status, objCreate(&pScrubber->pCeUtils, pHeap, CeUtils, pGpu, pKernelMIGGPUInstance, &ceUtilsAllocParams), destroyscrublist);
+            pScrubber->bIsEngineTypeSec2 = NV_FALSE;
+        }
         NV_ASSERT_OK_OR_GOTO(status, pmaRegMemScrub(pPma, pScrubber), destroyscrublist);
     }
 
@@ -194,7 +216,17 @@ _isScrubWorkPending(
     }
     else
     {
-        if (pScrubber->lastSubmittedWorkId != ceutilsUpdateProgress(pScrubber->pCeUtils))
+        NvU64 lastCompleted;
+        if (pScrubber->bIsEngineTypeSec2)
+        {
+            lastCompleted = sec2utilsUpdateProgress(pScrubber->pSec2Utils);
+        }
+        else
+        {
+            lastCompleted = ceutilsUpdateProgress(pScrubber->pCeUtils);
+        }
+
+        if (pScrubber->lastSubmittedWorkId != lastCompleted)
             workPending = NV_TRUE;
     }
     return workPending;
@@ -268,7 +300,14 @@ scrubberDestruct
 
     portMemFree(pScrubber->pScrubList);
     {
-        objDelete(pScrubber->pCeUtils);
+        if (pScrubber->bIsEngineTypeSec2)
+        {
+            objDelete(pScrubber->pSec2Utils);
+        }
+        else
+        {
+            objDelete(pScrubber->pCeUtils);
+        }
     }
 
     portSyncMutexRelease(pScrubber->pScrubberMutex);
@@ -366,68 +405,85 @@ scrubSubmitPages
     NvU64       *pPages,
     NvU64        pageCount,
     PSCRUB_NODE *ppList,
-    NvU64       *pSize
+    NvU64       *pSize,
+    NvU32        flags
 )
 {
     NvU64       curPagesSaved     = 0;
     PSCRUB_NODE pScrubList        = NULL;
+    PSCRUB_NODE pScrubListCopy    = NULL;
+    NvU64       scrubListSize     = 0;
     NvLength    pagesToScrubCheck = 0;
     NvU64       totalSubmitted    = 0;
     NvU64       numFinished       = 0;
     NvU64       freeEntriesInList = 0;
     NvU64       scrubCount        = 0;
-    NvU64       numPagesToScrub   = pageCount;
+    NvU64       numPagesToScrub   = 0;
     NV_STATUS   status            = NV_OK;
 
     portSyncMutexAcquire(pScrubber->pScrubberMutex);
     *pSize  = 0;
     *ppList = pScrubList;
 
-    NV_PRINTF(LEVEL_INFO, "submitting pages, pageCount:%llx\n", pageCount);
+    NV_CHECK_OR_GOTO(LEVEL_INFO, pageCount > 0, cleanup);
+
+    NV_PRINTF(LEVEL_INFO, "submitting pages, pageCount = 0x%llx chunkSize = 0x%llx\n", pageCount, chunkSize);
 
     freeEntriesInList = _scrubGetFreeEntries(pScrubber);
-    if (freeEntriesInList < pageCount)
-    {
-        pScrubList = (PSCRUB_NODE)
-                     portMemAllocNonPaged((NvLength)(sizeof(SCRUB_NODE) * (pageCount - freeEntriesInList)));
 
-        if (pScrubList == NULL)
+    NV_ASSERT_OK_OR_GOTO(status,
+                         _scrubCombinePages(pPages,
+                                            chunkSize,
+                                            pageCount,
+                                            &pScrubList,
+                                            &scrubListSize),
+                         cleanup);
+
+    numPagesToScrub = scrubListSize;
+
+    if (freeEntriesInList < scrubListSize)
+    {
+        pScrubListCopy = (PSCRUB_NODE)
+                          portMemAllocNonPaged((NvLength)(sizeof(SCRUB_NODE) * (scrubListSize - freeEntriesInList)));
+
+        if (pScrubListCopy == NULL)
         {
             status = NV_ERR_NO_MEMORY;
             goto cleanup;
         }
 
-        while (freeEntriesInList < pageCount)
+        while (freeEntriesInList < scrubListSize)
         {
-            if (pageCount > MAX_SCRUB_ITEMS)
+            if (scrubListSize > MAX_SCRUB_ITEMS)
             {
                 pagesToScrubCheck = (NvLength)(MAX_SCRUB_ITEMS - freeEntriesInList);
                 scrubCount        = MAX_SCRUB_ITEMS;
             }
             else
             {
-                pagesToScrubCheck  = (NvLength)(pageCount - freeEntriesInList);
-                scrubCount         = pageCount;
+                pagesToScrubCheck = (NvLength)(scrubListSize - freeEntriesInList);
+                scrubCount        = scrubListSize;
             }
 
-            numFinished = _scrubCheckAndSubmit(pScrubber, chunkSize, &pPages[totalSubmitted],
-                                               scrubCount, &pScrubList[curPagesSaved],
-                                               pagesToScrubCheck);
+            numFinished = _scrubCheckAndSubmit(pScrubber, scrubCount,
+                                               &pScrubList[totalSubmitted],
+                                               &pScrubListCopy[curPagesSaved],
+                                               pagesToScrubCheck,
+                                               flags);
 
-            pageCount         -= numFinished;
+            scrubListSize     -= numFinished;
             curPagesSaved     += pagesToScrubCheck;
             totalSubmitted    += numFinished;
             freeEntriesInList  = _scrubGetFreeEntries(pScrubber);
         }
 
-        *ppList = pScrubList;
+        *ppList = pScrubListCopy;
         *pSize  = curPagesSaved;
     }
     else
     {
-        totalSubmitted = _scrubCheckAndSubmit(pScrubber, chunkSize, pPages,
-                                              pageCount, NULL,
-                                              0);
+        totalSubmitted = _scrubCheckAndSubmit(pScrubber, scrubListSize,
+                                              pScrubList, NULL, 0, flags);
         *ppList = NULL;
         *pSize  = 0;
     }
@@ -435,6 +491,12 @@ scrubSubmitPages
 cleanup:
     portSyncMutexRelease(pScrubber->pScrubberMutex);
 
+    if (pScrubList != NULL)
+    {
+        portMemFree(pScrubList);
+        pScrubList = NULL;
+    }
+
     NV_CHECK_OK_OR_RETURN(LEVEL_INFO, status);
 
     if (totalSubmitted == numPagesToScrub)
@@ -475,15 +537,33 @@ scrubWaitPages
 )
 {
 
-    NvU32     iter   = 0;
-    NV_STATUS status = NV_OK;
+    NvU32       iter          = 0;
+    NV_STATUS   status        = NV_OK;
+    PSCRUB_NODE pScrubList    = NULL;
+    NvU64       scrubListSize = 0;
+
+    NV_ASSERT_OK_OR_RETURN(_scrubCombinePages(pPages,
+                                              chunkSize,
+                                              pageCount,
+                                              &pScrubList,
+                                              &scrubListSize));
 
     portSyncMutexAcquire(pScrubber->pScrubberMutex);
-    for (iter = 0; iter < pageCount; iter++)
+
+    for (iter = 0; iter < scrubListSize; iter++)
     {
-        _waitForPayload(pScrubber, pPages[iter], (pPages[iter] + chunkSize - 1));
+        _waitForPayload(pScrubber,
+                        pScrubList[iter].base,
+                        (pScrubList[iter].base + pScrubList[iter].size - 1));
     }
     portSyncMutexRelease(pScrubber->pScrubberMutex);
+
+    if (pScrubList != NULL)
+    {
+        portMemFree(pScrubList);
+        pScrubList = NULL;
+    }
+
     return status;
 
 }
@@ -612,36 +692,36 @@ _scrubCopyListItems
 /*  This function is used to check and submit work items always within the
  *  available / maximum scrub list size.
  *
- *  @param[in]  pScrubber    OBJMEMSCRUB pointer
- *  @param[in]  chunkSize     size of each page
- *  @param[in]  pPages       Array of base address
- *  @param[in]  pageCount    number of pages in the array
- *  @param[in]  pList        pointer will store the return check array
+ *  @param[in]  pScrubber           OBJMEMSCRUB pointer
+ *  @param[in]  pageCount           number of pages in the array
+ *  @param[in]  pList               pointer will store the return check array
+ *  @param[in]  pScrubListCopy      List where pages are saved
+ *  @param[in]  pagesToScrubCheck   How many pages will need to be saved
  *  @returns the number of work successfully submitted, else 0
  */
 static NvU64
 _scrubCheckAndSubmit
 (
     OBJMEMSCRUB *pScrubber,
-    NvU64        chunkSize,
-    NvU64       *pPages,
     NvU64        pageCount,
     PSCRUB_NODE  pList,
-    NvLength     pagesToScrubCheck
+    PSCRUB_NODE  pScrubListCopy,
+    NvLength     pagesToScrubCheck,
+    NvU32        flags
 )
 {
-    NvU64        iter              = 0;
-    NvU64        newId;
-    NV_STATUS    status;
+    NvU64     iter = 0;
+    NvU64     newId;
+    NV_STATUS status;
 
-    if (pList == NULL && pagesToScrubCheck != 0)
+    if (pScrubListCopy == NULL && pagesToScrubCheck != 0)
     {
         NV_PRINTF(LEVEL_ERROR,
                   "pages need to be saved off, but stash list is invalid\n");
         goto exit;
     }
 
-    _scrubWaitAndSave(pScrubber, pList, pagesToScrubCheck);
+    _scrubWaitAndSave(pScrubber, pScrubListCopy, pagesToScrubCheck);
 
     for (iter = 0; iter < pageCount; iter++)
     {
@@ -649,19 +729,19 @@ _scrubCheckAndSubmit
 
         NV_PRINTF(LEVEL_INFO,
                   "Submitting work, Id: %llx, base: %llx, size: %llx\n",
-                  newId, pPages[iter], chunkSize);
+                  newId, pList[iter].base, pList[iter].size);
 
         {
-            status =_scrubMemory(pScrubber, pPages[iter], chunkSize, NV_MEMORY_DEFAULT,
-                                 (NvU32)newId);
+            status =_scrubMemory(pScrubber, pList[iter].base, pList[iter].size, NV_MEMORY_DEFAULT,
+                                 (NvU32)newId, flags);
         }
 
         if(status != NV_OK)
         {
-            NV_PRINTF(LEVEL_ERROR, "Failing because the work dint submit.\n");
+            NV_PRINTF(LEVEL_ERROR, "Failing because the work didn't submit.\n");
             goto exit;
         }
-        _scrubAddWorkToList(pScrubber, pPages[iter], chunkSize, newId);
+        _scrubAddWorkToList(pScrubber, pList[iter].base, pList[iter].size, newId);
         _scrubCheckProgress(pScrubber);
     }
 
@@ -750,7 +830,10 @@ _scrubWaitAndSave
     while (currentCompletedId < (pScrubber->lastSeenIdByClient + itemsToSave))
     {
         {
-            ceutilsServiceInterrupts(pScrubber->pCeUtils);
+            if (pScrubber->bIsEngineTypeSec2)
+                sec2utilsServiceInterrupts(pScrubber->pSec2Utils);
+            else
+                ceutilsServiceInterrupts(pScrubber->pCeUtils);
         }
         currentCompletedId = _scrubCheckProgress(pScrubber);
     }
@@ -857,9 +940,12 @@ _scrubCheckProgress
     }
     else
     {
-        lastSWSemaphoreDone = ceutilsUpdateProgress(pScrubber->pCeUtils);
+        if (pScrubber->bIsEngineTypeSec2)
+            lastSWSemaphoreDone = sec2utilsUpdateProgress(pScrubber->pSec2Utils);
+        else
+            lastSWSemaphoreDone = ceutilsUpdateProgress(pScrubber->pCeUtils);
     }
-    
+
     pScrubber->lastSWSemaphoreDone = lastSWSemaphoreDone;
 
     return lastSWSemaphoreDone;
@@ -877,29 +963,82 @@ _scrubMemory
     RmPhysAddr   base,
     NvU64        size,
     NvU32        dstCpuCacheAttrib,
-    NvU32        payload
+    NvU32        payload,
+    NvU32        flags
 )
 {
     NV_STATUS status = NV_OK;
     MEMORY_DESCRIPTOR *pMemDesc = NULL;
-    CEUTILS_MEMSET_PARAMS memsetParams = {0};
-    
-    status = memdescCreate(&pMemDesc, pScrubber->pGpu, size, 0, NV_TRUE,
-                           ADDR_FBMEM, dstCpuCacheAttrib, MEMDESC_FLAGS_NONE);
-    NV_ASSERT_OR_RETURN(status == NV_OK, status);
+    NV_ASSERT_OK_OR_RETURN(memdescCreate(&pMemDesc, pScrubber->pGpu, size, 0, NV_TRUE,
+                                          ADDR_FBMEM, dstCpuCacheAttrib, MEMDESC_FLAGS_NONE));
 
     memdescDescribe(pMemDesc, ADDR_FBMEM, base, size);
 
-    memsetParams.pMemDesc = pMemDesc;
-    memsetParams.length = size;
-    memsetParams.flags = NV0050_CTRL_MEMSET_FLAGS_ASYNC | NV0050_CTRL_MEMSET_FLAGS_PIPELINED;
-
-    status = ceutilsMemset(pScrubber->pCeUtils, &memsetParams);
-    if (status == NV_OK)
+    if ((flags & SCRUBBER_SUBMIT_FLAGS_LOCALIZED_SCRUB) != 0)
     {
+        memdescSetFlag(pMemDesc, MEMDESC_FLAGS_ALLOC_AS_LOCALIZED, NV_TRUE);
+    }
+
+    if (pScrubber->bIsEngineTypeSec2)
+    {
+        SEC2UTILS_MEMSET_PARAMS memsetParams = {0};
+        memsetParams.pMemDesc = pMemDesc;
+        memsetParams.length = size;
+
+        NV_ASSERT_OK_OR_GOTO(status, sec2utilsMemset(pScrubber->pSec2Utils, &memsetParams), cleanup);
+        pScrubber->lastSubmittedWorkId = memsetParams.submittedWorkId;
+    }
+    else
+    {
+        CEUTILS_MEMSET_PARAMS memsetParams = {0};
+        memsetParams.pMemDesc = pMemDesc;
+        memsetParams.length = size;
+        memsetParams.flags = NV0050_CTRL_MEMSET_FLAGS_ASYNC | NV0050_CTRL_MEMSET_FLAGS_PIPELINED;
+
+        NV_ASSERT_OK_OR_GOTO(status, ceutilsMemset(pScrubber->pCeUtils, &memsetParams), cleanup);
         pScrubber->lastSubmittedWorkId = memsetParams.submittedWorkId;
     }
 
+cleanup:
     memdescDestroy(pMemDesc);
     return status;
 }
+
+static NV_STATUS
+_scrubCombinePages
+(
+    NvU64       *pPages,
+    NvU64        pageSize,
+    NvU64        pageCount,
+    PSCRUB_NODE *ppScrubList,
+    NvU64       *pSize
+)
+{
+    NvU64 i, j;
+
+    *ppScrubList = (PSCRUB_NODE)portMemAllocNonPaged(sizeof(SCRUB_NODE) * pageCount);
+    NV_ASSERT_OR_RETURN(*ppScrubList != NULL, NV_ERR_NO_MEMORY);
+
+    // Copy first element from original list to new list
+    (*ppScrubList)[0].base = pPages[0];
+    (*ppScrubList)[0].size = pageSize;
+
+    for (i = 0, j = 0; i < (pageCount - 1); i++)
+    {
+        if ((((*ppScrubList)[j].size + pageSize) > SCRUB_MAX_BYTES_PER_LINE) ||
+            ((pPages[i] + pageSize) != pPages[i+1]))
+        {
+            j++;
+            (*ppScrubList)[j].base = pPages[i+1];
+            (*ppScrubList)[j].size = pageSize;
+        }
+        else
+        {
+            (*ppScrubList)[j].size += pageSize;
+        }
+    }
+
+    *pSize = j + 1;
+
+    return NV_OK;
+}
