@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2015-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2015-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -31,30 +31,7 @@
 // These files are not found on SRT builds
 #include "os/os.h"
 #else
-static NvU64 osGetPageRefcount(NvU64 sysPagePhysAddr)
-{
-    return 0;
-}
-
-static NvU64 osCountTailPages(NvU64 sysPagePhysAddr)
-{
-    return 0;
-}
-
-static void osAllocReleasePage(NvU64 sysPagePhysAddr, NvU32 pageCount)
-{
-    return;
-}
-
-static NV_STATUS osOfflinePageAtAddress(NvU64 address)
-{
-    return NV_ERR_GENERIC;
-}
-
-static NvU8 osGetPageShift(void)
-{
-    return 0;
-}
+#include "pma_test_stubs.h"
 
 NV_STATUS scrubCheck(OBJMEMSCRUB *pScrubber, PSCRUB_NODE *ppList, NvU64 *size)
 {
@@ -62,7 +39,7 @@ NV_STATUS scrubCheck(OBJMEMSCRUB *pScrubber, PSCRUB_NODE *ppList, NvU64 *size)
 }
 
 NV_STATUS scrubSubmitPages(OBJMEMSCRUB *pScrubber, NvU64 chunkSize, NvU64* pages,
-                           NvU64 pageCount, PSCRUB_NODE *ppList, NvU64 *size)
+                           NvU64 pageCount, PSCRUB_NODE *ppList, NvU64 *size, NvU32 flags)
 {
     return NV_ERR_GENERIC;
 }
@@ -486,9 +463,12 @@ _pmaEvictContiguous
             // and hence there will be no page stealing.
             //
             NvU64 count;
+            NvU32 flags = 0;
+
+            // Localized not supported on NUMA yet
 
             if ((status = scrubSubmitPages(pPma->pScrubObj, (NvU32)evictSize, &evictStart,
-                                           1, &pPmaScrubList, &count)) != NV_OK)
+                                           1, &pPmaScrubList, &count, flags)) != NV_OK)
             {
                 status = NV_ERR_INSUFFICIENT_RESOURCES;
                 goto scrub_exit;
@@ -600,9 +580,33 @@ _pmaEvictPages
             goto evict_cleanup;
         }
 
+        NvU32 flags = 0;
+
+        // Localized memory scrub
+        PMA_PAGESTATUS state;
+        NvU32 regId;
+        NvU64 frameNum, addrBase;
+
+        //
+        // Optimization: the first one is expected to be the same as the rest of them.
+        // If no localized pages, then don't spend time checking through the
+        // rest of them
+        //
+        if (evictPageCount > 0)
+        {
+            regId = findRegionID(pPma, evictPages[0]);
+            addrBase = pPma->pRegDescriptors[regId]->base;
+            frameNum = PMA_ADDR2FRAME(evictPages[0], addrBase);
+            state = pPma->pMapInfo->pmaMapRead(pPma->pRegions[regId], frameNum, NV_TRUE);
+            if ((state & ATTRIB_LOCALIZED) != 0)
+            {
+                flags |= SCRUBBER_SUBMIT_FLAGS_LOCALIZED_SCRUB;
+            }
+        }
+
         // Don't need to mark ATTRIB_SCRUBBING to protect the pages because they are already pinned
         status = scrubSubmitPages(pPma->pScrubObj, pageSize, evictPages,
-                                  (NvU32)evictPageCount, &pPmaScrubList, &count);
+                                  (NvU32)evictPageCount, &pPmaScrubList, &count, flags);
         NV_ASSERT_OR_GOTO((status == NV_OK), scrub_exit);
 
         if (count > 0)
@@ -1239,7 +1243,8 @@ pmaRegisterBlacklistInfo
     PMA                    *pPma,
     NvU64                   physAddrBase,
     PMA_BLACKLIST_ADDRESS  *pBlacklistPageBase,
-    NvU32                   blacklistCount
+    NvU32                   blacklistCount,
+    NvBool                  bBlacklistFromInforom
 )
 {
     NvU32 i;
@@ -1287,24 +1292,34 @@ pmaRegisterBlacklistInfo
         // This is only needed for NUMA systems that auto online NUMA memory.
         // Other systems (e.g., P9) already do blacklisting in nvidia-persistenced.
         //
-        // Page blacklisting is done regardless of whether it will also be done by
-        // CPU RAS_FW via CPER handling (which is done in non-vGPU cases). 
-        // This is fine, because multiple simultaneous calls to page blacklisting
-        // API memory_failure() do not cause any issues.
-        //
-        if (pPma->bNuma && pPma->bNumaAutoOnline)
+        if (pPma->bNuma && pPma->bNumaAutoOnline && pPma->nodeOnlined)
         {
-            NV_STATUS status;
-
-            NV_PRINTF(LEVEL_INFO,
-                      "NUMA enabled - blacklisting page through kernel at address 0x%llx (GPA) 0x%llx (SPA)\n",
-                      pBlacklistPageBase[blacklistEntryIn].physOffset,
-                      pBlacklistPageBase[blacklistEntryIn].physOffset + pPma->coherentCpuFbBase);
-
-            status = osOfflinePageAtAddress(pBlacklistPageBase[blacklistEntryIn].physOffset + pPma->coherentCpuFbBase);
-            if (status != NV_OK)
+            //
+            // Only blacklist pages from inforom (i.e., during heap/PMA init) need 
+            // to be blacklisted with kernel here. The blacklist pages stored in 
+            // inforom need to remain blacklisted persistently across GPU resets -
+            // kernel won't automatically blacklist these so RM must do it
+            // explicitly here.
+            //
+            // Blacklist pages not from inforom (i.e., from ECC interrupt handling)
+            // do not need to be blacklisted with kernel. This is because the ECC
+            // interrupt will automatically trigger kernel itself to blacklist the page.
+            //
+            if (bBlacklistFromInforom)
             {
-                NV_PRINTF(LEVEL_ERROR, "osOfflinePageAtAddress() failed with status: %d\n", status);
+                NV_STATUS status;
+
+                // Use physOffset without 64K alignment, because kernel may use a different page size.
+                NV_PRINTF(LEVEL_INFO,
+                          "NUMA enabled - blacklisting page through kernel at address 0x%llx (GPA) 0x%llx (SPA)\n",
+                          pBlacklistPageBase[blacklistEntryIn].physOffset,
+                          pBlacklistPageBase[blacklistEntryIn].physOffset + pPma->coherentCpuFbBase);
+
+                status = osOfflinePageAtAddress(pBlacklistPageBase[blacklistEntryIn].physOffset + pPma->coherentCpuFbBase);
+                if (status != NV_OK)
+                {
+                    NV_PRINTF(LEVEL_ERROR, "osOfflinePageAtAddress() failed with status: %d\n", status);
+                }
             }
         }
 
