@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2013-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2013-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -21,6 +21,8 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
+#define NVOC_KERN_GMMU_H_PRIVATE_ACCESS_ALLOWED
+
 #include "mmu/gmmu_fmt.h"
 #include "gpu/mmu/kern_gmmu.h"
 
@@ -124,13 +126,7 @@ _gmmuGetPdePa
 
         if (pAddrFld)
         {
-            KernelGmmu *pKernelGmmu = GPU_GET_KERNEL_GMMU(pGpu);
-            NvU64    addr  = gmmuFieldGetAddress(pAddrFld, pGmmuEntry->v8);
-            if ((aperture == GMMU_APERTURE_SYS_NONCOH) || (aperture == GMMU_APERTURE_SYS_COH))
-            {
-                addr += pKernelGmmu->sysmemBaseAddress;
-            }
-            return addr;
+            return gmmuFieldGetAddress(pAddrFld, pGmmuEntry->v8);
         }
     }
 
@@ -155,13 +151,7 @@ _gmmuGetPtePa
         const GMMU_FIELD_ADDRESS *pAddrFld = gmmuFmtPtePhysAddrFld(pFmt, aperture);
         if (pAddrFld)
         {
-            KernelGmmu *pKernelGmmu = GPU_GET_KERNEL_GMMU(pGpu);
-            NvU64    addr  = gmmuFieldGetAddress(pAddrFld, pGmmuEntry->v8);
-            if ((aperture == GMMU_APERTURE_SYS_NONCOH) || (aperture == GMMU_APERTURE_SYS_COH))
-            {
-                addr += pKernelGmmu->sysmemBaseAddress;
-            }
-            return addr;
+            return gmmuFieldGetAddress(pAddrFld, pGmmuEntry->v8);
         }
     }
 
@@ -177,7 +167,11 @@ _gmmuPrintPa
     NvU32         peerIndex
 )
 {
+#if NV_PRINTF_STRINGS_ALLOWED
     NV_PRINTF_EX(NV_PRINTF_MODULE, LEVEL_INFO, "%s", _decodeAperture[aperture]);
+#else
+    NV_PRINTF_EX(NV_PRINTF_MODULE, LEVEL_INFO, "%c", _decodeAperture[aperture]);
+#endif
     if (GMMU_APERTURE_PEER == aperture)
     {
         NV_PRINTF_EX(NV_PRINTF_MODULE, LEVEL_INFO, "[%d]", peerIndex);
@@ -280,7 +274,8 @@ _gmmuPrintPde
 
         PRINT_FIELD_32("PdePcf=%d", pFmtPde, PdePcf, pGmmuEntry);
         pdePcfHw = nvFieldGet32(&pFmtPde->fldPdePcf, pGmmuEntry->v8);
-        NV_ASSERT(kgmmuTranslatePdePcfFromHw_HAL(GPU_GET_KERNEL_GMMU(pGpu),
+        NV_CHECK(LEVEL_INFO,
+                 kgmmuTranslatePdePcfFromHw_HAL(GPU_GET_KERNEL_GMMU(pGpu),
                                                 pdePcfHw,
                                                 aperture,
                                                 &pdePcfSw) == NV_OK);
@@ -365,6 +360,9 @@ _gmmuPrintPte
 
     switch (mmuFmtLevelPageSize(pFmtLevel))
     {
+        case RM_PAGE_SIZE_256G:
+            NV_PRINTF_EX(NV_PRINTF_MODULE, LEVEL_INFO, "PTE_256G");
+            break;
         case RM_PAGE_SIZE_512M:
             NV_PRINTF_EX(NV_PRINTF_MODULE, LEVEL_INFO, "PTE_512M");
             break;
