@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2013-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2013-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -24,10 +24,12 @@
 #include "gpu/gpu.h"
 #include "gpu/mem_mgr/mem_mgr.h"
 #include "mem_mgr/gpu_vaspace.h"
+#include "mem_mgr/fabric_vaspace.h"
 #include "gpu/mmu/kern_gmmu.h"
 #include "kernel/gpu/nvlink/kernel_nvlink.h"
+#include "kernel/gpu/fifo/kernel_fifo.h"
 #include "gpu/mem_mgr/mem_desc.h"
-#include "nvRmReg.h"  // NV_REG_STR_RM_*
+#include "nvrm_registry.h"  // NV_REG_STR_RM_*
 
 #include "mmu/gmmu_fmt.h"
 #include "mmu/mmu_fmt.h"
@@ -285,7 +287,7 @@ _gmmuWalkCBLevelAlloc
 
     // Add memList end entry.
     memPoolList[memPoolListCount++] = ADDR_UNKNOWN;
-    NV_ASSERT(memPoolListCount <= NV_ARRAY_ELEMENTS32(memPoolList));
+    NV_ASSERT(memPoolListCount <= NV_ARRAY_ELEMENTS(memPoolList));
 
     // MEMDESC flags
     memDescFlags = MEMDESC_FLAGS_LOCKLESS_SYSMEM_ALLOC  |
@@ -326,16 +328,19 @@ _gmmuWalkCBLevelAlloc
             {
                 case ADDR_FBMEM:
                     if (RMCFG_FEATURE_PMA &&
-                        (pGVAS->flags & VASPACE_FLAGS_PTETABLE_PMA_MANAGED) &&
-                        (pGVAS->pPageTableMemPool != NULL))
+                        (pGVAS->flags & VASPACE_FLAGS_PTETABLE_PMA_MANAGED))
                     {
+                        NV_ASSERT_OR_RETURN(pUserCtx->pGpuState->pPageTableMemPool != NULL,
+                                            NV_ERR_INVALID_STATE);
+
                         pMemDescTemp->ActualSize = RM_ALIGN_UP(newMemSize, alignment);
-                        status = rmMemPoolAllocate(pGVAS->pPageTableMemPool,
+                        status = rmMemPoolAllocate(pUserCtx->pGpuState->pPageTableMemPool,
                                          (RM_POOL_ALLOC_MEMDESC*)pMemDescTemp);
                         break;
                     }
                 case ADDR_SYSMEM:
-                    status = memdescAlloc(pMemDescTemp);
+                    memdescTagAlloc(status, NV_FB_ALLOC_RM_INTERNAL_OWNER_UNNAMED_TAG_143,
+                                    pMemDescTemp);
                     break;
                 default:
                     NV_ASSERT_OR_GOTO(0, done);
@@ -410,10 +415,9 @@ _gmmuWalkCBLevelAlloc
                   mmuFmtLevelVirtAddrHi(pLevelFmt, vaLimit));
 #else // NV_PRINTF_STRINGS_ALLOWED
         NV_PRINTF(LEVEL_INFO,
-                  "[GPU%u]:  [Packed: %c] %sPA 0x%llX (0x%X bytes) for VA 0x%llX-0x%llX\n",
+                  "[GPU%u]:  [Packed: %c] PA 0x%llX (0x%X bytes) for VA 0x%llX-0x%llX\n",
                   pUserCtx->pGpu->gpuInstance,
                   bPacked ? 'Y' : 'N',
-                  bMirror ? _gmmuUVMMirroringDirString[i] : ' ',
                   memdescGetPhysAddr(pMemDesc[i], AT_GPU, 0), newMemSize,
                   mmuFmtLevelVirtAddrLo(pLevelFmt, vaBase),
                   mmuFmtLevelVirtAddrHi(pLevelFmt, vaLimit));
@@ -607,13 +611,11 @@ _gmmuWalkCBLevelFree
         }
         else
         {
-            if (RMCFG_FEATURE_PMA &&
-                (pUserCtx->pGVAS->flags & VASPACE_FLAGS_PTETABLE_PMA_MANAGED) &&
-                (pMemDesc[i]->pPageHandleList != NULL) &&
+            if ((pMemDesc[i]->pPageHandleList != NULL) &&
                 (listCount(pMemDesc[i]->pPageHandleList) != 0) &&
-                (pUserCtx->pGVAS->pPageTableMemPool != NULL))
+                (pUserCtx->pGpuState->pPageTableMemPool != NULL))
             {
-                rmMemPoolFree(pUserCtx->pGVAS->pPageTableMemPool,
+                rmMemPoolFree(pUserCtx->pGpuState->pPageTableMemPool,
                               (RM_POOL_ALLOC_MEMDESC*)pMemDesc[i],
                               pUserCtx->pGVAS->flags);
             }
@@ -687,8 +689,23 @@ _gmmuWalkCBUpdatePde
     KernelGmmu        *pKernelGmmu = GPU_GET_KERNEL_GMMU(pGpu);
     const GMMU_FMT    *pFmt        = pUserCtx->pGpuState->pFmt;
     MEMORY_DESCRIPTOR *pMemDesc[GMMU_MAX_PAGE_DIR_INDEX_COUNT] = {NULL};
-    NvU32                      recipExp  = NV_U32_MAX;
-    const GMMU_FMT_PDE_MULTI  *pPdeMulti = pFmt->pPdeMulti;
+    NvU32                      recipExp      = NV_U32_MAX;
+    const GMMU_FMT_PDE_MULTI  *pPdeMulti     = pFmt->pPdeMulti;
+    NvU32                      transferFlags = TRANSFER_FLAGS_NONE;
+
+    // FABRIC_VASPACE object of pGpu.
+    FABRIC_VASPACE *pFabricVAS = (pGpu->pFabricVAS != NULL) ? (dynamicCast(pGpu->pFabricVAS, FABRIC_VASPACE)) : NULL;
+
+    // GVASPACE object associated with this fabric vaspace.
+    OBJGVASPACE *pGVAS_FLA = (pFabricVAS != NULL) ? (dynamicCast(pFabricVAS->pGVAS, OBJGVASPACE)) : NULL;
+
+    // Apply the WAR to flush CPU cache if the VA space is of BAR1/FLA.
+    if (((pUserCtx->pGVAS->flags & VASPACE_FLAGS_BAR_BAR1) ||
+         (pGVAS == pGVAS_FLA)) &&
+        pKernelGmmu->bBug4686457WAR)
+    {
+        transferFlags |= TRANSFER_FLAGS_FLUSH_CPU_CACHE_WAR_BUG4686457;
+    }
 
     pMemDesc[GMMU_USER_PAGE_DIR_INDEX] = (MEMORY_DESCRIPTOR*)pLevelMem;
     if (bMirror)
@@ -795,7 +812,7 @@ _gmmuWalkCBUpdatePde
         dest.offset = entryIndex * pLevelFmt->entrySize;
         NV_ASSERT_OK(memmgrMemWrite(GPU_GET_MEMORY_MANAGER(pGpu), &dest,
                                     entry.v8, pLevelFmt->entrySize,
-                                    TRANSFER_FLAGS_NONE));
+                                    transferFlags));
     }
 
     return NV_TRUE;
@@ -826,6 +843,7 @@ _gmmuWalkCBFillEntries
     NvU32              sizeOfEntries = (entryIndexHi - entryIndexLo + 1) *
                                         pLevelFmt->entrySize;
     NvU8              *pEntries;
+    NvU32              transferFlags = TRANSFER_FLAGS_SHADOW_ALLOC;
 
     pMemDesc[GMMU_USER_PAGE_DIR_INDEX] = (MEMORY_DESCRIPTOR*)pLevelMem;
     if (bMirror)
@@ -834,6 +852,18 @@ _gmmuWalkCBFillEntries
             (MEMORY_DESCRIPTOR*)pUserCtx->pGpuState->pMirroredRoot;
     }
 
+    // FABRIC_VASPACE object of pGpu.
+    FABRIC_VASPACE *pFabricVAS = (pGpu->pFabricVAS != NULL) ? (dynamicCast(pGpu->pFabricVAS, FABRIC_VASPACE)) : NULL;
+    // GVASPACE object associated with this fabric vaspace.
+    OBJGVASPACE *pGVAS_FLA = (pFabricVAS != NULL) ? (dynamicCast(pFabricVAS->pGVAS, OBJGVASPACE)) : NULL;
+    // Apply the WAR to flush CPU cache if the VA space is of BAR1/FLA.
+    if (((pUserCtx->pGVAS->flags & VASPACE_FLAGS_BAR_BAR1) ||
+         (pUserCtx->pGVAS == pGVAS_FLA)) &&
+        pKernelGmmu->bBug4686457WAR)
+    {
+        transferFlags |= TRANSFER_FLAGS_FLUSH_CPU_CACHE_WAR_BUG4686457;
+    }
+
     for (j = 0; j < maxPgDirs; j++)
     {
         TRANSFER_SURFACE dest = {0};
@@ -847,7 +877,7 @@ _gmmuWalkCBFillEntries
         // path on Windows and shadow buffer allocation may fail there.
         //
         pEntries = memmgrMemBeginTransfer(pMemoryManager, &dest, sizeOfEntries,
-                                          TRANSFER_FLAGS_SHADOW_ALLOC);
+                                          transferFlags);
         NV_ASSERT_OR_RETURN_VOID(pEntries != NULL);
 
 #if NV_PRINTF_STRINGS_ALLOWED
@@ -877,23 +907,32 @@ _gmmuWalkCBFillEntries
             {
                 const GMMU_FMT_FAMILY  *pFam = kgmmuFmtGetFamily(pKernelGmmu, pFmt->version);
                 const GMMU_ENTRY_VALUE *pSparseEntry;
+                // Fake sparse entry is needed for GH100 in CC mode for PDE2-PDE4. Ref: Bug 3341692
+                NvU8 *pFakeSparse = kgmmuGetFakeSparseEntry_HAL(pGpu, pKernelGmmu, pLevelFmt);
 
-                // Select sparse entry template based on number of sub-levels.
-                if (pLevelFmt->numSubLevels > 1)
+                if (pFakeSparse != NULL)
                 {
-                    pSparseEntry = &pFam->sparsePdeMulti;
-                }
-                else if (pLevelFmt->numSubLevels == 1)
-                {
-                    pSparseEntry = &pFam->sparsePde;
+                    pSparseEntry = (const GMMU_ENTRY_VALUE *) pFakeSparse;
                 }
                 else
                 {
-                    if (kbusIsFlaDummyPageEnabled(pKernelBus) &&
-                        (pUserCtx->pGVAS->flags & VASPACE_FLAGS_FLA))
-                        pSparseEntry = &pUserCtx->pGpuState->flaDummyPage.pte;
+                    // Select sparse entry template based on number of sub-levels.
+                    if (pLevelFmt->numSubLevels > 1)
+                    {
+                        pSparseEntry = &pFam->sparsePdeMulti;
+                    }
+                    else if (pLevelFmt->numSubLevels == 1)
+                    {
+                        pSparseEntry = &pFam->sparsePde;
+                    }
                     else
-                        pSparseEntry = &pFam->sparsePte;
+                    {
+                        if (kbusIsFlaDummyPageEnabled(pKernelBus) &&
+                            (pUserCtx->pGVAS->flags & VASPACE_FLAGS_FLA))
+                            pSparseEntry = &pUserCtx->pGpuState->flaDummyPage.pte;
+                        else
+                            pSparseEntry = &pFam->sparsePte;
+                    }
                 }
 
                 // Copy sparse template to each entry.
@@ -956,7 +995,7 @@ _gmmuWalkCBFillEntries
         }
 
         memmgrMemEndTransfer(pMemoryManager, &dest, sizeOfEntries,
-                             TRANSFER_FLAGS_SHADOW_ALLOC);
+                             transferFlags);
     }
 
     *pProgress = entryIndexHi - entryIndexLo + 1;
@@ -974,11 +1013,27 @@ _gmmuWalkCBCopyEntries
     NvU32                     *pProgress
 )
 {
-    MEMORY_DESCRIPTOR *pSrcDesc = (MEMORY_DESCRIPTOR *)pSrcMem;
-    MEMORY_DESCRIPTOR *pDstDesc = (MEMORY_DESCRIPTOR *)pDstMem;
-    TRANSFER_SURFACE   src      = {0};
-    TRANSFER_SURFACE   dest     = {0};
+    OBJGPU            *pGpu          = pUserCtx->pGpu;
+    KernelGmmu        *pKernelGmmu   = GPU_GET_KERNEL_GMMU(pGpu);
+    MEMORY_DESCRIPTOR *pSrcDesc      = (MEMORY_DESCRIPTOR *)pSrcMem;
+    MEMORY_DESCRIPTOR *pDstDesc      = (MEMORY_DESCRIPTOR *)pDstMem;
+    TRANSFER_SURFACE   src           = {0};
+    TRANSFER_SURFACE   dest          = {0};
+    NvU32              transferFlags = TRANSFER_FLAGS_NONE;
 
+    // FABRIC_VASPACE object of pGpu.
+    FABRIC_VASPACE *pFabricVAS = (pGpu->pFabricVAS != NULL) ? (dynamicCast(pGpu->pFabricVAS, FABRIC_VASPACE)) : NULL;
+
+    // GVASPACE object associated with this fabric vaspace.
+    OBJGVASPACE *pGVAS_FLA = (pFabricVAS != NULL) ? (dynamicCast(pFabricVAS->pGVAS, OBJGVASPACE)) : NULL;
+
+    // Apply the WAR to flush CPU cache if the VA space is of BAR1/FLA
+    if (((pUserCtx->pGVAS->flags & VASPACE_FLAGS_BAR_BAR1) ||
+         (pUserCtx->pGVAS == pGVAS_FLA)) &&
+        pKernelGmmu->bBug4686457WAR)
+    {
+        transferFlags |= TRANSFER_FLAGS_FLUSH_CPU_CACHE_WAR_BUG4686457;
+    }
     src.pMemDesc = pSrcDesc;
     src.offset = entryIndexLo * pLevelFmt->entrySize;
     dest.pMemDesc = pDstDesc;
@@ -999,7 +1054,7 @@ _gmmuWalkCBCopyEntries
                   entryIndexHi);
 
         NV_ASSERT_OK(memmgrMemCopy(GPU_GET_MEMORY_MANAGER(pGpu), &dest, &src,
-                                   sizeOfEntries, TRANSFER_FLAGS_NONE));
+                                   sizeOfEntries, transferFlags));
     }
 
     // Report full range complete.
