@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2020-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2020-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -21,11 +21,13 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
+#define NVOC_KERNEL_NVLINK_H_PRIVATE_ACCESS_ALLOWED
+
 #include "os/os.h"
 #include "core/hal.h"
-#include "core/info_block.h"
 #include "core/locks.h"
 #include "core/thread_state.h"
+#include "gpu_mgr/gpu_mgr.h"
 #include "gpu/gpu.h"
 
 #include "kernel/gpu/nvlink/kernel_nvlink.h"
@@ -36,73 +38,73 @@
 static void _knvlinkCorePassiveLinkChangeCallback(NvU32, void *);
 
 /*!
- * Compile time assert to ensure NV2080_CTRL_NVLINK_MAX_SEED_BUFFER_SIZE ==
+ * Compile time assert to ensure NV2080_CTRL_INTERNAL_NVLINK_MAX_SEED_BUFFER_SIZE ==
  * NVLINK_MAX_SEED_BUFFER_SIZE
  */
-ct_assert(NV2080_CTRL_NVLINK_MAX_SEED_BUFFER_SIZE ==
+ct_assert(NV2080_CTRL_INTERNAL_NVLINK_MAX_SEED_BUFFER_SIZE ==
           NVLINK_MAX_SEED_BUFFER_SIZE);
 
 /*!
- * Compile time asserts to ensure NV2080_NVLINK_CORE_LINK_STATE* ==
+ * Compile time asserts to ensure NV2080_INTERNAL_NVLINK_CORE_LINK_STATE* ==
  * NVLINK_LINKSTATE*
  */
-ct_assert(NV2080_NVLINK_CORE_LINK_STATE_OFF ==
+ct_assert(NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_OFF ==
           NVLINK_LINKSTATE_OFF);
-ct_assert(NV2080_NVLINK_CORE_LINK_STATE_HS ==
+ct_assert(NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_HS ==
           NVLINK_LINKSTATE_HS);
-ct_assert(NV2080_NVLINK_CORE_LINK_STATE_SAFE ==
+ct_assert(NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_SAFE ==
           NVLINK_LINKSTATE_SAFE);
-ct_assert(NV2080_NVLINK_CORE_LINK_STATE_FAULT ==
+ct_assert(NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_FAULT ==
           NVLINK_LINKSTATE_FAULT);
-ct_assert(NV2080_NVLINK_CORE_LINK_STATE_RECOVERY ==
+ct_assert(NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_RECOVERY ==
           NVLINK_LINKSTATE_RECOVERY);
-ct_assert(NV2080_NVLINK_CORE_LINK_STATE_FAIL ==
+ct_assert(NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_FAIL ==
           NVLINK_LINKSTATE_FAIL);
-ct_assert(NV2080_NVLINK_CORE_LINK_STATE_DETECT ==
+ct_assert(NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_DETECT ==
           NVLINK_LINKSTATE_DETECT);
-ct_assert(NV2080_NVLINK_CORE_LINK_STATE_RESET ==
+ct_assert(NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_RESET ==
           NVLINK_LINKSTATE_RESET);
-ct_assert(NV2080_NVLINK_CORE_LINK_STATE_ENABLE_PM ==
+ct_assert(NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_ENABLE_PM ==
           NVLINK_LINKSTATE_ENABLE_PM);
-ct_assert(NV2080_NVLINK_CORE_LINK_STATE_DISABLE_PM ==
+ct_assert(NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_DISABLE_PM ==
           NVLINK_LINKSTATE_DISABLE_PM);
-ct_assert(NV2080_NVLINK_CORE_LINK_STATE_SLEEP ==
+ct_assert(NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_SLEEP ==
           NVLINK_LINKSTATE_SLEEP);
-ct_assert(NV2080_NVLINK_CORE_LINK_STATE_SAVE_STATE ==
+ct_assert(NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_SAVE_STATE ==
           NVLINK_LINKSTATE_SAVE_STATE);
-ct_assert(NV2080_NVLINK_CORE_LINK_STATE_RESTORE_STATE ==
+ct_assert(NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_RESTORE_STATE ==
           NVLINK_LINKSTATE_RESTORE_STATE);
-ct_assert(NV2080_NVLINK_CORE_LINK_STATE_PRE_HS ==
+ct_assert(NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_PRE_HS ==
           NVLINK_LINKSTATE_PRE_HS);
-ct_assert(NV2080_NVLINK_CORE_LINK_STATE_DISABLE_ERR_DETECT ==
+ct_assert(NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_DISABLE_ERR_DETECT ==
           NVLINK_LINKSTATE_DISABLE_ERR_DETECT);
-ct_assert(NV2080_NVLINK_CORE_LINK_STATE_LANE_DISABLE ==
+ct_assert(NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_LANE_DISABLE ==
           NVLINK_LINKSTATE_LANE_DISABLE);
-ct_assert(NV2080_NVLINK_CORE_LINK_STATE_LANE_SHUTDOWN ==
+ct_assert(NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_LANE_SHUTDOWN ==
           NVLINK_LINKSTATE_LANE_SHUTDOWN);
-ct_assert(NV2080_NVLINK_CORE_LINK_STATE_TRAFFIC_SETUP ==
+ct_assert(NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_TRAFFIC_SETUP ==
           NVLINK_LINKSTATE_TRAFFIC_SETUP);
-ct_assert(NV2080_NVLINK_CORE_LINK_STATE_INITPHASE1 ==
+ct_assert(NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_INITPHASE1 ==
           NVLINK_LINKSTATE_INITPHASE1);
-ct_assert(NV2080_NVLINK_CORE_LINK_STATE_INITNEGOTIATE ==
+ct_assert(NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_INITNEGOTIATE ==
           NVLINK_LINKSTATE_INITNEGOTIATE);
-ct_assert(NV2080_NVLINK_CORE_LINK_STATE_POST_INITNEGOTIATE ==
+ct_assert(NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_POST_INITNEGOTIATE ==
           NVLINK_LINKSTATE_POST_INITNEGOTIATE);
-ct_assert(NV2080_NVLINK_CORE_LINK_STATE_INITOPTIMIZE ==
+ct_assert(NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_INITOPTIMIZE ==
           NVLINK_LINKSTATE_INITOPTIMIZE);
-ct_assert(NV2080_NVLINK_CORE_LINK_STATE_POST_INITOPTIMIZE ==
+ct_assert(NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_POST_INITOPTIMIZE ==
           NVLINK_LINKSTATE_POST_INITOPTIMIZE);
-ct_assert(NV2080_NVLINK_CORE_LINK_STATE_DISABLE_HEARTBEAT ==
+ct_assert(NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_DISABLE_HEARTBEAT ==
           NVLINK_LINKSTATE_DISABLE_HEARTBEAT);
-ct_assert(NV2080_NVLINK_CORE_LINK_STATE_CONTAIN ==
+ct_assert(NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_CONTAIN ==
           NVLINK_LINKSTATE_CONTAIN);
-ct_assert(NV2080_NVLINK_CORE_LINK_STATE_INITTL ==
+ct_assert(NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_INITTL ==
           NVLINK_LINKSTATE_INITTL);
-ct_assert(NV2080_NVLINK_CORE_LINK_STATE_INITPHASE5 ==
+ct_assert(NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_INITPHASE5 ==
           NVLINK_LINKSTATE_INITPHASE5);
-ct_assert(NV2080_NVLINK_CORE_LINK_STATE_ALI ==
+ct_assert(NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_ALI ==
           NVLINK_LINKSTATE_ALI);
-ct_assert(NV2080_NVLINK_CORE_LINK_STATE_ACTIVE_PENDING ==
+ct_assert(NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_ACTIVE_PENDING ==
           NVLINK_LINKSTATE_ACTIVE_PENDING);
 
 /*!
@@ -435,7 +437,6 @@ knvlinkCoreQueueLinkChangeCallback
 
     KNVLINK_RM_LINK *pNvlinkLink;
     OBJGPU          *pGpu   = NULL;
-    OBJOS           *pOS    = NULL;
     NV_STATUS        status = NV_OK;
     void            *pWorkItemData;
 
@@ -450,8 +451,6 @@ knvlinkCoreQueueLinkChangeCallback
     // The master should be marked as such
     NV_ASSERT_OR_RETURN(link_change->master->master, NV_ERR_INVALID_STATE);
 
-    pOS = GPU_GET_OS(pGpu);
-
     pWorkItemData = portMemAllocNonPaged(sizeof(nvlink_link_change *));
     NV_ASSERT_OR_RETURN(pWorkItemData != NULL, NVL_NO_MEM);
 
@@ -461,8 +460,8 @@ knvlinkCoreQueueLinkChangeCallback
     // This function will free the argument if it succeeds, hence the need for
     // the work item data wrapper.
     //
-    status = pOS->osQueueWorkItem(pGpu, _knvlinkCorePassiveLinkChangeCallback,
-                                  pWorkItemData);
+    status = osQueueWorkItem(pGpu, _knvlinkCorePassiveLinkChangeCallback,
+                             pWorkItemData);
     if (status != NV_OK)
     {
         portMemFree(pWorkItemData);
@@ -493,10 +492,10 @@ knvlinkCoreSetDlLinkModeCallback
     OBJGPU           *pGpu          = NULL;
     KernelNvlink     *pKernelNvlink = NULL;
     KernelIoctrl     *pKernelIoctrl = NULL;
-    NV_STATUS         status        = NV_OK; 
+    NV_STATUS         status        = NV_OK;
     NvU8              linkIndex;
-    NV2080_CTRL_NVLINK_CORE_CALLBACK_PARAMS params;
-    NV2080_CTRL_NVLINK_CALLBACK_SET_DL_LINK_MODE_PARAMS
+    NV2080_CTRL_INTERNAL_NVLINK_CORE_CALLBACK_PARAMS params;
+    NV2080_CTRL_INTERNAL_NVLINK_CALLBACK_SET_DL_LINK_MODE_PARAMS
                     *pSetDlLinkModeParams;
 
     portMemSet(&params, 0, sizeof(params));
@@ -514,7 +513,7 @@ knvlinkCoreSetDlLinkModeCallback
     linkIndex     = pNvlinkLink->linkId;
     pKernelIoctrl = KNVLINK_LINK_GET_IOCTRL(pKernelNvlink, linkIndex);
 
-    if (pKernelIoctrl == NULL)
+    if ((pKernelNvlink->ipVerNvlink < NVLINK_VERSION_50) && (pKernelIoctrl == NULL))
         return 0;
 
     // If link training is disabled through regkey
@@ -524,7 +523,7 @@ knvlinkCoreSetDlLinkModeCallback
     }
 
     params.linkId = linkIndex;
-    params.callbackType.type = NV2080_CTRL_NVLINK_CALLBACK_TYPE_SET_DL_LINK_MODE;
+    params.callbackType.type = NV2080_CTRL_INTERNAL_NVLINK_CALLBACK_TYPE_SET_DL_LINK_MODE;
 
     pSetDlLinkModeParams = &params.callbackType.callbackParams.setDlLinkMode;
     pSetDlLinkModeParams->mode = mode;
@@ -536,14 +535,14 @@ knvlinkCoreSetDlLinkModeCallback
         case NVLINK_LINKSTATE_OFF:
         {
             pSetDlLinkModeParams->linkMode =
-                NV2080_NVLINK_CORE_LINK_STATE_OFF;
+                NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_OFF;
 
             break;
         }
         case NVLINK_LINKSTATE_PRE_HS:
         {
             pSetDlLinkModeParams->linkMode =
-                NV2080_NVLINK_CORE_LINK_STATE_PRE_HS;
+                NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_PRE_HS;
 
             pSetDlLinkModeParams->linkModeParams.linkModePreHsParams.remoteDeviceType =
                 pKernelNvlink->nvlinkLinks[linkIndex].remoteEndInfo.deviceType;
@@ -555,15 +554,15 @@ knvlinkCoreSetDlLinkModeCallback
         case NVLINK_LINKSTATE_INITPHASE1:
         {
             pSetDlLinkModeParams->linkMode =
-                NV2080_NVLINK_CORE_LINK_STATE_INITPHASE1;
+                NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_INITPHASE1;
 
-            if (pKernelIoctrl->getProperty(pKernelIoctrl, PDB_PROP_KIOCTRL_MINION_CACHE_SEEDS))
+            if ((pKernelIoctrl != NULL) && pKernelIoctrl->getProperty(pKernelIoctrl, PDB_PROP_KIOCTRL_MINION_CACHE_SEEDS))
             {
                 NvU32 *seedDataSrc  = pKernelNvlink->nvlinkLinks[linkIndex].core_link->seedData;
                 NvU32 *seedDataDest =
                     pSetDlLinkModeParams->linkModeParams.linkModeInitPhase1Params.seedData;
 
-                portMemCopy(seedDataDest, sizeof(*seedDataDest) * NV2080_CTRL_NVLINK_MAX_SEED_BUFFER_SIZE,
+                portMemCopy(seedDataDest, sizeof(*seedDataDest) * NV2080_CTRL_INTERNAL_NVLINK_MAX_SEED_BUFFER_SIZE,
                             seedDataSrc, sizeof(*seedDataSrc) * NVLINK_MAX_SEED_BUFFER_SIZE);
             }
 
@@ -572,7 +571,7 @@ knvlinkCoreSetDlLinkModeCallback
         case NVLINK_LINKSTATE_POST_INITNEGOTIATE:
         {
             pSetDlLinkModeParams->linkMode =
-                NV2080_NVLINK_CORE_LINK_STATE_POST_INITNEGOTIATE;
+                NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_POST_INITNEGOTIATE;
 
             break;
         }
@@ -620,13 +619,13 @@ knvlinkCoreSetDlLinkModeCallback
                                           (NvU64)initoptimizeTimeout / 1000);
 
             pSetDlLinkModeParams->linkMode =
-                NV2080_NVLINK_CORE_LINK_STATE_POST_INITOPTIMIZE;
+                NV2080_INTERNAL_NVLINK_CORE_LINK_STATE_POST_INITOPTIMIZE;
 
             // Poll for 10 seconds to avoid GSP timeout
             while(1)
             {
                 status = knvlinkExecGspRmRpc(pGpu, pKernelNvlink,
-                                             NV2080_CTRL_CMD_NVLINK_CORE_CALLBACK,
+                                             NV2080_CTRL_CMD_INTERNAL_NVLINK_CORE_CALLBACK,
                                              (void *)&params, sizeof(params));
                 if (status != NV_OK)
                 {
@@ -665,7 +664,7 @@ knvlinkCoreSetDlLinkModeCallback
     }
 
     status = knvlinkExecGspRmRpc(pGpu, pKernelNvlink,
-                                 NV2080_CTRL_CMD_NVLINK_CORE_CALLBACK,
+                                 NV2080_CTRL_CMD_INTERNAL_NVLINK_CORE_CALLBACK,
                                  (void *)&params, sizeof(params));
     if (status != NV_OK)
     {
@@ -690,12 +689,12 @@ knvlinkCoreSetDlLinkModeCallback
         }
         case NVLINK_LINKSTATE_OFF:
         {
-            if (pKernelIoctrl->getProperty(pKernelIoctrl, PDB_PROP_KIOCTRL_MINION_CACHE_SEEDS))
+            if ((pKernelIoctrl != NULL) && pKernelIoctrl->getProperty(pKernelIoctrl, PDB_PROP_KIOCTRL_MINION_CACHE_SEEDS))
             {
                 NvU32 *seedDataSrc  = pSetDlLinkModeParams->linkModeParams.linkModeOffParams.seedData;
                 NvU32 *seedDataDest = pKernelNvlink->nvlinkLinks[linkIndex].core_link->seedData;
 
-                portMemCopy(seedDataDest, sizeof(*seedDataDest) * NV2080_CTRL_NVLINK_MAX_SEED_BUFFER_SIZE,
+                portMemCopy(seedDataDest, sizeof(*seedDataDest) * NV2080_CTRL_INTERNAL_NVLINK_MAX_SEED_BUFFER_SIZE,
                             seedDataSrc, sizeof(*seedDataSrc) * NVLINK_MAX_SEED_BUFFER_SIZE);
             }
 
@@ -703,7 +702,7 @@ knvlinkCoreSetDlLinkModeCallback
         }
         case NVLINK_LINKSTATE_POST_INITNEGOTIATE:
         {
-            NV2080_CTRL_NVLINK_SET_DL_LINK_MODE_POST_INITNEGOTIATE_PARAMS
+            NV2080_CTRL_INTERNAL_NVLINK_SET_DL_LINK_MODE_POST_INITNEGOTIATE_PARAMS
                 *pPostInitNegotiateParams;
 
             pPostInitNegotiateParams =
@@ -752,7 +751,7 @@ knvlinkCoreGetDlLinkModeCallback
     OBJGPU          *pGpu          = NULL;
     KernelNvlink    *pKernelNvlink = NULL;
     NV_STATUS        status        = NV_OK;
-    NV2080_CTRL_NVLINK_CORE_CALLBACK_PARAMS params;
+    NV2080_CTRL_INTERNAL_NVLINK_CORE_CALLBACK_PARAMS params;
 
     portMemSet(&params, 0, sizeof(params));
 
@@ -768,10 +767,10 @@ knvlinkCoreGetDlLinkModeCallback
     pKernelNvlink = GPU_GET_KERNEL_NVLINK(pGpu);
 
     params.linkId            = pNvlinkLink->linkId;
-    params.callbackType.type = NV2080_CTRL_NVLINK_CALLBACK_TYPE_GET_DL_LINK_MODE;
+    params.callbackType.type = NV2080_CTRL_INTERNAL_NVLINK_CALLBACK_TYPE_GET_DL_LINK_MODE;
 
     status = knvlinkExecGspRmRpc(pGpu, pKernelNvlink,
-                                 NV2080_CTRL_CMD_NVLINK_CORE_CALLBACK,
+                                 NV2080_CTRL_CMD_INTERNAL_NVLINK_CORE_CALLBACK,
                                  (void *)&params, sizeof(params));
     if (status != NV_OK)
     {
@@ -804,8 +803,8 @@ knvlinkCoreSetTlLinkModeCallback
     OBJGPU          *pGpu          = NULL;
     KernelNvlink    *pKernelNvlink = NULL;
     NV_STATUS        status        = NV_OK;
-    NV2080_CTRL_NVLINK_CORE_CALLBACK_PARAMS params;
-    NV2080_CTRL_NVLINK_CALLBACK_SET_TL_LINK_MODE_PARAMS
+    NV2080_CTRL_INTERNAL_NVLINK_CORE_CALLBACK_PARAMS params;
+    NV2080_CTRL_INTERNAL_NVLINK_CALLBACK_SET_TL_LINK_MODE_PARAMS
                     *pSetTlLinkModeParams;
 
     portMemSet(&params, 0, sizeof(params));
@@ -828,7 +827,7 @@ knvlinkCoreSetTlLinkModeCallback
     }
 
     params.linkId            = pNvlinkLink->linkId;
-    params.callbackType.type = NV2080_CTRL_NVLINK_CALLBACK_TYPE_SET_TL_LINK_MODE;
+    params.callbackType.type = NV2080_CTRL_INTERNAL_NVLINK_CALLBACK_TYPE_SET_TL_LINK_MODE;
 
     pSetTlLinkModeParams        = &params.callbackType.callbackParams.setTlLinkMode;
     pSetTlLinkModeParams->mode  = mode;
@@ -836,7 +835,7 @@ knvlinkCoreSetTlLinkModeCallback
                                    NV_TRUE : NV_FALSE;
 
     status = knvlinkExecGspRmRpc(pGpu, pKernelNvlink,
-                                 NV2080_CTRL_CMD_NVLINK_CORE_CALLBACK,
+                                 NV2080_CTRL_CMD_INTERNAL_NVLINK_CORE_CALLBACK,
                                  (void *)&params, sizeof(params));
     if (status != NV_OK)
     {
@@ -866,7 +865,7 @@ knvlinkCoreGetTlLinkModeCallback
     OBJGPU          *pGpu          = NULL;
     KernelNvlink    *pKernelNvlink = NULL;
     NV_STATUS        status        = NV_OK;
-    NV2080_CTRL_NVLINK_CORE_CALLBACK_PARAMS params;
+    NV2080_CTRL_INTERNAL_NVLINK_CORE_CALLBACK_PARAMS params;
 
     portMemSet(&params, 0, sizeof(params));
 
@@ -882,10 +881,10 @@ knvlinkCoreGetTlLinkModeCallback
     pKernelNvlink = GPU_GET_KERNEL_NVLINK(pGpu);
 
     params.linkId            = pNvlinkLink->linkId;
-    params.callbackType.type = NV2080_CTRL_NVLINK_CALLBACK_TYPE_GET_TL_LINK_MODE;
+    params.callbackType.type = NV2080_CTRL_INTERNAL_NVLINK_CALLBACK_TYPE_GET_TL_LINK_MODE;
 
     status = knvlinkExecGspRmRpc(pGpu, pKernelNvlink,
-                                 NV2080_CTRL_CMD_NVLINK_CORE_CALLBACK,
+                                 NV2080_CTRL_CMD_INTERNAL_NVLINK_CORE_CALLBACK,
                                  (void *)&params, sizeof(params));
     if (status != NV_OK)
     {
@@ -917,8 +916,8 @@ knvlinkCoreSetTxSublinkModeCallback
     OBJGPU          *pGpu          = NULL;
     KernelNvlink    *pKernelNvlink = NULL;
     NV_STATUS        status        = NV_OK;
-    NV2080_CTRL_NVLINK_CORE_CALLBACK_PARAMS params;
-    NV2080_CTRL_NVLINK_CALLBACK_SET_TX_SUBLINK_MODE_PARAMS
+    NV2080_CTRL_INTERNAL_NVLINK_CORE_CALLBACK_PARAMS params;
+    NV2080_CTRL_INTERNAL_NVLINK_CALLBACK_SET_TX_SUBLINK_MODE_PARAMS
         *pSetTxSublinkModeParams;
 
     portMemSet(&params, 0, sizeof(params));
@@ -942,7 +941,7 @@ knvlinkCoreSetTxSublinkModeCallback
 
     params.linkId            = pNvlinkLink->linkId;
     params.callbackType.type =
-        NV2080_CTRL_NVLINK_CALLBACK_TYPE_SET_TX_SUBLINK_MODE;
+        NV2080_CTRL_INTERNAL_NVLINK_CALLBACK_TYPE_SET_TX_SUBLINK_MODE;
 
     pSetTxSublinkModeParams        = &params.callbackType.callbackParams.setTxSublinkMode;
     pSetTxSublinkModeParams->mode  = mode;
@@ -950,7 +949,7 @@ knvlinkCoreSetTxSublinkModeCallback
                                       NV_TRUE : NV_FALSE;
 
     status = knvlinkExecGspRmRpc(pGpu, pKernelNvlink,
-                                 NV2080_CTRL_CMD_NVLINK_CORE_CALLBACK,
+                                 NV2080_CTRL_CMD_INTERNAL_NVLINK_CORE_CALLBACK,
                                  (void *)&params, sizeof(params));
     if (status != NV_OK)
     {
@@ -993,8 +992,8 @@ knvlinkCoreSetRxSublinkModeCallback
     OBJGPU          *pGpu          = NULL;
     KernelNvlink    *pKernelNvlink = NULL;
     NV_STATUS        status        = NV_OK;
-    NV2080_CTRL_NVLINK_CORE_CALLBACK_PARAMS params;
-    NV2080_CTRL_NVLINK_CALLBACK_SET_RX_SUBLINK_MODE_PARAMS
+    NV2080_CTRL_INTERNAL_NVLINK_CORE_CALLBACK_PARAMS params;
+    NV2080_CTRL_INTERNAL_NVLINK_CALLBACK_SET_RX_SUBLINK_MODE_PARAMS
                     *pSetRxSublinkModeParams;
 
     pNvlinkLink = (KNVLINK_RM_LINK *)link->link_info;
@@ -1016,7 +1015,7 @@ knvlinkCoreSetRxSublinkModeCallback
 
     params.linkId            = pNvlinkLink->linkId;
     params.callbackType.type =
-        NV2080_CTRL_NVLINK_CALLBACK_TYPE_SET_RX_SUBLINK_MODE;
+        NV2080_CTRL_INTERNAL_NVLINK_CALLBACK_TYPE_SET_RX_SUBLINK_MODE;
 
     pSetRxSublinkModeParams        =
         &params.callbackType.callbackParams.setRxSublinkMode;
@@ -1025,7 +1024,7 @@ knvlinkCoreSetRxSublinkModeCallback
                                       NV_TRUE : NV_FALSE;
 
     status = knvlinkExecGspRmRpc(pGpu, pKernelNvlink,
-                                 NV2080_CTRL_CMD_NVLINK_CORE_CALLBACK,
+                                 NV2080_CTRL_CMD_INTERNAL_NVLINK_CORE_CALLBACK,
                                  (void *)&params, sizeof(params));
     if (status != NV_OK)
     {
@@ -1066,7 +1065,7 @@ knvlinkCoreGetTxSublinkModeCallback
     OBJGPU          *pGpu          = NULL;
     KernelNvlink    *pKernelNvlink = NULL;
     NV_STATUS        status        = NV_OK;
-    NV2080_CTRL_NVLINK_CORE_CALLBACK_PARAMS params;
+    NV2080_CTRL_INTERNAL_NVLINK_CORE_CALLBACK_PARAMS params;
 
     portMemSet(&params, 0, sizeof(params));
 
@@ -1089,10 +1088,10 @@ knvlinkCoreGetTxSublinkModeCallback
 
     params.linkId            = pNvlinkLink->linkId;
     params.callbackType.type =
-        NV2080_CTRL_NVLINK_CALLBACK_TYPE_GET_TX_SUBLINK_MODE;
+        NV2080_CTRL_INTERNAL_NVLINK_CALLBACK_TYPE_GET_TX_SUBLINK_MODE;
 
     status = knvlinkExecGspRmRpc(pGpu, pKernelNvlink,
-                                 NV2080_CTRL_CMD_NVLINK_CORE_CALLBACK,
+                                 NV2080_CTRL_CMD_INTERNAL_NVLINK_CORE_CALLBACK,
                                  (void *)&params, sizeof(params));
     if (status != NV_OK)
     {
@@ -1125,7 +1124,7 @@ knvlinkCoreGetRxSublinkModeCallback
     OBJGPU          *pGpu          = NULL;
     KernelNvlink    *pKernelNvlink = NULL;
     NV_STATUS        status        = NV_OK;
-    NV2080_CTRL_NVLINK_CORE_CALLBACK_PARAMS params;
+    NV2080_CTRL_INTERNAL_NVLINK_CORE_CALLBACK_PARAMS params;
 
     portMemSet(&params, 0, sizeof(params));
 
@@ -1148,10 +1147,10 @@ knvlinkCoreGetRxSublinkModeCallback
 
     params.linkId            = pNvlinkLink->linkId;
     params.callbackType.type =
-        NV2080_CTRL_NVLINK_CALLBACK_TYPE_GET_RX_SUBLINK_MODE;
+        NV2080_CTRL_INTERNAL_NVLINK_CALLBACK_TYPE_GET_RX_SUBLINK_MODE;
 
     status = knvlinkExecGspRmRpc(pGpu, pKernelNvlink,
-                                 NV2080_CTRL_CMD_NVLINK_CORE_CALLBACK,
+                                 NV2080_CTRL_CMD_INTERNAL_NVLINK_CORE_CALLBACK,
                                  (void *)&params, sizeof(params));
     if (status != NV_OK)
     {
@@ -1182,8 +1181,8 @@ knvlinkCoreSetRxSublinkDetectCallback
     OBJGPU          *pGpu          = NULL;
     KernelNvlink    *pKernelNvlink = NULL;
     NV_STATUS        status        = NV_OK;
-    NV2080_CTRL_NVLINK_CORE_CALLBACK_PARAMS params;
-    NV2080_CTRL_NVLINK_CALLBACK_SET_RX_DETECT_PARAMS
+    NV2080_CTRL_INTERNAL_NVLINK_CORE_CALLBACK_PARAMS params;
+    NV2080_CTRL_INTERNAL_NVLINK_CALLBACK_SET_RX_DETECT_PARAMS
                     *pSetRxDetectParams;
 
     portMemSet(&params, 0, sizeof(params));
@@ -1201,7 +1200,7 @@ knvlinkCoreSetRxSublinkDetectCallback
 
     params.linkId            = pNvlinkLink->linkId;
     params.callbackType.type =
-        NV2080_CTRL_NVLINK_CALLBACK_TYPE_SET_RX_SUBLINK_DETECT;
+        NV2080_CTRL_INTERNAL_NVLINK_CALLBACK_TYPE_SET_RX_SUBLINK_DETECT;
 
     pSetRxDetectParams        =
         &params.callbackType.callbackParams.setRxSublinkDetect;
@@ -1209,7 +1208,7 @@ knvlinkCoreSetRxSublinkDetectCallback
                                  NV_TRUE : NV_FALSE;
 
     status = knvlinkExecGspRmRpc(pGpu, pKernelNvlink,
-                                 NV2080_CTRL_CMD_NVLINK_CORE_CALLBACK,
+                                 NV2080_CTRL_CMD_INTERNAL_NVLINK_CORE_CALLBACK,
                                  (void *)&params, sizeof(params));
     if (status != NV_OK)
     {
@@ -1239,8 +1238,8 @@ knvlinkCoreGetRxSublinkDetectCallback
     KernelNvlink    *pKernelNvlink = NULL;
     NV_STATUS        status        = NV_OK;
     NvU32            linkId;
-    NV2080_CTRL_NVLINK_CORE_CALLBACK_PARAMS params;
-    NV2080_CTRL_NVLINK_CALLBACK_GET_RX_DETECT_PARAMS
+    NV2080_CTRL_INTERNAL_NVLINK_CORE_CALLBACK_PARAMS params;
+    NV2080_CTRL_INTERNAL_NVLINK_CALLBACK_GET_RX_DETECT_PARAMS
                     *pGetRxDetectParams;
 
     portMemSet(&params, 0,  sizeof(params));
@@ -1259,12 +1258,12 @@ knvlinkCoreGetRxSublinkDetectCallback
 
     params.linkId            = linkId;
     params.callbackType.type =
-        NV2080_CTRL_NVLINK_CALLBACK_TYPE_GET_RX_SUBLINK_DETECT;
+        NV2080_CTRL_INTERNAL_NVLINK_CALLBACK_TYPE_GET_RX_SUBLINK_DETECT;
 
     pGetRxDetectParams = &params.callbackType.callbackParams.getRxSublinkDetect;
 
     status = knvlinkExecGspRmRpc(pGpu, pKernelNvlink,
-                                 NV2080_CTRL_CMD_NVLINK_CORE_CALLBACK,
+                                 NV2080_CTRL_CMD_INTERNAL_NVLINK_CORE_CALLBACK,
                                  (void *)&params, sizeof(params));
 
     // Store RXDET status mask
@@ -1317,12 +1316,12 @@ knvlinkCoreWriteDiscoveryTokenCallback
     //
     if (pNvlinkLink->ipVerDlPl >= NVLINK_VERSION_40)
     {
-        NV2080_CTRL_NVLINK_UPDATE_REMOTE_LOCAL_SID_PARAMS params;
+        NV2080_CTRL_INTERNAL_NVLINK_UPDATE_REMOTE_LOCAL_SID_PARAMS params;
         portMemSet(&params, 0, sizeof(params));
         params.linkId = pNvlinkLink->linkId;
 
         status = knvlinkExecGspRmRpc(pGpu, pKernelNvlink,
-                                     NV2080_CTRL_CMD_NVLINK_UPDATE_REMOTE_LOCAL_SID,
+                                     NV2080_CTRL_CMD_INTERNAL_NVLINK_UPDATE_REMOTE_LOCAL_SID,
                                      (void *)&params, sizeof(params));
         if (status != NV_OK)
         {
@@ -1342,14 +1341,14 @@ knvlinkCoreWriteDiscoveryTokenCallback
     else
     {
 
-        NV2080_CTRL_NVLINK_CORE_CALLBACK_PARAMS params;
-        NV2080_CTRL_NVLINK_CALLBACK_RD_WR_DISCOVERY_TOKEN_PARAMS
+        NV2080_CTRL_INTERNAL_NVLINK_CORE_CALLBACK_PARAMS params;
+        NV2080_CTRL_INTERNAL_NVLINK_CALLBACK_RD_WR_DISCOVERY_TOKEN_PARAMS
                 *pWriteDiscoveryTokenParams;
 
         portMemSet(&params, 0, sizeof(params));
         params.linkId            = pNvlinkLink->linkId;
         params.callbackType.type =
-                NV2080_CTRL_NVLINK_CALLBACK_TYPE_WRITE_DISCOVERY_TOKEN;
+                NV2080_CTRL_INTERNAL_NVLINK_CALLBACK_TYPE_WRITE_DISCOVERY_TOKEN;
 
         pWriteDiscoveryTokenParams            =
             &params.callbackType.callbackParams.writeDiscoveryToken;
@@ -1357,7 +1356,7 @@ knvlinkCoreWriteDiscoveryTokenCallback
         pWriteDiscoveryTokenParams->token     = token;
 
         status = knvlinkExecGspRmRpc(pGpu, pKernelNvlink,
-                                     NV2080_CTRL_CMD_NVLINK_CORE_CALLBACK,
+                                     NV2080_CTRL_CMD_INTERNAL_NVLINK_CORE_CALLBACK,
                                      (void *)&params, sizeof(params));
     }
 
@@ -1397,9 +1396,8 @@ knvlinkCoreReadDiscoveryTokenCallback
     OBJGPU          *pGpu          = pNvlinkLink->pGpu;
     KernelNvlink    *pKernelNvlink = NULL;
     NV_STATUS        status        = NV_OK;
-    NV2080_CTRL_NVLINK_CORE_CALLBACK_PARAMS params;
-    NV2080_CTRL_NVLINK_CALLBACK_RD_WR_DISCOVERY_TOKEN_PARAMS
-        *pReadDiscoveryTokenParams;
+    NV2080_CTRL_INTERNAL_NVLINK_CORE_CALLBACK_PARAMS params;
+    NV2080_CTRL_INTERNAL_NVLINK_CALLBACK_RD_WR_DISCOVERY_TOKEN_PARAMS *pReadDiscoveryTokenParams = NULL;
 
     portMemSet(&params, 0, sizeof(params));
 
@@ -1426,14 +1424,14 @@ knvlinkCoreReadDiscoveryTokenCallback
     {
         params.linkId            = pNvlinkLink->linkId;
         params.callbackType.type =
-            NV2080_CTRL_NVLINK_CALLBACK_TYPE_READ_DISCOVERY_TOKEN;
+            NV2080_CTRL_INTERNAL_NVLINK_CALLBACK_TYPE_READ_DISCOVERY_TOKEN;
 
         pReadDiscoveryTokenParams            =
             &params.callbackType.callbackParams.readDiscoveryToken;
         pReadDiscoveryTokenParams->ipVerDlPl = pNvlinkLink->ipVerDlPl;
 
         status = knvlinkExecGspRmRpc(pGpu, pKernelNvlink,
-                                     NV2080_CTRL_CMD_NVLINK_CORE_CALLBACK,
+                                     NV2080_CTRL_CMD_INTERNAL_NVLINK_CORE_CALLBACK,
                                      (void *)&params, sizeof(params));
     }
 
@@ -1471,7 +1469,7 @@ knvlinkCoreTrainingCompleteCallback
     OBJGPU          *pGpu          = NULL;
     KernelNvlink    *pKernelNvlink = NULL;
     NV_STATUS        status;
-    NV2080_CTRL_NVLINK_CORE_CALLBACK_PARAMS params;
+    NV2080_CTRL_INTERNAL_NVLINK_CORE_CALLBACK_PARAMS params;
 
     portMemSet(&params, 0, sizeof(params));
 
@@ -1481,10 +1479,10 @@ knvlinkCoreTrainingCompleteCallback
     pKernelNvlink = GPU_GET_KERNEL_NVLINK(pGpu);
 
     params.linkId = pNvlinkLink->linkId;
-    params.callbackType.type = NV2080_CTRL_NVLINK_CALLBACK_TYPE_TRAINING_COMPLETE;
+    params.callbackType.type = NV2080_CTRL_INTERNAL_NVLINK_CALLBACK_TYPE_TRAINING_COMPLETE;
 
     status = knvlinkExecGspRmRpc(pGpu, pKernelNvlink,
-                                 NV2080_CTRL_CMD_NVLINK_CORE_CALLBACK,
+                                 NV2080_CTRL_CMD_INTERNAL_NVLINK_CORE_CALLBACK,
                                  (void *)&params, sizeof(params));
     if (status != NV_OK)
     {
@@ -1492,6 +1490,16 @@ knvlinkCoreTrainingCompleteCallback
     }
 }
 
+NvlStatus
+knvlinkCoreGetCciLinkModeCallback
+(
+    nvlink_link *link,
+    NvU64       *mode
+)
+{
+    return NVL_SUCCESS;
+}
+
 /*
  * @brief nvlinkCoreGetUphyLoadCallback send ALI training on the specified link
  *
@@ -1516,6 +1524,9 @@ knvlinkCoreAliTrainingCallback
 
     pKernelNvlink = GPU_GET_KERNEL_NVLINK(pGpu);
 
+    if (pKernelNvlink->ipVerNvlink >= NVLINK_VERSION_50)
+        return 0;
+
     status = knvlinkPreTrainLinksToActiveAli(pGpu, pKernelNvlink,
                                                  BIT(pNvlinkLink->linkId), NV_TRUE);
     if (status != NV_OK)
@@ -1554,8 +1565,8 @@ knvlinkCoreGetUphyLoadCallback
     OBJGPU          *pGpu          = pNvlinkLink->pGpu;
     KernelNvlink    *pKernelNvlink = NULL;
     NV_STATUS        status;
-    NV2080_CTRL_NVLINK_CORE_CALLBACK_PARAMS params;
-    NV2080_CTRL_NVLINK_CALLBACK_GET_UPHY_LOAD_PARAMS
+    NV2080_CTRL_INTERNAL_NVLINK_CORE_CALLBACK_PARAMS params;
+    NV2080_CTRL_INTERNAL_NVLINK_CALLBACK_GET_UPHY_LOAD_PARAMS
         *pGetUphyLoadParams;
 
     portMemSet(&params, 0, sizeof(params));
@@ -1570,13 +1581,13 @@ knvlinkCoreGetUphyLoadCallback
 
     params.linkId            = pNvlinkLink->linkId;
     params.callbackType.type =
-        NV2080_CTRL_NVLINK_CALLBACK_TYPE_GET_UPHY_LOAD;
+        NV2080_CTRL_INTERNAL_NVLINK_CALLBACK_TYPE_GET_UPHY_LOAD;
 
     pGetUphyLoadParams =
         &params.callbackType.callbackParams.getUphyLoad;
 
     status = knvlinkExecGspRmRpc(pGpu, pKernelNvlink,
-                                 NV2080_CTRL_CMD_NVLINK_CORE_CALLBACK,
+                                 NV2080_CTRL_CMD_INTERNAL_NVLINK_CORE_CALLBACK,
                                  (void *)&params, sizeof(params));
     if (status != NV_OK)
     {
