@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2021-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2021-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -23,25 +23,25 @@
 
 #include "kernel/gpu/rc/kernel_rc.h"
 
+#include "kernel/core/locks.h"
 #include "kernel/core/system.h"
 #include "kernel/gpu/bif/kernel_bif.h"
 #include "kernel/gpu/mig_mgr/kernel_mig_manager.h"
+#include "kernel/gpu/timer/objtmr.h"
 #include "kernel/os/os.h"
 #include "kernel/platform/chipset/chipset.h"
 #include "kernel/rmapi/client.h"
 
-
+#include "lib/base_utils.h"
 #include "libraries/utils/nvprintf.h"
-#include "nvRmReg.h"
+#include "nvrm_registry.h"
 #include "nverror.h"
 #include "nvtypes.h"
-#include "objtmr.h"
 
 
 static void _krcInitRegistryOverrides(OBJGPU *pGpu, KernelRc *pKernelRc);
 static void _krcLogUuidOnce(OBJGPU *pGpu, KernelRc *pKernelRc);
 
-
 NV_STATUS
 krcConstructEngine_IMPL
 (
@@ -135,15 +135,6 @@ _krcInitRegistryOverrides
         pKernelRc->bBreakOnRc = NV_TRUE;
     }
 
-    if (pKernelRc->bBreakOnRc)
-    {
-        NV_PRINTF(LEVEL_INFO, "Breakpoint on RC Error is enabled\n");
-    }
-    else
-    {
-        NV_PRINTF(LEVEL_INFO, "Breakpoint on RC Error is disabled\n");
-    }
-
 
     if (osReadRegistryDword(pGpu,
                             NV_REG_STR_RM_WATCHDOG_TIMEOUT,
@@ -184,7 +175,7 @@ _krcInitRegistryOverrides
     {
         pKernelRc->watchdog.flags |= WATCHDOG_FLAGS_DISABLED;
     }
-    else if (gpuIsCCFeatureEnabled(pGpu) && !gpuIsCCDevToolsModeEnabled(pGpu))
+    else if (gpuIsCCFeatureEnabled(pGpu))
     {
         pKernelRc->watchdog.flags |= WATCHDOG_FLAGS_DISABLED;
     }
@@ -203,6 +194,13 @@ _krcInitRegistryOverrides
 #endif
         }
     }
+
+    //
+    // Do RC on BAR faults by default (For bug 1842228).
+    // Only applicable to Volta+ chips.
+    //
+    pKernelRc->bRcOnBar2Fault = NV_TRUE;
+
 }
 
 
@@ -269,7 +267,7 @@ krcReportXid_IMPL
 (
     OBJGPU     *pGpu,
     KernelRc   *pKernelRc,
-    NvU32       exceptType,
+    XidContext  context,
     const char *pMsg
 )
 {
@@ -285,82 +283,106 @@ krcReportXid_IMPL
         NvU16          gpuPartitionId;
         NvU16          computeInstanceId;
         KernelChannel *pKernelChannel = krcGetChannelInError(pKernelRc);
-        char          *current_procname = NULL;
-
-        // Channels are populated with osGetCurrentProcessName() and pid of
-        // their process at creation-time. If no channel was found, mark unknown
-        const char *procname = "<unknown>";
-        char pid_string[12] = "'<unknown>'";
+        char          *allocProcName = NULL;
+        const char    *procName = NULL;
+        char           pidStr[12] = "";
 
         //
-        // Get PID of channel creator if available, or get the current PID for
-        // exception types that never have an associated channel
+        // When the channel context is available, use the allocating process'
+        // PID/name, which were populated when the channel was allocated.
+        // For errors without a channel context, use the current process
+        // information, if there's a Resource Server CALL_CONTEXT present
+        // in TLS, indicating we're servicing an RMAPI request.
         //
         if (pKernelChannel != NULL)
         {
+            //
+            // This function executes under (at least) the GPU lock, so the
+            // pKernelChannel resource can be accessed, regardless of whether
+            // the API lock is held: the pKernelChannel can't go away while the
+            // GPU lock is held, so the owning client can't go away either.
+            // Duping requires holding all GPU locks, so the client's process/
+            // name can't change in this context either.
+            //
             RsClient *pClient = RES_GET_CLIENT(pKernelChannel);
             RmClient *pRmClient = dynamicCast(pClient, RmClient);
-            procname = pRmClient->name;
-            nvDbgSnprintf(pid_string, sizeof(pid_string), "%u", pKernelChannel->ProcessID);
+            procName = pRmClient->name;
+            nvDbgSnprintf(pidStr, sizeof(pidStr), "%u", pKernelChannel->ProcessID);
         }
-        else if (exceptType == GSP_RPC_TIMEOUT)
+        else if (resservGetTlsCallContext() != NULL)
         {
-            NvU32 current_pid = osGetCurrentProcess();
+            NvU32 currentPid = osGetCurrentProcess();
 
-            nvDbgSnprintf(pid_string, sizeof(pid_string), "%u", current_pid);
+            nvDbgSnprintf(pidStr, sizeof(pidStr), "%u", currentPid);
 
-            current_procname = portMemAllocNonPaged(NV_PROC_NAME_MAX_LENGTH);
-            if (current_procname != NULL)
+            allocProcName = portMemAllocNonPaged(NV_PROC_NAME_MAX_LENGTH);
+            if (allocProcName != NULL)
             {
-                osGetCurrentProcessName(current_procname, NV_PROC_NAME_MAX_LENGTH);
-                procname = current_procname;
+                osGetCurrentProcessName(allocProcName, NV_PROC_NAME_MAX_LENGTH);
+                procName = allocProcName;
             }
         }
 
         _krcLogUuidOnce(pGpu, pKernelRc);
 
         krcGetMigAttributionForError_HAL(pKernelRc,
-                                         exceptType,
+                                         context.xid,
                                          &gpuPartitionId,
                                          &computeInstanceId);
 
+        NvBool bPrintRootCause = (
+            (context.xid == ROBUST_CHANNEL_PREEMPTIVE_REMOVAL) &&
+            (context.rootCause.preemptiveRemovalPreviousXid != 0));
+
+        char rootCauseXidStr[64] = "";
+        if (bPrintRootCause)
+        {
+            nvDbgSnprintf(rootCauseXidStr,
+                          sizeof(rootCauseXidStr),
+            // this space -v- is intentional
+                          " caused by previous Xid %d",
+                          context.rootCause.preemptiveRemovalPreviousXid);
+        }
+
+        // Code-generating macro to reduce duplication
+        #define XID_PRINT_WITH_ATTR(attr, ...)                                          \
+            do {                                                                        \
+                if (procName != NULL)                                                   \
+                    portDbgPrintf("NVRM: Xid (" attr "): %d, pid=%s, name=%s, %s%s\n",  \
+                                  __VA_ARGS__, context.xid, pidStr, procName,           \
+                                  pMsg != NULL ? pMsg : "",                             \
+                                  rootCauseXidStr);                                     \
+                else                                                                    \
+                    portDbgPrintf("NVRM: Xid (" attr "): %d, %s%s\n",                   \
+                                  __VA_ARGS__, context.xid, pMsg != NULL ? pMsg : "",   \
+                                  rootCauseXidStr);                                     \
+            } while (0)
+
         if (gpuPartitionId    != KMIGMGR_INSTANCE_ATTRIBUTION_ID_INVALID &&
             computeInstanceId != KMIGMGR_INSTANCE_ATTRIBUTION_ID_INVALID)
         {
             // Attribute this XID to both GPU / Compute instance
-            portDbgPrintf(
-                "NVRM: Xid (PCI:%04x:%02x:%02x GPU-I:%02u GPU-CI:%02u): %d, pid=%s, name=%s, %s\n",
-                gpuGetDomain(pGpu), gpuGetBus(pGpu), gpuGetDevice(pGpu),
-                gpuPartitionId, computeInstanceId,
-                exceptType,
-                pid_string,
-                procname,
-                pMsg != NULL ? pMsg : "");
+            XID_PRINT_WITH_ATTR("PCI:%04x:%02x:%02x GPU-I:%02u GPU-CI:%02u",
+                                gpuGetDomain(pGpu), gpuGetBus(pGpu), gpuGetDevice(pGpu),
+                                gpuPartitionId, computeInstanceId);
         }
         else if (gpuPartitionId != KMIGMGR_INSTANCE_ATTRIBUTION_ID_INVALID)
         {
             // Attribute this XID to GPU instance only
-            portDbgPrintf(
-                "NVRM: Xid (PCI:%04x:%02x:%02x GPU-I:%02u): %d, pid=%s, name=%s, %s\n",
-                gpuGetDomain(pGpu), gpuGetBus(pGpu), gpuGetDevice(pGpu),
-                gpuPartitionId,
-                exceptType,
-                pid_string,
-                procname,
-                pMsg != NULL ? pMsg : "");
+            XID_PRINT_WITH_ATTR("PCI:%04x:%02x:%02x GPU-I:%02u",
+                                gpuGetDomain(pGpu), gpuGetBus(pGpu), gpuGetDevice(pGpu),
+                                gpuPartitionId);
         }
         else
         {
-            // Legacy (no attribution) XID reporting
-            portDbgPrintf("NVRM: Xid (PCI:%04x:%02x:%02x): %d, pid=%s, name=%s, %s\n",
-                gpuGetDomain(pGpu), gpuGetBus(pGpu), gpuGetDevice(pGpu),
-                exceptType,
-                pid_string,
-                procname,
-                pMsg != NULL ? pMsg : "");
+            // Attribute this Xid to the device only
+            XID_PRINT_WITH_ATTR("PCI:%04x:%02x:%02x",
+                                gpuGetDomain(pGpu), gpuGetBus(pGpu), gpuGetDevice(pGpu));
         }
 
-        portMemFree(current_procname);
+        #undef XID_PRINT_WITH_ATTR
+
+        portMemFree(allocProcName);
     }
 }
 
@@ -426,8 +448,8 @@ krcCheckBusError_KERNEL
                                 &clDevCtrlStatus) == NV_OK &&
         clDevCtrlStatusFlags != 0)
     {
-        NV_PRINTF(LEVEL_ERROR,
-            "PCI-E corelogic status has pending errors (CL_PCIE_DEV_CTRL_STATUS = %08X):\n",
+        NV_PRINTF(LEVEL_INFO,
+            "PCI-E corelogic: Pending errors in DEV_CTRL_STATUS = %08X\n",
             clDevCtrlStatus);
 
         clDevCtrlStatusFlags_Org = clDevCtrlStatusFlags;
@@ -435,7 +457,7 @@ krcCheckBusError_KERNEL
         if (clDevCtrlStatusFlags &
             NV2080_CTRL_BUS_INFO_PCIE_LINK_ERRORS_CORR_ERROR)
         {
-            NV_PRINTF(LEVEL_ERROR, "     _CORR_ERROR_DETECTED\n");
+            NV_PRINTF(LEVEL_INFO, "PCI-E corelogic: CORR_ERROR_DETECTED\n");
             // not much interested in this one
             clDevCtrlStatusFlags &=
                 ~NV2080_CTRL_BUS_INFO_PCIE_LINK_ERRORS_CORR_ERROR;
@@ -443,26 +465,27 @@ krcCheckBusError_KERNEL
         if (clDevCtrlStatusFlags &
             NV2080_CTRL_BUS_INFO_PCIE_LINK_ERRORS_NON_FATAL_ERROR)
         {
-            NV_PRINTF(LEVEL_ERROR, "     _NON_FATAL_ERROR_DETECTED\n");
+            NV_PRINTF(LEVEL_INFO, "PCI-E corelogic: NON_FATAL_ERROR_DETECTED\n");
         }
         if (clDevCtrlStatusFlags &
             NV2080_CTRL_BUS_INFO_PCIE_LINK_ERRORS_FATAL_ERROR)
         {
-            NV_PRINTF(LEVEL_ERROR, "     _FATAL_ERROR_DETECTED\n");
+            NV_PRINTF(LEVEL_ERROR, "PCI-E corelogic: FATAL_ERROR_DETECTED\n");
         }
         if (clDevCtrlStatusFlags &
             NV2080_CTRL_BUS_INFO_PCIE_LINK_ERRORS_UNSUPP_REQUEST)
         {
-            NV_PRINTF(LEVEL_ERROR, "     _UNSUPP_REQUEST_DETECTED\n");
+            NV_PRINTF(LEVEL_INFO, "PCI-E corelogic: UNSUPP_REQUEST_DETECTED\n");
         }
     }
 
     // Corelogic AER
     if (pCl != NULL && clPcieReadAerCapability(pGpu, pCl, &clAer) == NV_OK &&
-        (clAer.UncorrErrStatusReg != 0 || clAer.RooErrStatus != 0))
+        (clAer.UncorrErrStatusReg != 0 || 
+         (clAer.RooErrStatus & ~CL_AER_ROOT_ERROR_STATUS_ERR_COR_SUBCLASS_MASK) != 0))
     {
         NV_PRINTF(LEVEL_ERROR,
-                  "PCE-I Advanced Error Reporting Corelogic Info:\n");
+                  "PCI-E Advanced Error Reporting Corelogic Info:\n");
         NV_PRINTF(LEVEL_ERROR,
                   "     Uncorr Error Status Register    : %08X\n",
                   clAer.UncorrErrStatusReg);
@@ -528,3 +551,86 @@ krcGetChannelInError_FWCLIENT
     NV_ASSERT_OR_RETURN(IS_GSP_CLIENT(ENG_GET_GPU(pKernelRc)), NULL);
     return pKernelRc->pPreviousChannelInError;
 }
+
+#if !(defined(DEBUG) || defined(DEVELOP))
+//
+// Skip dumping the NvLog for the next XIDs which gets triggered
+// within 1 sec.
+// This will prevent logs from getting flooded whenever there is any
+// XID storm due to some bad state.
+//
+static void
+_krcValidateAndDumpToKernelLog(NvU64 *lastXidTimestamp)
+{
+    NvU32 sec, usec;
+
+    if (((osGetCurrentTime(&sec, &usec) == NV_OK) &&
+       ((((sec * 1000000) + usec) - *lastXidTimestamp) > 1000000)))
+    {
+        nvlogDumpToKernelLog(NV_TRUE);
+        *lastXidTimestamp = ((NvU64)sec * 1000000) + usec;
+    }
+}
+#endif //!(DEBUG || DEVELOP)
+
+void krcDumpNvLog(OBJGPU *pGpu,
+                  NvU32 exceptType)
+{
+#if !(defined(DEBUG) || defined(DEVELOP))
+    NvU32 data32 = 0;
+    static NvU64 lastXidTimestamp;
+
+    if ((osReadRegistryDword(pGpu, NV_REG_STR_RM_DUMP_NVLOG, &data32) == NV_OK) &&
+        (data32 == NV_REG_STR_RM_DUMP_NVLOG_ENABLE))
+    {
+        NvU8 suppressXid[MAX_XID_SUPPRESS_KEY_LENGTH];
+        NvU32 size = sizeof(suppressXid);
+        NvBool xidDumpSuppressed = NV_FALSE;
+
+        if (osReadRegistryBinary(pGpu, NV_REG_SUPPRESS_XID_DUMP,
+            (NvU8 *)suppressXid, &size) == NV_OK)
+        {
+            NvU8 *pStr, *pEndStr;
+            NvU32 exceptionType = 0;
+            NvU32 numFound;
+
+            suppressXid[MAX_XID_SUPPRESS_KEY_LENGTH - 1] = '\0';
+            pStr = (NvU8 *)suppressXid;
+
+            while (pStr != NULL)
+            {
+                exceptionType = nvStrToL(pStr, &pEndStr, BASE10, ',', &numFound);
+                if (exceptionType == exceptType)
+                {
+                    xidDumpSuppressed = NV_TRUE;
+                    break;
+                }
+                pStr = pEndStr;
+            }
+
+
+            if (!xidDumpSuppressed)
+            {
+                _krcValidateAndDumpToKernelLog(&lastXidTimestamp);
+            }
+        }
+
+        //
+        // Dump logs after RC recovery for below Xids. Be careful while enabling
+        // this regkey since we are dumping buffers with GPU locks held (takes ~500 us).
+        //
+        else if (exceptType == ROBUST_CHANNEL_GR_EXCEPTION || // 13
+            exceptType == ROBUST_CHANNEL_FIFO_ERROR_MMU_ERR_FLT || // 31
+            exceptType == ROBUST_CHANNEL_PBDMA_ERROR || // 32
+            exceptType == ROBUST_CHANNEL_RESETCHANNEL_VERIF_ERROR || // 43
+            exceptType == PMU_BREAKPOINT || // 61
+            exceptType == PMU_HALT_ERROR || // 62
+            exceptType == ROBUST_CHANNEL_GPU_HAS_FALLEN_OFF_THE_BUS || // 79
+            exceptType == GSP_RPC_TIMEOUT || // 119
+            exceptType == GSP_ERROR) // 120
+        {
+            _krcValidateAndDumpToKernelLog(&lastXidTimestamp);
+        }
+    }
+#endif //!(DEBUG || DEVELOP)
+}
