@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1993-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1993-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -27,7 +27,7 @@
  */
 
 /* ------------------------ Includes ---------------------------------------- */
-#include "objtmr.h"
+#include "gpu/timer/objtmr.h"
 #include "class/cl0004.h" // NV004_NOTIFIERS_SET_ALARM_NOTIFY
 #include "gpu/gpu_resource.h"
 #include "core/locks.h"
@@ -35,12 +35,12 @@
 #include "kernel/gpu/intr/intr.h"
 
 /* ------------------------ Static Function Prototypes ---------------------- */
-static PTMR_EVENT_PVT   _tmrPullCallbackFromHead (OBJTMR *);
+static TMR_EVENT_PVT *  _tmrPullCallbackFromHead (OBJTMR *);
 static void             _tmrScanCallback(OBJTMR *, void *);
-static void             _tmrScanCallbackOSTimer(OBJTMR *, PTMR_EVENT_PVT);
-static PTMR_EVENT_PVT   _tmrGetNextFreeCallback(OBJTMR *);
-static NV_STATUS        _tmrInsertCallback(OBJTMR *, PTMR_EVENT_PVT, NvU64);
-static void             _tmrInsertCallbackInList(OBJGPU *pGpu, OBJTMR *pTmr, PTMR_EVENT_PVT pEvent);
+static void             _tmrScanCallbackOSTimer(OBJTMR *, TMR_EVENT_PVT *);
+static TMR_EVENT_PVT *  _tmrGetNextFreeCallback(OBJTMR *);
+static NV_STATUS        _tmrInsertCallback(OBJTMR *, TMR_EVENT_PVT *, NvU64);
+static void             _tmrInsertCallbackInList(OBJGPU *pGpu, OBJTMR *pTmr, TMR_EVENT_PVT *pEvent);
 static void             _tmrStateLoadCallbacks(OBJGPU *, OBJTMR *);
 static NV_STATUS        _tmrGetNextAlarmTime(OBJTMR *, NvU64 *);
 static void             _tmrScheduleCallbackInterrupt(OBJGPU *, OBJTMR *, NvU64);
@@ -206,7 +206,7 @@ static NV_INLINE NvBool tmrEventsExist(OBJTMR *pTmr)
     return pTmr->pRmActiveEventList != NULL;
 }
 
-static NV_INLINE NvBool tmrIsOSTimer(OBJTMR *pTmr, PTMR_EVENT pEventPublic)
+static NV_INLINE NvBool tmrIsOSTimer(OBJTMR *pTmr, TMR_EVENT *pEventPublic)
 {
     return ((pEventPublic != NULL) &&
            (pTmr->getProperty(pTmr, PDB_PROP_TMR_USE_OS_TIMER_FOR_CALLBACKS) ||
@@ -221,13 +221,13 @@ static NV_INLINE NvBool tmrIsOSTimer(OBJTMR *pTmr, PTMR_EVENT pEventPublic)
 NV_STATUS tmrEventCreate_IMPL
 (
     OBJTMR     *pTmr,
-    PTMR_EVENT *ppEventPublic,
+    TMR_EVENT **ppEventPublic,
     TIMEPROC    Proc,
     void       *pUserData,
     NvU32       flags
 )
 {
-    PTMR_EVENT_PVT *ppEvent = (PTMR_EVENT_PVT*)ppEventPublic;
+    TMR_EVENT_PVT **ppEvent = (TMR_EVENT_PVT **)ppEventPublic;
     NV_STATUS status = NV_OK;
 
     *ppEvent = portMemAllocNonPaged(sizeof(TMR_EVENT_PVT));
@@ -273,10 +273,19 @@ _tmrScheduleCallbackInterrupt
     if (pTmr->getProperty(pTmr, PDB_PROP_TMR_USE_COUNTDOWN_TIMER_FOR_RM_CALLBACKS))
     {
         NvU64 currentTime;
-        NvU32 countdownTime;
+        // 
+        // Bug: 5071665, 4417666. High spikes of DPC Activity in ARM based system.
+        // The issue is caused due to an ARM compiler optimization bug which is doing 
+        // 32-bit subtraction instead of 64-bit subtraction casted to 32-bits. 
+        // Fix: Set 'volatile' qualifier to' countdownTime' so that it is not optimized.
+        //
+        volatile NvU32 countdownTime = 0;
 
         tmrGetCurrentTime(pTmr, &currentTime);
-        countdownTime = currentTime < alarmTime ? NvU64_LO32(alarmTime - currentTime) : 0;
+        if (currentTime < alarmTime)
+        {
+            countdownTime = NvU64_LO32(alarmTime - currentTime);
+        }
         tmrSetCountdown_HAL(pGpu, pTmr, countdownTime, 0, NULL);
     }
     else
@@ -328,13 +337,13 @@ tmrGetCallbackInterruptPending_IMPL
 void tmrEventCancel_IMPL
 (
     OBJTMR         *pTmr,
-    PTMR_EVENT      pEventPublic
+    TMR_EVENT      *pEventPublic
 )
 {
     NvU64 nextAlarmTime;
     OBJGPU *pGpu = ENG_GET_GPU(pTmr);
-    PTMR_EVENT_PVT pEvent = (PTMR_EVENT_PVT)pEventPublic;
-    PTMR_EVENT_PVT pChaser = pTmr->pRmActiveEventList;
+    TMR_EVENT_PVT *pEvent = (TMR_EVENT_PVT *)pEventPublic;
+    TMR_EVENT_PVT *pChaser = pTmr->pRmActiveEventList;
     NvBool bRemovedHead = pChaser == pEvent;
 
     if (pEventPublic == NULL)
@@ -397,10 +406,10 @@ void tmrEventCancel_IMPL
 void tmrEventDestroy_IMPL
 (
     OBJTMR     *pTmr,
-    PTMR_EVENT  pEventPublic
+    TMR_EVENT  *pEventPublic
 )
 {
-    PTMR_EVENT_PVT pEvent = (PTMR_EVENT_PVT)pEventPublic;
+    TMR_EVENT_PVT *pEvent = (TMR_EVENT_PVT *)pEventPublic;
 
     if (pEvent != NULL)
     {
@@ -421,12 +430,51 @@ void tmrEventDestroy_IMPL
 }
 
 /*!
- * TODO: document
+ * Returns time until next callback for a given event
+ *
+ * @param[in]   pEvent   The event whose remaining time needs to be determined.
+ */
+NV_STATUS
+tmrEventTimeUntilNextCallback_IMPL
+(
+    OBJTMR     *pTmr,
+    TMR_EVENT  *pEventPublic,
+    NvU64      *pTimeUntilCallbackNs
+)
+{
+    NvU64 currentTime;
+    NvU64 nextAlarmTime;
+
+    TMR_EVENT_PVT *pEvent = (TMR_EVENT_PVT*)pEventPublic;
+
+    if (tmrIsOSTimer(pTmr, pEventPublic))
+    {
+        currentTime = osGetCurrentTick();
+        // timens corresponds to relative time for OS timer
+        NV_CHECK_OR_RETURN(LEVEL_ERROR, portSafeAddU64(pEvent->timens, pEvent->startTimeNs, &nextAlarmTime),
+                           NV_ERR_INVALID_ARGUMENT);
+    }
+    else
+    {
+        NV_ASSERT_OK_OR_RETURN(tmrGetCurrentTime(pTmr, &currentTime));
+        // timens corresponds to abs time in case of ptimer
+        nextAlarmTime = pEvent->timens;
+    }
+    if (currentTime > nextAlarmTime)
+        return NV_ERR_INVALID_STATE;
+
+    *pTimeUntilCallbackNs = nextAlarmTime - currentTime;
+    return NV_OK;
+}
+
+
+/*!
+ * Callback invoked by NV0004_CTRL_CMD_TMR_SET_ALARM_NOTIFY
  */
 static NV_STATUS
-_nv0004CtrlCmdTmrSetAlarmNotifyCallback(OBJGPU *pGpu, OBJTMR *pTmr, void *pData)
+_nv0004CtrlCmdTmrSetAlarmNotifyCallback(OBJGPU *pGpu, OBJTMR *pTmr, TMR_EVENT *pTmrEvent)
 {
-    PEVENTNOTIFICATION pNotifyEvent = pData;
+    PEVENTNOTIFICATION pNotifyEvent = pTmrEvent->pUserData;
     NV_STATUS status = NV_OK;
 
     // perform a direct callback to the client
@@ -434,15 +482,16 @@ _nv0004CtrlCmdTmrSetAlarmNotifyCallback(OBJGPU *pGpu, OBJTMR *pTmr, void *pData)
     {
         //one shot signal
         status = osNotifyEvent(pGpu, pNotifyEvent, NV004_NOTIFIERS_SET_ALARM_NOTIFY, 0, NV_OK);
+        if (status != NV_OK)
+        {
+            NV_PRINTF(LEVEL_ERROR, "failed to notify event in callback, status: 0x%08x\n", status);
+            return status;
+        }
     }
 
     return status;
 }
 
-/*!
- * TODO: document
- * TODO: Migrate this to match current API (probably)
- */
 NV_STATUS
 tmrapiCtrlCmdTmrSetAlarmNotify_IMPL
 (
@@ -450,8 +499,10 @@ tmrapiCtrlCmdTmrSetAlarmNotify_IMPL
     NV0004_CTRL_TMR_SET_ALARM_NOTIFY_PARAMS *pParams
 )
 {
+    NV_STATUS status;
     OBJGPU *pGpu = GPU_RES_GET_GPU(pTimerApi);
     OBJTMR *pTmr = GPU_GET_TIMER(pGpu);
+    TMR_EVENT *pTmrEvent = NULL;
     PEVENTNOTIFICATION pNotifyEvent = inotifyGetNotificationList(staticCast(pTimerApi, INotifier));
 
     // Validate the timer event
@@ -466,11 +517,29 @@ tmrapiCtrlCmdTmrSetAlarmNotify_IMPL
         return NV_ERR_INVALID_ARGUMENT;
     }
 
-    // schedule the timer
-    tmrScheduleCallbackRel(pTmr,
-                           _nv0004CtrlCmdTmrSetAlarmNotifyCallback,
-                           pNotifyEvent,
-                           pParams->alarmTimeNsecs, 0, 0);
+    // Create an OS timer if not already created
+    if (pNotifyEvent->pTmrEvent == NULL)
+    {
+        status = tmrEventCreate(pTmr,
+                                &pTmrEvent,
+                                _nv0004CtrlCmdTmrSetAlarmNotifyCallback,
+                                (void*)pNotifyEvent,
+                                0);
+        if (status != NV_OK)
+            return status;
+
+        pNotifyEvent->pGpu = pGpu;
+        pNotifyEvent->pTmrEvent = pTmrEvent;
+    }
+    else
+    {
+        pTmrEvent = pNotifyEvent->pTmrEvent;
+    }
+
+    // Schedule the timer
+    status = tmrEventScheduleRel(pTmr, pTmrEvent, pParams->alarmTimeNsecs);
+    if (status != NV_OK)
+        return status;
 
     return NV_OK;
 }
@@ -549,27 +618,36 @@ NV_STATUS tmrGetCurrentDiffTime_IMPL
 NV_STATUS tmrEventScheduleRel_IMPL
 (
     OBJTMR     *pTmr,
-    PTMR_EVENT  pEvent,
+    TMR_EVENT  *pEvent,
     NvU64       RelTime
 )
 {
     NvU64   AbsTime, currentTime;
     NV_STATUS rmStatus;
 
+    if ((pEvent != NULL) && tmrIsOSTimer(pTmr, pEvent))
+    {
+        TMR_EVENT_PVT *pEventPvt = (TMR_EVENT_PVT *)pEvent;
+
+        NV_CHECK_OK(rmStatus, LEVEL_ERROR,
+                    tmrEventScheduleRelOSTimer_HAL(pTmr, pEvent, RelTime));
+        //
+        // Capture system time here, this will help in scheduling callbacks
+        // if there is a state unload before receiving the OS timer callback.
+        //
+        pEventPvt->startTimeNs = osGetCurrentTick();
+        if (!tmrEventOnList(pTmr, pEvent))
+        {
+            _tmrInsertCallback(pTmr, pEventPvt, RelTime);
+        }
+        return rmStatus;
+    }
+
     rmStatus = tmrGetCurrentTime(pTmr, &currentTime);
     if (rmStatus != NV_OK)
         return rmStatus;
 
-    if (tmrIsOSTimer(pTmr, pEvent))
-    {
-        /*HR timer scheduled in relative mode*/
-        /*TBD : This condition needs to be moved to OS timer handling functions */
-        AbsTime = RelTime;
-    }
-    else
-    {
-        NV_CHECK_OR_RETURN(LEVEL_ERROR, portSafeAddU64(currentTime, RelTime, &AbsTime), NV_ERR_INVALID_ARGUMENT);
-    }
+    NV_CHECK_OR_RETURN(LEVEL_ERROR, portSafeAddU64(currentTime, RelTime, &AbsTime), NV_ERR_INVALID_ARGUMENT);
 
     return tmrEventScheduleAbs(pTmr, pEvent, AbsTime);
 }
@@ -631,12 +709,12 @@ NV_STATUS tmrScheduleCallbackRelSec_IMPL
 NvBool tmrEventOnList_IMPL
 (
     OBJTMR     *pTmr,
-    PTMR_EVENT  pEventPublic
+    TMR_EVENT  *pEventPublic
 )
 {
-    PTMR_EVENT_PVT pEvent = (PTMR_EVENT_PVT)pEventPublic;
-    PTMR_EVENT_PVT pScan  = tmrIsOSTimer(pTmr, pEventPublic) ? 
-                            pTmr->pRmActiveOSTimerEventList : 
+    TMR_EVENT_PVT *pEvent = (TMR_EVENT_PVT *)pEventPublic;
+    TMR_EVENT_PVT *pScan  = tmrIsOSTimer(pTmr, pEventPublic) ?
+                            pTmr->pRmActiveOSTimerEventList :
                             pTmr->pRmActiveEventList;
 
     while (pScan != NULL)
@@ -664,8 +742,8 @@ NvBool tmrCallbackOnList_IMPL
 )
 {
     NvBool onList = NV_FALSE;
-    PTMR_EVENT_PVT tmrScan;
-    PTMR_EVENT_PVT tmrList;
+    TMR_EVENT_PVT *tmrScan;
+    TMR_EVENT_PVT *tmrList;
 
     tmrList = pTmr->pRmActiveEventList;
 
@@ -685,13 +763,13 @@ NvBool tmrCallbackOnList_IMPL
 /*!
  * OBSOLETE: This will be removed very soon!
  */
-static PTMR_EVENT_PVT
+static TMR_EVENT_PVT *
 _tmrGetNextFreeCallback
 (
     OBJTMR *pTmr
 )
 {
-    PTMR_EVENT_PVT pEvent = NULL;
+    TMR_EVENT_PVT *pEvent = NULL;
 
     pEvent = pTmr->pRmCallbackFreeList_OBSOLETE;
     if (pEvent != NULL)
@@ -709,7 +787,7 @@ _tmrGetNextFreeCallback
  * Creates and inserts a node into the callback list.
  *
  *  @param[in]  pEvent     Callback memory structure, provided by user.
- *  @param[in]  Time       Absolute nanoseconds at which to call Proc.
+ *  @param[in]  Time       Absolute(for ptimer) or relative (for OS timer) nanoseconds at which to call Proc.
  *
  *  @returns               Status
  */
@@ -717,7 +795,7 @@ static NV_STATUS
 _tmrInsertCallback
 (
     OBJTMR         *pTmr,
-    PTMR_EVENT_PVT  pEvent,
+    TMR_EVENT_PVT *pEvent,
     NvU64           Time
 )
 {
@@ -725,7 +803,7 @@ _tmrInsertCallback
     OBJGPU *pGpu = ENG_GET_GPU(pTmr);
 
     // If this is a free callback
-    if (!pEvent->bInUse && !tmrEventOnList(pTmr, (PTMR_EVENT)pEvent))
+    if (!pEvent->bInUse && !tmrEventOnList(pTmr, (TMR_EVENT *)pEvent))
     {
         pEvent->timens    = Time;
 
@@ -756,21 +834,20 @@ _tmrInsertCallbackInList
 (
     OBJGPU         *pGpu,
     OBJTMR         *pTmr,
-    PTMR_EVENT_PVT  pEvent
+    TMR_EVENT_PVT *pEvent
 )
 {
-    PTMR_EVENT_PVT   pScan;
+    TMR_EVENT_PVT   *pScan;
     NvBool           bAddedAsHead  = NV_TRUE;
     NvU64            nextAlarmTime;
 
     NV_ASSERT(!pEvent->bInUse);
     pEvent->bInUse = NV_TRUE;
 
-    if (tmrIsOSTimer(pTmr, (PTMR_EVENT)pEvent))
+    if (tmrIsOSTimer(pTmr, (TMR_EVENT *)pEvent))
     {
         pEvent->pNext = pTmr->pRmActiveOSTimerEventList;
         pTmr->pRmActiveOSTimerEventList = pEvent;
-        pEvent->bInUse = NV_FALSE;
         return;
     }
 
@@ -849,30 +926,32 @@ _tmrInsertCallbackInList
 NV_STATUS tmrEventScheduleAbs_IMPL
 (
     OBJTMR     *pTmr,
-    PTMR_EVENT  pEventPublic,
-    NvU64       Time
+    TMR_EVENT  *pEventPublic,
+    NvU64       timeAbsNs
 )
 {
     NV_STATUS      rmStatus = NV_OK;
-    PTMR_EVENT_PVT pEvent = (PTMR_EVENT_PVT)pEventPublic;
+    TMR_EVENT_PVT *pEvent = (TMR_EVENT_PVT *)pEventPublic;
 
-    if ((pEventPublic != NULL) && (tmrIsOSTimer(pTmr, pEventPublic)))
+    if ((pEvent != NULL) && tmrIsOSTimer(pTmr, pEventPublic))
     {
-        NV_CHECK_OK(rmStatus, LEVEL_ERROR,
-            tmrEventScheduleAbsOSTimer_HAL(pTmr, pEventPublic, Time));
+        NvU64 timeRelNs, currentTime;
+
+        rmStatus = tmrGetCurrentTime(pTmr, &currentTime);
+        if (rmStatus != NV_OK)
+            return rmStatus;
+
         //
-        // Capture system time here, this will help in scheduling callbacks
-        // if there is a state unload before receiving the OS timer callback.
+        // If absolute time is less than current time, then timer has already
+        // expired. For this case, schedule timer with zero delay to trigger
+        // immediate callback.
         //
-        osGetCurrentTick(&pEvent->startTimeNs);
-        if (!tmrEventOnList(pTmr, pEventPublic))
-        {
-            _tmrInsertCallback(pTmr, pEvent, Time);
-        }
-        return rmStatus;
+        timeRelNs = (timeAbsNs > currentTime) ? (timeAbsNs - currentTime) : 0;
+        return tmrEventScheduleRel(pTmr, pEventPublic, timeRelNs);
     }
 
-    if (pEventPublic->pTimeProc == NULL && pEvent->pTimeProc_OBSOLETE == NULL)
+    if ((pEvent == NULL) || (pEventPublic->pTimeProc == NULL &&
+                             pEvent->pTimeProc_OBSOLETE == NULL))
     {
         //
         // Bug 372159: Not sure exactly how this is happening, but we are seeing
@@ -903,7 +982,7 @@ NV_STATUS tmrEventScheduleAbs_IMPL
         // removed the 250 ns threshold, so that we will always re-read the
         // current time after setting the alarm to prevent the wrap-around.
         //
-        rmStatus = _tmrInsertCallback(pTmr, pEvent, Time);
+        rmStatus = _tmrInsertCallback(pTmr, pEvent, timeAbsNs);
     }
 
     return rmStatus;
@@ -923,7 +1002,7 @@ NV_STATUS tmrScheduleCallbackAbs_IMPL
     NvU32               ChId
 )
 {
-    PTMR_EVENT_PVT tmrInsert;
+    TMR_EVENT_PVT *tmrInsert;
     // Get a free callback from the free list.
     if(pTmr->pRmCallbackFreeList_OBSOLETE == NULL)
     {
@@ -939,7 +1018,7 @@ NV_STATUS tmrScheduleCallbackAbs_IMPL
             tmrInsert->super.flags = Flags;
             tmrInsert->super.chId = ChId;
 
-            return tmrEventScheduleAbs(pTmr, (PTMR_EVENT)tmrInsert, Time);
+            return tmrEventScheduleAbs(pTmr, (TMR_EVENT *)tmrInsert, Time);
         }
         else
         {
@@ -955,7 +1034,7 @@ NV_STATUS tmrScheduleCallbackAbs_IMPL
 }
 
 /*!
- *  Searches specified lists for PTMR_EVENT  associated with Object and
+ *  Searches specified lists for TMR_EVENT* associated with Object and
  *  removes it.
  *
  *  @param[in]  Object  Unique identifier based on TMR_POBJECT_BASE (tmr.h)
@@ -968,9 +1047,9 @@ static void _tmrScanCallback
     void   *pObject
 )
 {
-    PTMR_EVENT_PVT  tmrScan;
-    PTMR_EVENT_PVT  tmrNext;
-    PTMR_EVENT_PVT  tmrCurrent;
+    TMR_EVENT_PVT *tmrScan;
+    TMR_EVENT_PVT *tmrNext;
+    TMR_EVENT_PVT *tmrCurrent;
 
     //
     // Start at the beginning of the callback list.
@@ -1058,15 +1137,16 @@ static void
 _tmrScanCallbackOSTimer
 (
     OBJTMR *pTmr,
-    PTMR_EVENT_PVT  pEvent
+    TMR_EVENT_PVT *pEvent
 )
 {
-    PTMR_EVENT_PVT  pCurrent = pTmr->pRmActiveOSTimerEventList;
+    TMR_EVENT_PVT *pCurrent = pTmr->pRmActiveOSTimerEventList;
 
     if (pCurrent == pEvent)
     {
         pTmr->pRmActiveOSTimerEventList = pCurrent->pNext;
         pEvent->pNext = NULL;
+        pEvent->bInUse = NV_FALSE;
         return;
     }
 
@@ -1076,6 +1156,7 @@ _tmrScanCallbackOSTimer
         {
             pCurrent->pNext = pEvent->pNext;
             pEvent->pNext = NULL;
+            pEvent->bInUse = NV_FALSE;
             break;
         }
         pCurrent = pCurrent->pNext;
@@ -1105,12 +1186,12 @@ _tmrGetNextAlarmTime
  * Return the very next callback to be scheduled, removing it from the list
  * and marking it as free (only "In Use" when in the list)
  */
-static PTMR_EVENT_PVT  _tmrPullCallbackFromHead
+static TMR_EVENT_PVT * _tmrPullCallbackFromHead
 (
     OBJTMR         *pTmr
 )
 {
-    PTMR_EVENT_PVT tmrDelete = pTmr->pRmActiveEventList;
+    TMR_EVENT_PVT *tmrDelete = pTmr->pRmActiveEventList;
     if (tmrDelete)
     {
         // remove from callbackList
@@ -1174,7 +1255,7 @@ tmrCallExpiredCallbacks_IMPL
 {
     NvU64           currentTime = 0;
     NvU64           nextAlarmTime;
-    PTMR_EVENT_PVT  pEvent;
+    TMR_EVENT_PVT  *pEvent;
     NV_STATUS       rmStatus;
     NvBool          bProccessedCallback = NV_FALSE;
 
@@ -1199,34 +1280,27 @@ tmrCallExpiredCallbacks_IMPL
             // Pull from head of list.
             pEvent = _tmrPullCallbackFromHead(pTmr);
 
-            if (pEvent &&
-                ((pEvent->super.pTimeProc != NULL) ||
-                (pEvent->pTimeProc_OBSOLETE != NULL)) )
+            if (pEvent != NULL)
             {
                 // Call callback.  This could insert a new callback into the list.
-                if (pEvent->bLegacy)
+                if (pEvent->bLegacy && pEvent->pTimeProc_OBSOLETE != NULL)
                 {
                     pEvent->pTimeProc_OBSOLETE(pGpu, pTmr, pEvent->super.pUserData);
+                    bProccessedCallback = NV_TRUE;
+                }
+                else if (!pEvent->bLegacy && pEvent->super.pTimeProc != NULL)
+                {
+                    pEvent->super.pTimeProc(pGpu, pTmr, (TMR_EVENT *)pEvent);
+                    bProccessedCallback = NV_TRUE;
                 }
                 else
                 {
-                    pEvent->super.pTimeProc(pGpu, pTmr, (PTMR_EVENT)pEvent);
+                    NV_ASSERT_FAILED("Attempting to execute callback with NULL procedure.");
                 }
-
-                bProccessedCallback = NV_TRUE;
             }
             else
             {
-                //
-                // Bug 372159: Hopefully by checking that the callback procedure
-                // is not NULL in tmrEventScheduleAbs() we should never hit
-                // this point, but this is just to be certain.  If you hit this
-                // assert please update Bug 3721259 with pertinent details
-                // (Swak, !stacks, what you were developing/testing, etc.).
-                //
-                NV_ASSERT_FAILED(
-                    "Attempting to execute callback with NULL procedure.  "
-                    "Please update Bug 372159 with appropriate information.");
+                NV_ASSERT_FAILED("Attempting to execute callback with NULL timer event.");
             }
         }
 
@@ -1258,7 +1332,7 @@ _tmrStateLoadCallbacks
 )
 {
     NvU64 nextAlarmTime = 0;
-    PTMR_EVENT_PVT pScan = pTmr->pRmActiveOSTimerEventList;
+    TMR_EVENT_PVT *pScan = pTmr->pRmActiveOSTimerEventList;
 
     if (tmrEventsExist(pTmr))
     {
@@ -1291,8 +1365,8 @@ _tmrStateLoadCallbacks
         // Capture system time here, this will help in scheduling callbacks
         // if there is a state unload before receiving the OS timer callback.
         //
-        osGetCurrentTick(&pScan->startTimeNs);
-        tmrEventScheduleAbsOSTimer_HAL(pTmr, (PTMR_EVENT)pScan, pScan->timens);
+        pScan->startTimeNs = osGetCurrentTick();
+        tmrEventScheduleRelOSTimer_HAL(pTmr, (TMR_EVENT *)pScan, pScan->timens);
         pScan = pScan->pNext;
     }
 }
@@ -1388,7 +1462,7 @@ tmrCheckCallbacksReleaseSem_IMPL
     NvU32   chId
 )
 {
-    PTMR_EVENT_PVT pScan;
+    TMR_EVENT_PVT *pScan;
 
     for (pScan = pTmr->pRmActiveEventList; pScan != NULL; pScan = pScan->pNext)
     {
@@ -1518,9 +1592,17 @@ tmrDiffExceedsTime_IMPL
     return bRetVal;
 }
 
-/*!
- * TODO: document
- */
+NV_STATUS
+tmrStatePreInitLocked_IMPL
+(
+    OBJGPU *pGpu,
+    OBJTMR *pTmr
+)
+{
+
+    return NV_OK;
+}
+
 NV_STATUS
 tmrStateInitLocked_IMPL
 (
@@ -1560,7 +1642,7 @@ tmrStateUnload_IMPL
     NvU32   flags
 )
 {
-    PTMR_EVENT_PVT pScan = pTmr->pRmActiveOSTimerEventList;
+    TMR_EVENT_PVT *pScan = pTmr->pRmActiveOSTimerEventList;
     NvU64 currentSysTime, elapsedTime;
 
     // Disable Timer interrupt.
@@ -1573,7 +1655,7 @@ tmrStateUnload_IMPL
     //
     while (pScan != NULL)
     {
-        osGetCurrentTick(&currentSysTime);
+        currentSysTime = osGetCurrentTick();
         //
         // If somehow any of the time difference is negative,
         // we will use the  original time duration.
@@ -1587,7 +1669,7 @@ tmrStateUnload_IMPL
             }
         }
 
-        tmrEventCancelOSTimer_HAL(pTmr, (PTMR_EVENT)pScan);
+        tmrEventCancelOSTimer_HAL(pTmr, (TMR_EVENT *)pScan);
         pScan = pScan->pNext;
     }
     return NV_OK;
@@ -1661,7 +1743,17 @@ tmrapiDeregisterEvents_IMPL(TimerApi *pTimerApi)
     // Validate the timer event
     while (pNotifyEvent != NULL)
     {
-        tmrCancelCallback(pTmr, pNotifyEvent);
+        //
+        // TimerApi events are only set through NV0004_CTRL_CMD_TMR_SET_ALARM_NOTIFY
+        // which only schedules TMR_EVENT type callbacks. So only call the new API.
+        //
+        TMR_EVENT *pTmrEvent = pNotifyEvent->pTmrEvent;
+        if (pTmrEvent != NULL)
+        {
+            tmrEventDestroy(pTmr, pTmrEvent);
+            pNotifyEvent->pGpu = NULL;
+            pNotifyEvent->pTmrEvent = NULL;
+        }
 
         pNotifyEvent = pNotifyEvent->Next;
     }
@@ -1692,7 +1784,7 @@ static NV_STATUS _tmrCallbackWrapperfunction
 (
     OBJGPU *pGpu,
     OBJTMR *pTmr,
-    PTMR_EVENT pEvent
+    TMR_EVENT *pEvent
 )
 {
     wrapperStorage_t *pObj_Inner = (wrapperStorage_t *)pEvent->pUserData;
@@ -1733,7 +1825,7 @@ tmrCtrlCmdEventCreate
 )
 {
     NV_STATUS         rc;
-    PTMR_EVENT        pEvent;
+    TMR_EVENT        *pEvent;
     wrapperStorage_t *pWrapper;
     OBJTMR *pTmr = GPU_GET_TIMER(pGpu);
 
@@ -1773,9 +1865,9 @@ tmrCtrlCmdEventSchedule
 {
     NV_STATUS rc;
     OBJTMR    *pTmr   = GPU_GET_TIMER(pGpu);
-    PTMR_EVENT pEvent = (PTMR_EVENT)NvP64_VALUE(pParams->pEvent);
+    TMR_EVENT *pEvent = (TMR_EVENT *)NvP64_VALUE(pParams->pEvent);
 
-    if(pParams->bUseTimeAbs)
+    if (pParams->bUseTimeAbs)
     {
         rc = tmrEventScheduleAbs(pTmr, pEvent, pParams->timeNs);
     }
@@ -1800,7 +1892,7 @@ tmrCtrlCmdEventCancel
 )
 {
     OBJTMR    *pTmr   = GPU_GET_TIMER(pGpu);
-    PTMR_EVENT pEvent = (PTMR_EVENT)NvP64_VALUE(pParams->pEvent);
+    TMR_EVENT *pEvent = (TMR_EVENT *)NvP64_VALUE(pParams->pEvent);
     tmrEventCancel(pTmr, pEvent);
 
     return NV_OK;
@@ -1820,7 +1912,7 @@ tmrCtrlCmdEventDestroy
 )
 {
     OBJTMR    *pTmr   = GPU_GET_TIMER(pGpu);
-    PTMR_EVENT pEvent = (PTMR_EVENT)NvP64_VALUE(pParams->pEvent);
+    TMR_EVENT *pEvent = (TMR_EVENT *)NvP64_VALUE(pParams->pEvent);
 
     // Free our temporary wrapper storage
     portMemFree(pEvent->pUserData);
@@ -1834,18 +1926,18 @@ NV_STATUS tmrEventServiceTimer_IMPL
 (
     OBJGPU             *pGpu,
     OBJTMR             *pTmr,
-    PTMR_EVENT          pPublicEvent
+    TMR_EVENT          *pPublicEvent
 )
 {
-    PTMR_EVENT_PVT pEvent = (PTMR_EVENT_PVT)pPublicEvent;
+    TMR_EVENT_PVT *pEvent = (TMR_EVENT_PVT *)pPublicEvent;
     NV_STATUS status      = NV_ERR_INVALID_REQUEST;
 
-    if ((pEvent == NULL) || !tmrIsOSTimer(pTmr, (PTMR_EVENT)pEvent))
+    if ((pEvent == NULL) || !tmrIsOSTimer(pTmr, (TMR_EVENT *)pEvent))
     {
         return status;
     }
 
-    if (tmrEventOnList(pTmr, (PTMR_EVENT)pEvent))
+    if (tmrEventOnList(pTmr, (TMR_EVENT *)pEvent))
     {
         _tmrScanCallbackOSTimer(pTmr, pEvent);
         status = tmrEventServiceOSTimerCallback_HAL(pGpu, pTmr, pPublicEvent);
