@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2020-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2020-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -32,9 +32,11 @@
 #include "core/locks.h"
 #include "kernel/gpu/rc/kernel_rc.h"
 #include "diagnostics/gpu_acct.h"
+#include "gpu/device/device.h"
 #include "Nvcm.h"
 #include "gpu/bus/third_party_p2p.h"
 #include "gpu/bus/kern_bus.h"
+#include "platform/sli/sli.h"
 
 #include "class/cl0040.h" // NV01_MEMORY_LOCAL_USER
 
@@ -125,35 +127,38 @@ _vidmemPmaAllocate
     MemoryManager               *pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
     PMA                         *pPma           = &pHeap->pmaObject;
     NvU64                        size           = 0;
-    NvU32                        pageCount;
+    NvU32                        pageCount      = 0;
     NvU32                        pmaInfoSize;
     NvU64                        pageSize;
     NV_STATUS                    status;
     NvU64                        sizeAlign    = 0;
     PMA_ALLOCATION_OPTIONS       allocOptions = {0};
-    NvBool                       bContig      = !FLD_TEST_DRF(OS32, _ATTR,
-                                                   _PHYSICALITY, _NONCONTIGUOUS,
-                                                   pAllocData->attr);
+    NvBool                       bContig;
     NvU32                        subdevInst   = gpumgrGetSubDeviceInstanceFromGpu(pGpu);
-    NvBool                       bCompressed  = !FLD_TEST_DRF(OS32, _ATTR, _COMPR,
-                                                  _NONE, pAllocData->attr);
-    KernelBus                   *pKernelBus = GPU_GET_KERNEL_BUS(pGpu);
-    NvU32                        gfid;
+    NvU32                        pmaConfig    = PMA_QUERY_NUMA_ENABLED;
 
-    // LOCK: acquire device lock
-    status = rmDeviceGpuLocksAcquire(pGpu, GPUS_LOCK_FLAGS_NONE,
-                                     RM_LOCK_MODULES_MEM_PMA);
-    NV_ASSERT_OR_RETURN(status == NV_OK, status);
+    status = pmaQueryConfigs(pPma, &pmaConfig);
+    NV_ASSERT(status == NV_OK);
 
-    if (bCompressed &&
-        (vgpuGetCallingContextGfid(pGpu, &gfid) == NV_OK) &&
-        pKernelBus->bar1[gfid].bStaticBar1Enabled)
+    //
+    // In NUMA platforms, contig memory is allocated using page order from
+    // kernel and that could lead to memory wastage when the size is not
+    // naturally aligned to page order. Prefer non-contig when clients
+    // are okay with NON_CONTIG.
+    //
+    if ((status == NV_OK) && (pmaConfig & PMA_QUERY_NUMA_ENABLED))
     {
-        // Override the attr to use 2MB page size
-        pAllocData->attr = FLD_SET_DRF(OS32, _ATTR, _PAGE_SIZE, _HUGE, pAllocData->attr);
-
-        NV_PRINTF(LEVEL_INFO,
-                  "Overrode the page size to 2MB on this compressed vidmem for the static bar1\n");
+        bContig =
+            !FLD_TEST_DRF(OS32, _ATTR, _PHYSICALITY,
+                                _ALLOW_NONCONTIGUOUS, pAllocData->attr) &&
+            !FLD_TEST_DRF(OS32, _ATTR, _PHYSICALITY,
+                          _NONCONTIGUOUS, pAllocData->attr);
+    }
+    else
+    {
+        bContig = !FLD_TEST_DRF(OS32, _ATTR,
+                                _PHYSICALITY, _NONCONTIGUOUS,
+                                pAllocData->attr);
     }
 
     NV_PRINTF(LEVEL_INFO, "PMA input\n");
@@ -200,16 +205,11 @@ _vidmemPmaAllocate
     // Bug:2451834, gpuCheckPageRetirementSupport should not be called outside
     // RM lock.
     //
-    if (pGpu->getProperty(pGpu, PDB_PROP_GPU_ALLOW_PAGE_RETIREMENT) &&
-           gpuCheckPageRetirementSupport_HAL(pGpu) &&
-           FLD_TEST_DRF(OS32, _ATTR2, _BLACKLIST, _OFF, pAllocData->attr2))
+    if (FLD_TEST_DRF(OS32, _ATTR2, _BLACKLIST, _OFF, pAllocData->attr2))
     {
         allocOptions.flags |= PMA_ALLOCATE_TURN_BLACKLIST_OFF;
     }
 
-    // UNLOCK: release device lock
-    rmDeviceGpuLocksRelease(pGpu, GPUS_LOCK_FLAGS_NONE, NULL);
-
     NV_ASSERT_OR_RETURN(NV_OK == status, status);
 
     // RM allocations are always pinned.
@@ -265,8 +265,19 @@ _vidmemPmaAllocate
     allocOptions.flags |= PMA_ALLOCATE_FORCE_ALIGNMENT;
     allocOptions.alignment = NV_MAX(sizeAlign, pageSize);
 
+    if (FLD_TEST_DRF(OS32, _ATTR2, _ENABLE_LOCALIZED_MEMORY, _UGPU0, pAllocData->attr2))
+    {
+        allocOptions.flags |= PMA_ALLOCATE_LOCALIZED_UGPU0;
+    }
+    else if (FLD_TEST_DRF(OS32, _ATTR2, _ENABLE_LOCALIZED_MEMORY, _UGPU1, pAllocData->attr2))
+    {
+        allocOptions.flags |= PMA_ALLOCATE_LOCALIZED_UGPU1;
+    }
 
     // Get the number of pages to be allocated by PMA
+    NV_CHECK_OR_RETURN(LEVEL_ERROR,
+        (NV_DIV_AND_CEIL(size, pageSize) <= NV_U32_MAX),
+        NV_ERR_NO_MEMORY);
     pageCount = (NvU32) NV_DIV_AND_CEIL(size, pageSize);
 
 retry_alloc:
@@ -278,7 +289,12 @@ retry_alloc:
     }
     else
     {
-        pmaInfoSize = sizeof(PMA_ALLOC_INFO) + ((pageCount - 1) * sizeof(NvU64));
+        NV_CHECK_OR_RETURN(LEVEL_ERROR,
+            portSafeMulU32((pageCount - 1), (sizeof(NvU64)), &pmaInfoSize),
+            NV_ERR_NO_MEMORY);
+        NV_CHECK_OR_RETURN(LEVEL_ERROR,
+            portSafeAddU32(pmaInfoSize, (sizeof(PMA_ALLOC_INFO)), &pmaInfoSize),
+            NV_ERR_NO_MEMORY);
     }
 
     // Alloc the tracking structure and store the values in it.
@@ -397,7 +413,8 @@ vidmemGetHeap
 (
     OBJGPU  *pGpu,
     Device  *pDevice,
-    NvBool   bSubheap
+    NvBool   bSubheap,
+    NvBool   bForceGlobalHeap
 )
 {
     MemoryManager *pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
@@ -411,7 +428,7 @@ vidmemGetHeap
         return pHeap;
     }
 
-    if (IS_MIG_IN_USE(pGpu))
+    if (IS_MIG_IN_USE(pGpu) && !bForceGlobalHeap)
     {
         KernelMIGManager *pKernelMIGManager = GPU_GET_KERNEL_MIG_MANAGER(pGpu);
         Heap *pMemoryPartitionHeap = NULL;
@@ -443,20 +460,45 @@ vidmemCopyConstruct
     RS_RES_ALLOC_PARAMS_INTERNAL    *pParams
 )
 {
-    Memory    *pMemorySrc            = dynamicCast(pParams->pSrcRef->pResource, Memory);
-    OBJGPU    *pGpu                  = pMemorySrc->pGpu;
-    NV_STATUS  status;
+    Memory            *pMemorySrc   = dynamicCast(pParams->pSrcRef->pResource, Memory);
+    OBJGPU            *pGpu         = pMemorySrc->pGpu;
+    MEMORY_DESCRIPTOR *pMemDesc     = pMemorySrc->pMemDesc;
+    NV_STATUS          status       = NV_ERR_INVALID_ARGUMENT;
+    KernelBus         *pKernelBus    = GPU_GET_KERNEL_BUS(pGpu);
 
-    NV_ASSERT_OR_RETURN(!memdescGetCustomHeap(pMemorySrc->pMemDesc), NV_ERR_INVALID_ARGUMENT);
+    // No flags specified on the initial static BAR1 mapping 
+    status = kbusIncreaseStaticBar1Refcount_HAL(pGpu, pKernelBus,
+                   pMemDesc, BUS_MAP_FB_FLAGS_NONE);
 
-    SLI_LOOP_START(SLI_LOOP_FLAGS_BC_ONLY)
-        MEMORY_DESCRIPTOR *pSrcMemDesc = memdescGetMemDescFromGpu(pMemorySrc->pMemDesc, pGpu);
-        status = heapReference(pGpu, pSrcMemDesc->pHeap, pMemorySrc->HeapOwner,
-                               pSrcMemDesc);
-        NV_ASSERT(status == NV_OK);
-    SLI_LOOP_END
+    if (status == NV_OK)
+    {
+        // nothing
+    }
+    else if (status == NV_ERR_NOT_SUPPORTED)
+    {
+        status = NV_OK;
+    }
+    else
+    {
+        return status;
+    }
 
-    return NV_OK;
+    switch (memdescGetCustomHeap(pMemDesc))
+    {
+        case MEMDESC_CUSTOM_HEAP_NONE:
+            SLI_LOOP_START(SLI_LOOP_FLAGS_BC_ONLY)
+                MEMORY_DESCRIPTOR *pSrcMemDesc = memdescGetMemDescFromGpu(pMemorySrc->pMemDesc, pGpu);
+                status = heapReference(pGpu, pSrcMemDesc->pHeap, pMemorySrc->HeapOwner,
+                                       pSrcMemDesc);
+                NV_ASSERT_OK_OR_RETURN(status);
+            SLI_LOOP_END
+            break;
+        default:
+            NV_ASSERT_OR_RETURN(0, NV_ERR_INVALID_ARGUMENT);
+            break;
+    }
+
+    return status;
 }
 
 /*!
@@ -498,12 +540,14 @@ vidmemConstruct_IMPL
     MEMORY_ALLOCATION_REQUEST   *pAllocRequest         = &allocRequest;
     OBJGPU                      *pGpu                  = pMemory->pGpu;
     MemoryManager               *pMemoryManager        = GPU_GET_MEMORY_MANAGER(pGpu);
-    Heap                        *pHeap;
+    Heap                        *pHeap                 = NULL;
     NvBool                       bSubheap              = NV_FALSE;
+    NvBool                       bRsvdHeap             = NV_FALSE;
     MEMORY_DESCRIPTOR           *pTopLevelMemDesc      = NULL;
     MEMORY_DESCRIPTOR           *pTempMemDesc          = NULL;
     HWRESOURCE_INFO              hwResource;
     RsClient                    *pRsClient             = pCallContext->pClient;
+    RmClient                    *pRmClient             = dynamicCast(pRsClient, RmClient);
     RsResourceRef               *pResourceRef          = pCallContext->pResourceRef;
     RsResourceRef               *pDeviceRef;
     Device                      *pDevice;
@@ -518,6 +562,9 @@ vidmemConstruct_IMPL
     FB_ALLOC_INFO               *pFbAllocInfo          = NULL;
     FB_ALLOC_PAGE_FORMAT        *pFbAllocPageFormat    = NULL;
     NV_STATUS                    rmStatus              = NV_OK;
+    KernelBus                   *pKernelBus            = GPU_GET_KERNEL_BUS(pGpu);
+
+    NV_ASSERT_OR_RETURN(pRmClient != NULL, NV_ERR_INVALID_CLIENT);
 
     NV_ASSERT_OK_OR_RETURN(
         refFindAncestorOfType(pResourceRef, classId(Device), &pDeviceRef));
@@ -539,7 +586,15 @@ vidmemConstruct_IMPL
         goto done;
     }
 
-    NV_CHECK_OK_OR_RETURN(LEVEL_WARNING, stdmemValidateParams(pGpu, hClient, pAllocData));
+    if (FLD_TEST_DRF(OS32, _ATTR, _PHYSICALITY, _DEFAULT, pAllocData->attr))
+    {
+        pAllocData->attr =
+            FLD_SET_DRF_NUM(OS32, _ATTR, _PHYSICALITY,
+                            pDevice->defaultVidmemPhysicalityOverride,
+                            pAllocData->attr);
+    }
+
+    NV_CHECK_OK_OR_RETURN(LEVEL_WARNING, stdmemValidateParams(pGpu, pRmClient, pAllocData));
     NV_CHECK_OR_RETURN(LEVEL_WARNING,
                        DRF_VAL(OS32, _ATTR, _LOCATION, pAllocData->attr) == NVOS32_ATTR_LOCATION_VIDMEM &&
                            !(pAllocData->flags & NVOS32_ALLOC_FLAGS_VIRTUAL),
@@ -547,10 +602,18 @@ vidmemConstruct_IMPL
 
     stdmemDumpInputAllocParams(pAllocData, pCallContext);
 
+    if (pCallContext->secInfo.privLevel >= RS_PRIV_LEVEL_KERNEL)
+    {
+        bRsvdHeap = FLD_TEST_DRF(OS32, _ATTR, _ALLOCATE_FROM_RESERVED_HEAP, _YES, pAllocData->attr);
+    }
+
     bSubheap = FLD_TEST_DRF(OS32, _ATTR2, _ALLOCATE_FROM_SUBHEAP, _YES, pAllocData->attr2);
-    pHeap = vidmemGetHeap(pGpu, pDevice, bSubheap);
+    pHeap = vidmemGetHeap(pGpu, pDevice, bSubheap, bRsvdHeap);
     NV_CHECK_OR_RETURN(LEVEL_INFO, pHeap != NULL, NV_ERR_INVALID_STATE);
 
+    attr  = pAllocData->attr;
+    attr2 = pAllocData->attr2;
+
     if (gpuIsCCorApmFeatureEnabled(pGpu) &&
         !FLD_TEST_DRF(OS32, _ATTR2, _MEMORY_PROTECTION, _UNPROTECTED, pAllocData->attr2))
     {
@@ -585,6 +648,7 @@ vidmemConstruct_IMPL
 
     bIsPmaAlloc = memmgrIsPmaInitialized(pMemoryManager) &&
                   !bSubheap &&
+                  !bRsvdHeap &&
                   !(pAllocData->flags & NVOS32_ALLOC_FLAGS_WPR1) &&
                   !(pAllocData->flags & NVOS32_ALLOC_FLAGS_WPR2) &&
                   (!(pAllocData->flags & NVOS32_ALLOC_FLAGS_FIXED_ADDRESS_ALLOCATE) ||
@@ -592,15 +656,25 @@ vidmemConstruct_IMPL
 
     // Scrub-on-free is not supported by heap. Make sure clients don't get unscrubbed allocations
     NV_CHECK_OR_RETURN(LEVEL_WARNING,
-        !memmgrIsScrubOnFreeEnabled(pMemoryManager) || bIsPmaAlloc || bSubheap,
+        !memmgrIsScrubOnFreeEnabled(pMemoryManager) || bIsPmaAlloc || bSubheap || bRsvdHeap,
         NV_ERR_INVALID_STATE);
 
+    if (!FLD_TEST_DRF(OS32, _ATTR2, _ENABLE_LOCALIZED_MEMORY, _DEFAULT, pAllocData->attr2))
+    {
+        if (!bIsPmaAlloc)
+        {
+            // heap does not support localized allocations
+            rmStatus = NV_ERR_NOT_SUPPORTED;
+            goto done;
+        }
+    }
+
     // Get the allocation from PMA if enabled.
     if (bIsPmaAlloc)
     {
         SLI_LOOP_START(SLI_LOOP_FLAGS_BC_ONLY | SLI_LOOP_FLAGS_IGNORE_REENTRANCY)
         pAllocRequest->pGpu = pGpu;
-        rmStatus = _vidmemPmaAllocate(vidmemGetHeap(pGpu, pDevice, NV_FALSE), pAllocRequest);
+        rmStatus = _vidmemPmaAllocate(vidmemGetHeap(pGpu, pDevice, NV_FALSE, NV_FALSE), pAllocRequest);
         if (NV_OK != rmStatus)
             SLI_LOOP_GOTO(done);
         SLI_LOOP_END;
@@ -668,8 +742,9 @@ vidmemConstruct_IMPL
     {
         MEMORY_DESCRIPTOR *pPrev = NULL;
 
-        // VGPU won't run in SLI. So no need to set subheap flags in memdesc.
+        // VGPU won't run in SLI. So no need to set subheap and bRsvdHeap flags in memdesc.
         NV_ASSERT(!bSubheap);
+        NV_ASSERT(!bRsvdHeap);
 
         // Create dummy top level memdesc
         rmStatus = memdescCreate(&pTopLevelMemDesc, pGpu, RM_PAGE_SIZE, 0,
@@ -702,7 +777,7 @@ vidmemConstruct_IMPL
                 SLI_LOOP_GOTO(done);
 
             rmStatus = vidmemAllocResources(pGpu, pMemoryManager, pAllocRequest, pFbAllocInfo,
-                                            vidmemGetHeap(pGpu, pDevice, NV_FALSE));
+                                            vidmemGetHeap(pGpu, pDevice, NV_FALSE, NV_FALSE));
             if (rmStatus != NV_OK)
                 SLI_LOOP_GOTO(done);
 
@@ -798,49 +873,51 @@ vidmemConstruct_IMPL
         SLI_LOOP_END;
     }
 
-    //
-    // Video memory is always locally transparently cached.  It does not require
-    // any cache managment.  Marked cached unconditionally.  Non-coherent peer
-    // caching is handled with an override at mapping time.
-    //
-    if (DRF_VAL(OS32, _ATTR2, _GPU_CACHEABLE, pAllocData->attr2) ==
-        NVOS32_ATTR2_GPU_CACHEABLE_DEFAULT)
     {
-        pAllocData->attr2 = FLD_SET_DRF(OS32, _ATTR2, _GPU_CACHEABLE, _YES,
-                                        pAllocData->attr2);
-    }
-    gpuCacheAttrib = NV_MEMORY_CACHED;
-
-    // ClientDB can set the pagesize for memdesc.
-    // With GPU SMMU mapping, this needs to be set on the SMMU memdesc.
-    // So SMMU allocation should happen before memConstructCommon()
-    // Eventaully SMMU allocation will be part of memdescAlloc().
-
-    //
-    // There are a few cases where the heap will return an existing
-    // memdesc.  Only update attributes if it is new.
-    //
-    // @todo attr tracking should move into heapAlloc
-    //
-    if (pTempMemDesc->RefCount == 1)
-    {
-        SLI_LOOP_START(SLI_LOOP_FLAGS_BC_ONLY | SLI_LOOP_FLAGS_IGNORE_REENTRANCY);
-        memdescSetGpuCacheAttrib(memdescGetMemDescFromGpu(pTopLevelMemDesc, pGpu), gpuCacheAttrib);
-        SLI_LOOP_END;
-
-
-        // An SMMU mapping will be added to FB allocations in the following cases:
-        // 1. RM clients forcing SMMU mapping via flags
-        //    GPU Arch verification with VPR is one such usecase.
-
-        if (FLD_TEST_DRF(OS32, _ATTR2, _SMMU_ON_GPU, _ENABLE, pAllocData->attr2))
+        //
+        // Video memory is always locally transparently cached.  It does not require
+        // any cache management.  Marked cached unconditionally.  Non-coherent peer
+        // caching is handled with an override at mapping time.
+        //
+        if (DRF_VAL(OS32, _ATTR2, _GPU_CACHEABLE, pAllocData->attr2) ==
+            NVOS32_ATTR2_GPU_CACHEABLE_DEFAULT)
         {
-            NV_ASSERT_FAILED("SMMU mapping allocation is not supported for ARMv7");
-            rmStatus = NV_ERR_NOT_SUPPORTED;
+            pAllocData->attr2 = FLD_SET_DRF(OS32, _ATTR2, _GPU_CACHEABLE, _YES,
+                                            pAllocData->attr2);
+        }
+        gpuCacheAttrib = NV_MEMORY_CACHED;
 
-            memdescFree(pTopLevelMemDesc);
-            memdescDestroy(pTopLevelMemDesc);
-            goto done;
+        // ClientDB can set the pagesize for memdesc.
+        // With GPU SMMU mapping, this needs to be set on the SMMU memdesc.
+        // So SMMU allocation should happen before memConstructCommon()
+        // Eventually SMMU allocation will be part of memdescAlloc().
+
+        //
+        // There are a few cases where the heap will return an existing
+        // memdesc.  Only update attributes if it is new.
+        //
+        // @todo attr tracking should move into heapAlloc
+        //
+        if (pTempMemDesc->RefCount == 1)
+        {
+            SLI_LOOP_START(SLI_LOOP_FLAGS_BC_ONLY | SLI_LOOP_FLAGS_IGNORE_REENTRANCY);
+            memdescSetGpuCacheAttrib(memdescGetMemDescFromGpu(pTopLevelMemDesc, pGpu), gpuCacheAttrib);
+            SLI_LOOP_END;
+
+
+            // An SMMU mapping will be added to FB allocations in the following cases:
+            // 1. RM clients forcing SMMU mapping via flags
+            //    GPU Arch verification with VPR is one such usecase.
+
+            if (FLD_TEST_DRF(OS32, _ATTR2, _SMMU_ON_GPU, _ENABLE, pAllocData->attr2))
+            {
+                NV_ASSERT_FAILED("SMMU mapping allocation is not supported for ARMv7");
+                rmStatus = NV_ERR_NOT_SUPPORTED;
+
+                memdescFree(pTopLevelMemDesc);
+                memdescDestroy(pTopLevelMemDesc);
+                goto done;
+            }
         }
     }
 
@@ -868,6 +945,13 @@ vidmemConstruct_IMPL
     // XXX: This is a hack for now. No Hw resources are assumed to be used in the call.
     // The host is only requested to make an alias to the allocated heap.
 
+    //
+    // Heap alloc may allocate non-contiguous pages when it is not able to
+    // find contiguous pages. Replace this field before passing to RPC.
+    //
+    attr = (pAllocData->attr &  DRF_SHIFTMASK(NVOS32_ATTR_PHYSICALITY)) |
+           (attr             & ~DRF_SHIFTMASK(NVOS32_ATTR_PHYSICALITY));
+
     if (!IS_GSP_CLIENT(pGpu))
     {
         NV_RM_RPC_ALLOC_VIDMEM(pGpu,
@@ -953,6 +1037,34 @@ vidmemConstruct_IMPL
         }
     }
 
+    // No flags specified on the initial static BAR1 mapping 
+    rmStatus = kbusIncreaseStaticBar1Refcount_HAL(pGpu, pKernelBus,
+                   pMemory->pMemDesc, BUS_MAP_FB_FLAGS_NONE);
+
+    if (rmStatus == NV_OK)
+    {
+        // nothing
+    }
+    else if (rmStatus == NV_ERR_NOT_SUPPORTED)
+    {
+        rmStatus = NV_OK;
+    }
+    else
+    {
+        if (pMemory->bRpcAlloc)
+        {
+            NV_STATUS status = NV_OK;
+            NV_RM_RPC_FREE(pGpu, hClient, hParent,
+                           pAllocRequest->hMemory, status);
+            NV_ASSERT(status == NV_OK);
+        }
+        memDestructCommon(pMemory);
+        memdescFree(pTopLevelMemDesc);
+        memdescDestroy(pTopLevelMemDesc);
+        pTopLevelMemDesc = NULL;
+        goto done;
+    }
+
     pAllocData->size = sizeOut;
     pAllocData->offset = offsetOut;
 
@@ -976,7 +1088,7 @@ done:
         SLI_LOOP_START(SLI_LOOP_FLAGS_BC_ONLY | SLI_LOOP_FLAGS_IGNORE_REENTRANCY)
 
         if (pAllocRequest->pPmaAllocInfo[gpumgrGetSubDeviceInstanceFromGpu(pGpu)])
-            vidmemPmaFree(pGpu, vidmemGetHeap(pGpu, pDevice, NV_FALSE),
+            vidmemPmaFree(pGpu, vidmemGetHeap(pGpu, pDevice, NV_FALSE, NV_FALSE),
                           pAllocRequest->pPmaAllocInfo[gpumgrGetSubDeviceInstanceFromGpu(pGpu)], 0);
         SLI_LOOP_END;
     }
@@ -993,15 +1105,26 @@ vidmemDestruct_IMPL
     Memory             *pMemory        = staticCast(pVideoMemory, Memory);
     OBJGPU             *pGpu           = pMemory->pGpu;
     MEMORY_DESCRIPTOR  *pMemDesc       = pMemory->pMemDesc;
+    MEMDESC_CUSTOM_HEAP customHeap;
 
     // Free any association of the memory with existing third-party p2p object
     CliUnregisterMemoryFromThirdPartyP2P(pMemory);
 
     memDestructCommon(pMemory);
 
+    //
+    // static BAR1: memory must be released from the static BAR1 mapping
+    // to restore the static BAR1 mapping to a default state since it can
+    // immediately be allocated by UVM after PMA free
+    //
+    (void)kbusDecreaseStaticBar1Refcount_HAL(pGpu,
+                    GPU_GET_KERNEL_BUS(pGpu), pMemDesc,
+                    NULL);
+
     // free the video memory based on how it was alloced ... a non-zero
     // heapOwner indicates it was heapAlloc-ed.
-    if (!memdescGetCustomHeap(pMemDesc))
+    customHeap = memdescGetCustomHeap(pMemDesc);
+    if (customHeap == MEMDESC_CUSTOM_HEAP_NONE)
     {
         MemoryManager      *pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
         NvHandle            hClient        = RES_GET_CLIENT_HANDLE(pVideoMemory);
@@ -1201,6 +1324,23 @@ vidmemAllocResources
                          pAllocRequest->pPmaAllocInfo[subdeviceInst]->pageCount,
                          pAllocRequest->pPmaAllocInfo[subdeviceInst]->pageSize);
         }
+        if (!FLD_TEST_DRF(OS32, _ATTR2, _ENABLE_LOCALIZED_MEMORY, _DEFAULT, pVidHeapAlloc->attr2))
+        {
+            KernelMIGManager *pKernelMIGManager = GPU_GET_KERNEL_MIG_MANAGER(pGpu);
+            NvBool bIsMIGMemPartitioningEnabled = (pKernelMIGManager != NULL) && kmigmgrIsMIGMemPartitioningEnabled(pGpu, pKernelMIGManager);
+
+            if (pMemoryManager->bLocalizedMemorySupported &&
+                !bIsMIGMemPartitioningEnabled)
+            {
+                memdescSetFlag(pAllocRequest->pMemDesc, MEMDESC_FLAGS_ALLOC_AS_LOCALIZED, NV_TRUE);
+            }
+            else
+            {
+                NV_PRINTF(LEVEL_ERROR, "Localized memory requested when localized memory not enabled\n");
+                status = NV_ERR_INVALID_ARGUMENT;
+                goto failed;
+            }
+        }
     }
     else
     {
@@ -1232,6 +1372,11 @@ vidmemAllocResources
         }
 
         bAllocedMemory = NV_TRUE;
+
+        if (pVidHeapAlloc->flags & NVOS32_ALLOC_FLAGS_PERSISTENT_VIDMEM)
+        {
+            memdescSetFlag(pMemDesc, MEMDESC_FLAGS_PRESERVE_CONTENT_ON_SUSPEND, NV_TRUE);
+        }
     }
 
     if (!bIsPmaOwned && (pVidHeapAlloc->type != NVOS32_TYPE_PMA))
@@ -1350,12 +1495,16 @@ vidmemCheckCopyPermissions_IMPL
 {
     Memory           *pMemory               = staticCast(pVideoMemory, Memory);
     OBJGPU           *pSrcGpu               = pMemory->pGpu;
-    NvHandle          hSrcClient            = RES_GET_CLIENT_HANDLE(pVideoMemory);
-    NvHandle          hDstClient            = RES_GET_CLIENT_HANDLE(pDstDevice);
+    RsClient         *pSrcClient            = RES_GET_CLIENT(pVideoMemory);
+    RsClient         *pDstClient            = RES_GET_CLIENT(pDstDevice);
     KernelMIGManager *pSrcKernelMIGManager  = GPU_GET_KERNEL_MIG_MANAGER(pSrcGpu);
     KernelMIGManager *pDstKernelMIGManager  = GPU_GET_KERNEL_MIG_MANAGER(pDstGpu);
-    NvBool            bSrcClientKernel      = (rmclientGetCachedPrivilegeByHandle(hSrcClient) >= RS_PRIV_LEVEL_KERNEL);
-    NvBool            bDstClientKernel      = (rmclientGetCachedPrivilegeByHandle(hDstClient) >= RS_PRIV_LEVEL_KERNEL);
+    NvBool            bSrcClientKernel      =
+        (rmclientGetCachedPrivilege(dynamicCast(pSrcClient, RmClient)) >=
+         RS_PRIV_LEVEL_KERNEL);
+    NvBool            bDstClientKernel      =
+        (rmclientGetCachedPrivilege(dynamicCast(pDstClient, RmClient)) >=
+         RS_PRIV_LEVEL_KERNEL);
 
     //
     // XXX: In case of MIG memory, duping across GPU instances is not allowed
