@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2000-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2000-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -34,11 +34,15 @@
 #include "gps.h"
 #include "nvop.h"
 #include "nvhybridacpi.h"
+#include "nvmisc.h"
 #include "jt.h"
 #include "pex.h"
 #include "mxm_spec.h"
 #include "gpu/gsp/gsp_static_config.h"
 #include "platform/nbsi/nbsi_read.h"
+#include "nvrm_registry.h"
+
+#include "gpu/disp/kern_disp.h"
 
 //
 // DSM ACPI Routines common routines for Linux
@@ -358,13 +362,13 @@ _getRemappedDsmSubfunction
     // Use NV_ASSERT_OR_ELSE_STR for embedded %.
     //
     NV_ASSERT_OR_ELSE_STR(
-        !(NV_ARRAY_ELEMENTS32(genDsmSubFuncRemapTable) % (ACPI_DSM_FUNCTION_COUNT + 1)),
-        "!(NV_ARRAY_ELEMENTS32(genDsmSubFuncRemapTable) %% (ACPI_DSM_FUNCTION_COUNT + 1))",
+        !(NV_ARRAY_ELEMENTS(genDsmSubFuncRemapTable) % (ACPI_DSM_FUNCTION_COUNT + 1)),
+        "!(NV_ARRAY_ELEMENTS(genDsmSubFuncRemapTable) %% (ACPI_DSM_FUNCTION_COUNT + 1))",
         return NV_ERR_INVALID_ARGUMENT);
 
     // find the event in the table
     i = 0;
-    while (i <= (NV_ARRAY_ELEMENTS32(genDsmSubFuncRemapTable) - ACPI_DSM_FUNCTION_COUNT - 1))
+    while (i <= (NV_ARRAY_ELEMENTS(genDsmSubFuncRemapTable) - ACPI_DSM_FUNCTION_COUNT - 1))
     {
         if (acpiDsmSubFunction == genDsmSubFuncRemapTable[i])
         {
@@ -475,7 +479,7 @@ remapDsmFunctionAndSubFunction
 
     // Loop through (in precendence order) to find the GUID/subfunction until we find one that supports the call.
     testDSMfuncIndex = 0;
-    while(testDSMfuncIndex < sizeof(dsmOrderOfPrecedenceList)/sizeof(dsmOrderOfPrecedenceList[0]))
+    while(testDSMfuncIndex < NV_ARRAY_ELEMENTS(dsmOrderOfPrecedenceList))
     {
         // remap the generic subfunction number of the one that matches this function.
         if ((_getRemappedDsmSubfunction(dsmOrderOfPrecedenceList[testDSMfuncIndex], *pRemappedDsmSubFunction, &remappedDSMSubFunction)) == NV_OK)
@@ -554,7 +558,7 @@ getDsmGetObjectSubfunction
             if (remapDsmFunctionAndSubFunction(pGpu, &curFuncForGetAllObjects, &dummySubFunc) == NV_OK)
             {
                 testDSMfuncIndex = 0;
-                while(testDSMfuncIndex < sizeof(dsmOrderOfPrecedenceList)/sizeof(dsmOrderOfPrecedenceList[0]))
+                while(testDSMfuncIndex < NV_ARRAY_ELEMENTS(dsmOrderOfPrecedenceList))
                 {
                     if (dsmOrderOfPrecedenceList[testDSMfuncIndex] == curFuncForGetObjByType)
                     {
@@ -705,6 +709,17 @@ checkDsmCall
     NV_ASSERT_OR_RETURN(pInOut, NV_ERR_INVALID_ARGUMENT);
     NV_ASSERT_OR_RETURN(pSize, NV_ERR_INVALID_ARGUMENT);
 
+    KernelDisplay *pKernelDisplay = GPU_GET_KERNEL_DISPLAY(pGpu);
+
+    if (pKernelDisplay != NULL
+        && pKernelDisplay->getProperty(pKernelDisplay, PDB_PROP_KDISP_INTERNAL_PANEL_DISCONNECTED))
+    {
+        if (*pAcpiDsmFunction == ACPI_DSM_FUNCTION_NBCI)
+        {
+            return NV_ERR_NOT_SUPPORTED;
+        } 
+    }
+
     // Do any remapping of subfunction if function is current
     if (remapDsmFunctionAndSubFunction(pGpu, pAcpiDsmFunction, pAcpiDsmSubFunction) != NV_OK)
     {
@@ -857,7 +872,7 @@ _acpiDsmSupportedFuncCacheInit
     }
 }
 
-static void 
+static void
 _acpiDsmCallbackInit
 (
     OBJGPU *pGpu
@@ -888,7 +903,7 @@ _acpiDsmCallbackInit
 
     // Loop through the list of GUIDs that support callbacks in priority order
     for (testDSMfuncIndex = 0;
-         testDSMfuncIndex < (sizeof(callbackOrderOfPrecedenceList)/sizeof(callbackOrderOfPrecedenceList[0]));
+         testDSMfuncIndex < (NV_ARRAY_ELEMENTS(callbackOrderOfPrecedenceList));
          testDSMfuncIndex++)
     {
         func = callbackOrderOfPrecedenceList[testDSMfuncIndex];
@@ -963,7 +978,7 @@ _acpiDsmCallbackInit
  *
  * @returns
  */
-static void 
+static void
 _acpiDsmCapsInit
 (
     OBJGPU *pGpu
@@ -1110,7 +1125,7 @@ _acpiCacheMethodData
     NV_STATUS status;
     NvU32 inOut = 0;
     NvU16 rtnSize = sizeof(inOut);
-    NvU32 tableLen = 0, acpiidIndex = 0, mode = 0, muxPartId = 0;
+    NvU32 tableLen = 0, acpiidIndex = 0, mode = 0, muxPartId = 0, state = 0;
 
     // This bit is used for checking if pGpu::acpiMethodData need to be used or not.
     pGpu->acpiMethodData.bValid = NV_TRUE;
@@ -1133,6 +1148,7 @@ _acpiCacheMethodData
     // Fill in the MUX Method Data.
     portMemSet(pGpu->acpiMethodData.muxMethodData.acpiIdMuxModeTable, 0, sizeof(pGpu->acpiMethodData.muxMethodData.acpiIdMuxModeTable));
     portMemSet(pGpu->acpiMethodData.muxMethodData.acpiIdMuxPartTable, 0, sizeof(pGpu->acpiMethodData.muxMethodData.acpiIdMuxPartTable));
+    portMemSet(pGpu->acpiMethodData.muxMethodData.acpiIdMuxStateTable, 0, sizeof(pGpu->acpiMethodData.muxMethodData.acpiIdMuxStateTable));    
     if (pGpu->acpiMethodData.dodMethodData.status == NV_OK)
     {
         tableLen = pGpu->acpiMethodData.dodMethodData.acpiIdListLen / sizeof(NvU32);
@@ -1148,7 +1164,12 @@ _acpiCacheMethodData
             pGpu->acpiMethodData.muxMethodData.acpiIdMuxPartTable[acpiidIndex].acpiId = pGpu->acpiMethodData.dodMethodData.acpiIdList[acpiidIndex];
             pGpu->acpiMethodData.muxMethodData.acpiIdMuxPartTable[acpiidIndex].mode = muxPartId;
             pGpu->acpiMethodData.muxMethodData.acpiIdMuxPartTable[acpiidIndex].status = status;
-            mode = muxPartId = 0;
+
+            status = osCallACPI_MXDS(pGpu, pGpu->acpiMethodData.dodMethodData.acpiIdList[acpiidIndex], &state);
+            pGpu->acpiMethodData.muxMethodData.acpiIdMuxStateTable[acpiidIndex].acpiId = pGpu->acpiMethodData.dodMethodData.acpiIdList[acpiidIndex];
+            pGpu->acpiMethodData.muxMethodData.acpiIdMuxStateTable[acpiidIndex].mode = state;
+            pGpu->acpiMethodData.muxMethodData.acpiIdMuxStateTable[acpiidIndex].status = status;
+            mode = muxPartId = state = 0;
         }
     }
 
