@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1993-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1993-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -31,7 +31,7 @@
 #include "platform/platform.h"
 #include "platform/chipset/chipset_info.h"
 #include "os/os.h"
-#include "nvRmReg.h"
+#include "nvrm_registry.h"
 #include "nvpcie.h"
 #include "nv_ref.h"
 #include "kernel/gpu/bif/kernel_bif.h"
@@ -65,8 +65,6 @@ void
 clInitPropertiesFromRegistry_IMPL(OBJGPU *pGpu, OBJCL *pCl)
 {
     NvU32  data32;
-    OBJSYS *pSys = SYS_GET_INSTANCE();
-    OBJOS *pOS = SYS_GET_OS(pSys);
 
     if (osReadRegistryDword(pGpu, NV_REG_STR_RM_DISABLE_BR03_FLOW_CONTROL, &data32) == NV_OK
             && data32)
@@ -82,7 +80,13 @@ clInitPropertiesFromRegistry_IMPL(OBJGPU *pGpu, OBJCL *pCl)
         }
     }
 
-    pOS->osQADbgRegistryInit(pOS);
+    if ((osReadRegistryDword(pGpu, NV_REG_STR_RM_FORCE_DISABLE_IOMAP_WC, &data32) == NV_OK)
+            && (data32 == NV_REG_STR_RM_FORCE_DISABLE_IOMAP_WC_YES))
+    {
+        pCl->setProperty(pCl, PDB_PROP_CL_DISABLE_IOMAP_WC, NV_TRUE);
+    }
+
+    osQADbgRegistryInit();
 }
 
 static void
@@ -137,7 +141,7 @@ clInitMappingPciBusDevice_IMPL
         return NV0000_CTRL_GPU_INVALID_ID;
 
     // do we already know our domain/bus/device?
-    if (gpuGetDBDF(pGpu) == 0)
+    if (!gpuIsDBDFValid(pGpu))
     {
         // we're checking all the device/funcs for the first 10 buses!
         // Note that we give up the enumeration once we find our first
@@ -161,24 +165,16 @@ clInitMappingPciBusDevice_IMPL
                         continue;
 
                     // if the BAR0 matches our PhysAddr, it's the correct device
-                    if ((osPciReadDword(handle, PCI_BASE_ADDRESS_0)) !=
+                    if (((osPciReadDword(handle, PCI_BASE_ADDRESS_0) & PCI_BASE_ADDRESS_0_VALID_MASK)) !=
                         pGpu->busInfo.gpuPhysAddr)
                         continue;
 
                     // save our domain/bus/device/function
                     pGpu->busInfo.nvDomainBusDeviceFunc = gpuEncodeDomainBusDevice(domain, (NvU8)bus, device);
+                    pGpu->busInfo.bNvDomainBusDeviceFuncValid = NV_TRUE;
 
                     bFoundDevice = NV_TRUE;
 
-                    if (!(IS_SIMULATION(pGpu) || IS_SIM_MODS(GPU_GET_OS(pGpu))))
-                    {
-                        NV_ASSERT(gpuGetDBDF(pGpu) != 0);
-                    }
-                                                        // On the HP "Wilson's Peak"/McKinley system
-                                                        // the graphics is located at
-                                                        // domain==0, bus==0, device==0.
-                                                        // Why should this be invalid?
-                    // In simulation, the fmodel can put this at bus==0, device==0
                     break;
                 }
             }
@@ -189,10 +185,8 @@ clInitMappingPciBusDevice_IMPL
     bus = gpuGetBus(pGpu);
     device = gpuGetDevice(pGpu);
 
-    if (gpuGetDBDF(pGpu) == 0)
+    if (!gpuIsDBDFValid(pGpu))
     {
-        if (!(IS_SIMULATION(pGpu)|| IS_SIM_MODS(GPU_GET_OS(pGpu))) 
-               || (bFoundDevice == NV_FALSE))
         {
             NV_PRINTF(LEVEL_ERROR,
                     "NVRM initMappingPciBusDevice: can't find a device!\n");
@@ -210,7 +204,7 @@ clInitMappingPciBusDevice_IMPL
 static void getSubsystemFromPCIECapabilities
 (
     NvU32 domain,
-    NvU8 bus, 
+    NvU8 bus,
     NvU8 device,
     NvU8 func,
     NvU16 *subvendorID,
@@ -274,10 +268,15 @@ clFindFHBAndGetChipsetInfoIndex_IMPL
     //
     // PC motherboards have a host bridge to connect PCIE root complex to rest of the system.
     // However, Tegra devices only have a PCI-to-PCI bridge.
-    // So allow Tegra chipset initialization, even if a host bridge is not found.
-    // See bug 1547160 comment#17 for more details.
+    // So allow Tegra chipset initialization, even if a host bridge is not found (SUBBASECLASS_P2P).
+    // See bug 1547160 comment#17 for more details (dGPU connected with Tegra device through PCIe).
+    // There is no host bridge or PCI-to-PCI bridge present in between
+    // Tegra iGPUs and PCIe-RC. Tegra iGPUs are directly connected to endpoint.
+    // PCI config for Sub Base Class code will always be programmed as
+    // PCI_COMMON_CLASS_SUBBASECLASS_3DCTRL for Tegra iGPUs.
     //
-    NvU16    pciSubBaseClass[2] = {PCI_COMMON_CLASS_SUBBASECLASS_HOST, PCI_COMMON_CLASS_SUBBASECLASS_P2P};
+    NvU16    pciSubBaseClass[] = {PCI_COMMON_CLASS_SUBBASECLASS_HOST,
+        PCI_COMMON_CLASS_SUBBASECLASS_P2P, PCI_COMMON_CLASS_SUBBASECLASS_3DCTRL};
 
     // return it, if we've got it already
     if (pCl->chipsetIDBusAddr.valid)
@@ -394,65 +393,72 @@ clFindFHBAndGetChipsetInfoIndex_IMPL
     // match with Host Bridge ID. In that case we need to find FHB either in
     // cached bus topology or need to loop through PCI bus to find the FHB.
     //
-    for (i = 0; i < 2; i++)
+    for (i = 0; i < NV_ARRAY_ELEMENTS(pciSubBaseClass); i++)
     {
-    pBusTopologyInfo = pCl->pBusTopologyInfo;
-    while (pBusTopologyInfo)
-    {
-        if (pBusTopologyInfo->pciSubBaseClass == pciSubBaseClass[i])
+        pBusTopologyInfo = pCl->pBusTopologyInfo;
+        while (pBusTopologyInfo)
         {
-            pCl->FHBAddr.domain = pBusTopologyInfo->domain;
-            pCl->FHBAddr.bus    = pBusTopologyInfo->bus;
-            pCl->FHBAddr.device = pBusTopologyInfo->device;
-            pCl->FHBAddr.func   = pBusTopologyInfo->func;
-            pCl->FHBAddr.valid  = 0x1;
-            pCl->FHBAddr.handle = pBusTopologyInfo->handle;
-
-            // Store a copy of deviceID,  vendorID, subdeviceID and subvendorID;
-            pCl->FHBBusInfo.deviceID    = pBusTopologyInfo->busInfo.deviceID;
-            pCl->FHBBusInfo.vendorID    = pBusTopologyInfo->busInfo.vendorID;
-            pCl->FHBBusInfo.subdeviceID = pBusTopologyInfo->busInfo.subdeviceID;
-            pCl->FHBBusInfo.subvendorID = pBusTopologyInfo->busInfo.subvendorID;
-            pCl->FHBBusInfo.revisionID  = pBusTopologyInfo->busInfo.revisionID;
-
-            if (!matchFound)
+            if (pBusTopologyInfo->pciSubBaseClass == pciSubBaseClass[i])
             {
-                pCl->chipsetIDBusAddr.domain = pBusTopologyInfo->domain;
-                pCl->chipsetIDBusAddr.bus    = pBusTopologyInfo->bus;
-                pCl->chipsetIDBusAddr.device = pBusTopologyInfo->device;
-                pCl->chipsetIDBusAddr.func   = pBusTopologyInfo->func;
-                pCl->chipsetIDBusAddr.valid  = 0x1;
-                pCl->chipsetIDBusAddr.handle = pBusTopologyInfo->handle;
+                pCl->FHBAddr.domain = pBusTopologyInfo->domain;
+                pCl->FHBAddr.bus    = pBusTopologyInfo->bus;
+                pCl->FHBAddr.device = pBusTopologyInfo->device;
+                pCl->FHBAddr.func   = pBusTopologyInfo->func;
+                pCl->FHBAddr.valid  = 0x1;
+                pCl->FHBAddr.handle = pBusTopologyInfo->handle;
 
                 // Store a copy of deviceID,  vendorID, subdeviceID and subvendorID;
-                pCl->chipsetIDInfo.deviceID    = pBusTopologyInfo->busInfo.deviceID;
-                pCl->chipsetIDInfo.vendorID    = pBusTopologyInfo->busInfo.vendorID;
-                pCl->chipsetIDInfo.subdeviceID = pBusTopologyInfo->busInfo.subdeviceID;
-                pCl->chipsetIDInfo.subvendorID = pBusTopologyInfo->busInfo.subvendorID;
+                pCl->FHBBusInfo.deviceID    = pBusTopologyInfo->busInfo.deviceID;
+                pCl->FHBBusInfo.vendorID    = pBusTopologyInfo->busInfo.vendorID;
+                pCl->FHBBusInfo.subdeviceID = pBusTopologyInfo->busInfo.subdeviceID;
+                pCl->FHBBusInfo.subvendorID = pBusTopologyInfo->busInfo.subvendorID;
+                pCl->FHBBusInfo.revisionID  = pBusTopologyInfo->busInfo.revisionID;
 
-                if (pCl->chipsetIDInfo.subvendorID == 0)
+                if (!matchFound)
                 {
-                    getSubsystemFromPCIECapabilities(pCl->chipsetIDBusAddr.domain,
-                                                     pCl->chipsetIDBusAddr.bus,
-                                                     pCl->chipsetIDBusAddr.device,
-                                                     pCl->chipsetIDBusAddr.func,
-                                                     &pCl->chipsetIDInfo.subvendorID,
-                                                     &pCl->chipsetIDInfo.subdeviceID);
-                }
-            }
-            return NV_OK;
-        }
-        pBusTopologyInfo = pBusTopologyInfo->next;
-    }
+                    pCl->chipsetIDBusAddr.domain = pBusTopologyInfo->domain;
+                    pCl->chipsetIDBusAddr.bus    = pBusTopologyInfo->bus;
+                    pCl->chipsetIDBusAddr.device = pBusTopologyInfo->device;
+                    pCl->chipsetIDBusAddr.func   = pBusTopologyInfo->func;
+                    pCl->chipsetIDBusAddr.valid  = 0x1;
+                    pCl->chipsetIDBusAddr.handle = pBusTopologyInfo->handle;
 
-        NV_PRINTF(LEVEL_INFO,
-                  "NVRM : Host bridge device not found. Looking for a PCI-to-PCI bridge device!!!\n");
+                    // Store a copy of deviceID,  vendorID, subdeviceID and subvendorID;
+                    pCl->chipsetIDInfo.deviceID    = pBusTopologyInfo->busInfo.deviceID;
+                    pCl->chipsetIDInfo.vendorID    = pBusTopologyInfo->busInfo.vendorID;
+                    pCl->chipsetIDInfo.subdeviceID = pBusTopologyInfo->busInfo.subdeviceID;
+                    pCl->chipsetIDInfo.subvendorID = pBusTopologyInfo->busInfo.subvendorID;
+
+                    if (pCl->chipsetIDInfo.subvendorID == 0)
+                    {
+                        getSubsystemFromPCIECapabilities(pCl->chipsetIDBusAddr.domain,
+                                                         pCl->chipsetIDBusAddr.bus,
+                                                         pCl->chipsetIDBusAddr.device,
+                                                         pCl->chipsetIDBusAddr.func,
+                                                         &pCl->chipsetIDInfo.subvendorID,
+                                                         &pCl->chipsetIDInfo.subdeviceID);
+                    }
+                }
+
+                NV_PRINTF(LEVEL_INFO,
+                          "DeviceId[%x] VendorID[%x] BDF[%x:%x:%x] SubClassId[%x] device found.\n",
+                          pBusTopologyInfo->busInfo.deviceID,
+                          pBusTopologyInfo->busInfo.vendorID,
+                          pBusTopologyInfo->bus,
+                          pBusTopologyInfo->device,
+                          pBusTopologyInfo->func,
+                          pciSubBaseClass[i]);
+
+                return NV_OK;
+            }
+            pBusTopologyInfo = pBusTopologyInfo->next;
+        }
     }
 
     NV_PRINTF(LEVEL_ERROR,
-              "NVRM : This is Bad. FHB not found in cached bus topology!!!\n");
+              "NVRM : This is Bad. FHB/P2P/3DCTRL not found in cached bus topology!!!\n");
 
-    // HB is not present in cached bus topology.
+    // HB/P2P/3DCTRL is not present in cached bus topology.
     NV_ASSERT(0);
 
     //
@@ -554,8 +560,61 @@ clFindFHBAndGetChipsetInfoIndex_IMPL
     return NV_ERR_NOT_SUPPORTED;
 }
 
+/*!
+ * @brief Check if ASPM L1 is supported from upstream component
+ *
+ * @param[in] pGpu GPU object pointer
+ * @param[in] pCl  CL  object pointer
+ *
+ * @return NV_TRUE if L1 is supported
+ */
 NvBool
-clIsL1MaskEnabledForUpstreamPort_IMPL
+clIsL1SupportedForUpstreamPort_IMPL
+(
+    OBJGPU *pGpu,
+    OBJCL  *pCl
+)
+{
+    NvU32     linkCap;
+    NvBool    bSupported = NV_FALSE;
+    NV_STATUS status     = NV_OK;
+
+    if (pGpu->gpuClData.upstreamPort.addr.valid)
+    {
+        status = clPcieReadPortConfigReg(pGpu, pCl, &pGpu->gpuClData.upstreamPort, CL_PCIE_LINK_CAP, &linkCap);
+        if (status == NV_OK)
+        {
+            if ((CL_IS_L1_SUPPORTED(linkCap)))
+            {
+                bSupported = NV_TRUE;
+            }
+        }
+    }
+    else if (pGpu->gpuClData.rootPort.addr.valid)
+    {
+        status = clPcieReadPortConfigReg(pGpu, pCl, &pGpu->gpuClData.rootPort, CL_PCIE_LINK_CAP, &linkCap);
+        if (status == NV_OK)
+        {
+            if ((CL_IS_L1_SUPPORTED(linkCap)))
+            {
+                bSupported = NV_TRUE;
+            }
+        }
+    }
+
+    return bSupported;
+}
+
+/*!
+ * @brief Check if L0s mask is enabled for upstream component
+ *
+ * @param[in] pGpu GPU object pointer
+ * @param[in] pCl  CL  object pointer
+ *
+ * @return NV_TRUE if mask is enabled (implies L0s is disabled)
+ */
+NvBool
+clIsL0sMaskEnabledForUpstreamPort_IMPL
 (
     OBJGPU *pGpu,
     OBJCL  *pCl
@@ -579,7 +638,7 @@ clIsL1MaskEnabledForUpstreamPort_IMPL
             }
             else
             {
-                if (!(linkCtrl & CL_PCIE_LINK_CTRL_STATUS_ASPM_L1_BIT))
+                if (!(linkCtrl & CL_PCIE_LINK_CTRL_STATUS_ASPM_L0S_BIT))
                 {
                     bEnable = NV_TRUE;
                 }
@@ -595,7 +654,7 @@ clIsL1MaskEnabledForUpstreamPort_IMPL
         }
         else
         {
-            if (!(linkCtrl & CL_PCIE_LINK_CTRL_STATUS_ASPM_L1_BIT))
+            if (!(linkCtrl & CL_PCIE_LINK_CTRL_STATUS_ASPM_L0S_BIT))
             {
                 bEnable = NV_TRUE;
             }
@@ -728,7 +787,7 @@ void clSyncWithGsp_IMPL(OBJCL *pCl, GspSystemInfo *pGSI)
     CL_SYNC_PDB(PDB_PROP_CL_ASPM_L1_CHIPSET_DISABLED);
     CL_SYNC_PDB(PDB_PROP_CL_ASPM_L0S_CHIPSET_ENABLED_MOBILE_ONLY);
     CL_SYNC_PDB(PDB_PROP_CL_ASPM_L1_CHIPSET_ENABLED_MOBILE_ONLY);
-    CL_SYNC_PDB(PDB_PROP_CL_ASPM_UPSTREAM_PORT_L1_MASK_ENABLED);
+    CL_SYNC_PDB(PDB_PROP_CL_ASPM_L1_UPSTREAM_PORT_SUPPORTED);
     CL_SYNC_PDB(PDB_PROP_CL_PCIE_GEN1_GEN2_SWITCH_CHIPSET_DISABLED);
     CL_SYNC_PDB(PDB_PROP_CL_PCIE_GEN1_GEN2_SWITCH_CHIPSET_DISABLED_GEFORCE);
     CL_SYNC_PDB(PDB_PROP_CL_EXTENDED_TAG_FIELD_NOT_CAPABLE);
@@ -745,14 +804,14 @@ void clSyncWithGsp_IMPL(OBJCL *pCl, GspSystemInfo *pGSI)
     CL_SYNC_PDB(PDB_PROP_CL_UPSTREAM_LTR_SUPPORTED);
     CL_SYNC_PDB(PDB_PROP_CL_BUG_1340801_DISABLE_GEN3_ON_GIGABYTE_SNIPER_3);
     CL_SYNC_PDB(PDB_PROP_CL_BUG_1681803_WAR_DISABLE_MSCG);
-    CL_SYNC_PDB(PDB_PROP_CL_ON_HASWELL_HOST_BRIDGE);
     CL_SYNC_PDB(PDB_PROP_CL_PCIE_NON_COHERENT_USE_TC0_ONLY);
     CL_SYNC_PDB(PDB_PROP_CL_UNSUPPORTED_CHIPSET);
     CL_SYNC_PDB(PDB_PROP_CL_IS_CHIPSET_IO_COHERENT);
     CL_SYNC_PDB(PDB_PROP_CL_DISABLE_IOMAP_WC);
     CL_SYNC_PDB(PDB_PROP_CL_HAS_RESIZABLE_BAR_ISSUE);
-    CL_SYNC_PDB(PDB_PROP_CL_IS_EXTERNAL_GPU);
     CL_SYNC_PDB(PDB_PROP_CL_BUG_3751839_GEN_SPEED_WAR);
+    CL_SYNC_PDB(PDB_PROP_CL_BUG_3562968_WAR_ALLOW_PCIE_ATOMICS);
+    CL_SYNC_PDB(PDB_PROP_CL_WAR_AMD_5107271);
 
 #undef CL_SYNC_PDB
 
