@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1993-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1993-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -38,9 +38,8 @@
 
 #include "ctrl/ctrl0000/ctrl0000event.h" // NV0000_CTRL_EVENT_SET_NOTIFICATION_ACTION_*
 
-#if (!NV_RM_STUB_RPC)
 static NV_STATUS _eventRpcForType(NvHandle hClient, NvHandle hObject);
-#endif
+static void eventSystemDequeueEventLatest(SystemEventQueueList *pQueue);
 
 NV_STATUS
 eventConstruct_IMPL
@@ -89,7 +88,6 @@ eventConstruct_IMPL
         }
     }
 
-#if (!NV_RM_STUB_RPC)
     if (_eventRpcForType(hParentClient, pNv0050AllocParams->hSrcResource))
     {
         RsResourceRef *pSrcRef;
@@ -112,7 +110,6 @@ eventConstruct_IMPL
             }
         }
     }
-#endif
 
     NV_ASSERT_OK_OR_RETURN(clientGetResourceRef(pRsClient, pRsClient->hClient, &pClientRef));
 
@@ -139,7 +136,7 @@ eventConstruct_IMPL
         //
         if (pGpu != NULL)
         {
-            RsResourceRef *pSourceRef;
+            RsResourceRef *pSourceRef = NULL;
 
             if (IS_GSP_CLIENT(pGpu))
             {
@@ -264,7 +261,6 @@ NV_STATUS notifyUnregisterEvent_IMPL
     if (*ppEventNotification != NULL)
     {
 
-#if (!NV_RM_STUB_RPC)
         if (_eventRpcForType(hNotifierClient, hNotifierResource))
         {
             OBJGPU *pGpu = CliGetGpuFromHandle(hNotifierClient, hNotifierResource, NULL);
@@ -309,7 +305,6 @@ NV_STATUS notifyUnregisterEvent_IMPL
                           hNotifierClient, hNotifierResource);
             }
         }
-#endif
 
         unregisterEventNotification(ppEventNotification,
                           hEventClient,
@@ -517,32 +512,23 @@ CliGetEventInfo
 
 }
 
-NvBool
+void
 CliDelObjectEvents
 (
-    NvHandle hClient,
-    NvHandle hResource
+    RsResourceRef *pResourceRef
 )
 {
     NotifShare    *pNotifierShare;
     INotifier     *pNotifier;
-    RsClient      *pRsClient;
-    NV_STATUS      status = NV_OK;
-    RsResourceRef *pResourceRef;
 
-    status = serverGetClientUnderLock(&g_resServ, hClient, &pRsClient);
-    if (status != NV_OK)
-        return NV_FALSE;
-
-    status = clientGetResourceRef(pRsClient, hResource, &pResourceRef);
-    if (status != NV_OK)
-        return NV_FALSE;
+    if (pResourceRef == NULL)
+        return;
 
     // If not a notifier object, there aren't any events to free
     pNotifier = dynamicCast(pResourceRef->pResource, INotifier);
 
     if (pNotifier == NULL)
-        return NV_TRUE;
+        return;
 
     pNotifierShare = inotifyGetNotificationShare(pNotifier);
     if (pNotifierShare != NULL)
@@ -550,7 +536,7 @@ CliDelObjectEvents
         while(pNotifierShare->pEventList != NULL)
         {
             PEVENTNOTIFICATION pEventNotif = pNotifierShare->pEventList;
-            status = inotifyUnregisterEvent(pNotifier,
+            inotifyUnregisterEvent(pNotifier,
                     pNotifierShare->hNotifierClient,
                     pNotifierShare->hNotifierResource,
                     pEventNotif->hEventClient,
@@ -558,9 +544,6 @@ CliDelObjectEvents
         }
         pNotifierShare->pNotifier = NULL;
     }
-
-    return NV_TRUE;
-
 } // end of CliDelObjectEvents()
 
 // ****************************************************************************
@@ -569,11 +552,10 @@ CliDelObjectEvents
 
 void CliAddSystemEvent(
     NvU32 event,
-    NvU32 status,
+    void *pEventData,
     NvBool *isEventNotified
 )
 {
-    NvU32 temp;
     PEVENTNOTIFICATION pEventNotification = NULL;
     RmClient **ppClient;
     RmClient  *pClient;
@@ -597,18 +579,6 @@ void CliAddSystemEvent(
             continue;
         }
 
-        temp = (pClient->CliSysEventInfo.systemEventsQueue.Head + 1) % NV_SYSTEM_EVENT_QUEUE_SIZE;
-
-        if (temp == pClient->CliSysEventInfo.systemEventsQueue.Tail)
-        {
-            NV_PRINTF(LEVEL_ERROR, "system event queue is full");
-            return;
-        }
-
-        pClient->CliSysEventInfo.systemEventsQueue.EventQueue[pClient->CliSysEventInfo.systemEventsQueue.Head].event  = event;
-        pClient->CliSysEventInfo.systemEventsQueue.EventQueue[pClient->CliSysEventInfo.systemEventsQueue.Head].status = status;
-        pClient->CliSysEventInfo.systemEventsQueue.Head = temp;
-
         rmStatus = clientGetResourceRef(staticCast(pClient, RsClient), pRsClient->hClient, &pCliResRef);
         if (rmStatus != NV_OK)
         {
@@ -628,13 +598,30 @@ void CliAddSystemEvent(
             {
                 if (pEventNotification->NotifyIndex == event)
                 {
+                    // only log the system event that has data
+                    if (pEventData != NULL)
+                    {
+                        if (eventSystemEnqueueEvent(&pClient->CliSysEventInfo.eventQueue,
+                                                    event, pEventData) != NV_OK)
+                        {
+                            NV_PRINTF(LEVEL_ERROR, "fails to add event=%d\n", event);
+                            return;
+                        }
+                    }
+
                     if (osNotifyEvent(NULL, pEventNotification, 0, 0, 0) != NV_OK)
                     {
+                        if (pEventData != NULL)
+                            eventSystemDequeueEventLatest(&pClient->CliSysEventInfo.eventQueue);
+
                         NV_PRINTF(LEVEL_ERROR, "failed to deliver event 0x%x",
                                   event);
                     }
-                    if (isEventNotified != NULL)
-                        *isEventNotified = NV_TRUE;
+                    else
+                    {
+                        if (isEventNotified != NULL)
+                            *isEventNotified = NV_TRUE;
+                    }
                 }
                 pEventNotification = pEventNotification->Next;
             }
@@ -649,7 +636,6 @@ void CliAddSystemEvent(
     return;
 }
 
-#if (!NV_RM_STUB_RPC)
 static NV_STATUS
 _eventRpcForType(NvHandle hClient, NvHandle hObject)
 {
@@ -668,6 +654,7 @@ _eventRpcForType(NvHandle hClient, NvHandle hObject)
         objDynamicCastById(pResourceRef->pResource, classId(ChannelDescendant)) ||
         objDynamicCastById(pResourceRef->pResource, classId(ContextDma)) ||
         objDynamicCastById(pResourceRef->pResource, classId(DispChannel)) ||
+        objDynamicCastById(pResourceRef->pResource, classId(DispCommon)) ||        
         objDynamicCastById(pResourceRef->pResource, classId(TimerApi)) ||
         objDynamicCastById(pResourceRef->pResource, classId(KernelSMDebuggerSession)))
     {
@@ -676,4 +663,120 @@ _eventRpcForType(NvHandle hClient, NvHandle hObject)
 
     return NV_FALSE;
 }
-#endif
+
+NV_STATUS
+eventGetByHandle_IMPL
+(
+    RsClient *pClient,
+    NvHandle hEvent,
+    NvU32 *pNotifyIndex
+)
+{
+    RsResourceRef  *pEventResourceRef;
+    NV_STATUS status;
+    Event *pEvent;
+    NotifShare *pNotifierShare;
+    PEVENTNOTIFICATION  pEventNotification;
+
+    *pNotifyIndex = NV2080_NOTIFIERS_MAXCOUNT;
+
+    status = clientGetResourceRef(pClient, hEvent, &pEventResourceRef);
+    if (status != NV_OK)
+        return status;
+
+    pEvent = dynamicCast(pEventResourceRef->pResource, Event);
+    if (pEvent == NULL)
+    {
+        NV_PRINTF(LEVEL_ERROR, "Event is null \n");
+        return NV_ERR_INVALID_ARGUMENT;
+    }
+
+    // TODO: Check existing notifiers in that event
+    pNotifierShare = pEvent->pNotifierShare;
+    if ((pNotifierShare == NULL) || (pNotifierShare->pNotifier == NULL))
+    {
+        NV_PRINTF(LEVEL_ERROR, "pNotifierShare or pNotifier is NULL \n");
+        return NV_ERR_INVALID_ARGUMENT;
+    }
+
+    pEventNotification = inotifyGetNotificationList(pNotifierShare->pNotifier);
+    if (pEventNotification == NULL)
+    {
+        NV_PRINTF(LEVEL_ERROR, "pEventNotification is NULL \n");
+        return NV_ERR_INVALID_ARGUMENT;
+    }
+
+    *pNotifyIndex = pEventNotification->NotifyIndex;
+
+    return status;
+}
+
+void eventSystemInitEventQueue(SystemEventQueueList *pQueue)
+{
+    listInit(pQueue, portMemAllocatorGetGlobalNonPaged());
+}
+
+NV_STATUS eventSystemEnqueueEvent(SystemEventQueueList *pQueue, NvU32 event, void *pEventData)
+{
+    NV0000_CTRL_GET_SYSTEM_EVENT_DATA_PARAMS newNode = { 0 };
+
+    newNode.event = event;
+    switch (event)
+    {
+        case NV0000_NOTIFIERS_DISPLAY_CHANGE:
+            newNode.data.display = *((NV0000_CTRL_SYSTEM_EVENT_DATA_DISPLAY_CHANGE *)pEventData);
+            break;
+
+        case NV0000_NOTIFIERS_VGPU_UNBIND_EVENT:
+            newNode.data.vgpuUnbind = *((NV0000_CTRL_SYSTEM_EVENT_DATA_VGPU_UNBIND *)pEventData);
+            break;
+
+        case NV0000_NOTIFIERS_VGPU_BIND_EVENT:
+            newNode.data.vgpuBind = *((NV0000_CTRL_SYSTEM_EVENT_DATA_VGPU_BIND *)pEventData);
+            break;
+
+        case NV0000_NOTIFIERS_GPU_BIND_UNBIND_EVENT:
+            newNode.data.gpuBindUnbind = *((NV0000_CTRL_SYSTEM_EVENT_DATA_GPU_BIND_UNBIND *)pEventData);
+            break;
+
+        default:
+            return NV_ERR_INVALID_EVENT;
+    }
+
+    if (listAppendValue(pQueue, &newNode) == NULL)
+        return NV_ERR_NO_MEMORY;
+
+    return NV_OK;
+}
+
+static void eventSystemDequeueEventLatest(SystemEventQueueList *pQueue)
+{
+    NV0000_CTRL_GET_SYSTEM_EVENT_DATA_PARAMS *pLastNode = listTail(pQueue);
+
+    if (pLastNode == NULL)
+        return;
+
+    listRemove(pQueue, pLastNode);
+}
+
+NV_STATUS eventSystemDequeueEvent(SystemEventQueueList *pQueue, NV0000_CTRL_GET_SYSTEM_EVENT_DATA_PARAMS *pEvent)
+{
+    NV0000_CTRL_GET_SYSTEM_EVENT_DATA_PARAMS *pFirstNode = listHead(pQueue);
+
+    if (pFirstNode == NULL)
+    {
+        // Queue is empty
+        return NV_ERR_OBJECT_NOT_FOUND;
+    }
+
+    *pEvent = *pFirstNode;
+
+    listRemove(pQueue, pFirstNode);
+
+    return NV_OK;
+}
+
+void eventSystemClearEventQueue(SystemEventQueueList *pQueue)
+{
+    listClear(pQueue);
+}
