@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1993-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1993-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -48,6 +48,8 @@
 
 static void RmUnmapBusAperture (OBJGPU *, NvP64, NvU64, NvBool, NvP64);
 
+#include "gpu/conf_compute/conf_compute.h"
+
 typedef struct RS_CPU_MAP_PARAMS RmMapParams;
 typedef struct RS_CPU_UNMAP_PARAMS RmUnmapParams;
 
@@ -100,56 +102,7 @@ rmapiGetEffectiveAddrSpace
     NV_ADDRESS_SPACE *pAddrSpace
 )
 {
-    NV_ADDRESS_SPACE addrSpace;
-    NvBool bDirectSysMappingAllowed = NV_TRUE;
-
-    KernelBus *pKernelBus = GPU_GET_KERNEL_BUS(pGpu);
-
-    NV_ASSERT_OK_OR_RETURN(
-        kbusIsDirectMappingAllowed_HAL(pGpu, pKernelBus, pMemDesc, mapFlags,
-                                      &bDirectSysMappingAllowed));
-
-    //
-    // Bug 1482818: Deprecate reflected mappings in production code.
-    //  The usage of reflected writes, in addition to causing several deadlock
-    //  scenarios involving P2P transfers, are disallowed on NVLINK (along with
-    //  reflected reads), and should no longer be used.
-    //  The below PDB property should be unset once the remaining usages in MODS
-    //  have been culled. (Bug 1780557)
-    //
-    if ((memdescGetAddressSpace(pMemDesc) == ADDR_SYSMEM) &&
-        !bDirectSysMappingAllowed &&
-        (DRF_VAL(OS33, _FLAGS, _MAPPING, mapFlags) != NVOS33_FLAGS_MAPPING_DIRECT) &&
-        !kbusIsReflectedMappingAccessAllowed(pKernelBus))
-    {
-        NV_ASSERT(0);
-        return NV_ERR_NOT_SUPPORTED;
-    }
-
-    if (memdescGetFlag(pMemDesc, MEMDESC_FLAGS_MAP_SYSCOH_OVER_BAR1))
-    {
-        addrSpace = ADDR_FBMEM;
-    }
-    else if ((memdescGetAddressSpace(pMemDesc) == ADDR_SYSMEM) &&
-        (bDirectSysMappingAllowed || FLD_TEST_DRF(OS33, _FLAGS, _MAPPING, _DIRECT, mapFlags) ||
-        (IS_VIRTUAL_WITH_SRIOV(pGpu) && !IS_FMODEL(pGpu) && !IS_RTLSIM(pGpu))))
-    {
-        addrSpace = ADDR_SYSMEM;
-    }
-    else if ((memdescGetAddressSpace(pMemDesc) == ADDR_FBMEM) ||
-             ((memdescGetAddressSpace(pMemDesc) == ADDR_SYSMEM) && !bDirectSysMappingAllowed))
-    {
-        addrSpace = ADDR_FBMEM;
-    }
-    else
-    {
-        addrSpace = memdescGetAddressSpace(pMemDesc);
-    }
-
-    if (pAddrSpace)
-        *pAddrSpace = addrSpace;
-
-    return NV_OK;
+    return kbusGetEffectiveAddressSpace_HAL(pGpu, pMemDesc, mapFlags, pAddrSpace);
 }
 
 // Asserts to check caching type matches across sdk and nv_memory_types
@@ -176,8 +129,7 @@ memMap_IMPL
     KernelBus *pKernelBus = NULL;
     MemoryManager *pMemoryManager = NULL;
     KernelMemorySystem *pKernelMemorySystem = NULL;
-    RsClient *pRsClient;
-    RmClient *pRmClient;
+    RmClient *pClient;
     RsResourceRef *pContextRef;
     RsResourceRef *pMemoryRef;
     Memory *pMemoryInfo; // TODO: rename this field. pMemoryInfo is the legacy name.
@@ -206,8 +158,10 @@ memMap_IMPL
         pKernelMemorySystem = GPU_GET_KERNEL_MEMORY_SYSTEM(pGpu);
     }
 
-    NV_ASSERT_OK_OR_RETURN(serverGetClientUnderLock(&g_resServ, pMapParams->hClient, &pRsClient));
-    NV_ASSERT_OK_OR_RETURN(clientGetResourceRef(pRsClient, pMapParams->hMemory, &pMemoryRef));
+    pClient = serverutilGetClientUnderLock(pMapParams->hClient);
+    NV_ASSERT_OR_ELSE(pClient != NULL, return NV_ERR_INVALID_CLIENT);
+    NV_ASSERT_OK_OR_RETURN(clientGetResourceRef(staticCast(pClient, RsClient),
+                pMapParams->hMemory, &pMemoryRef));
 
     pMemoryInfo = dynamicCast(pMemoryRef->pResource, Memory);
     NV_ASSERT_OR_RETURN(pMemoryInfo != NULL, NV_ERR_NOT_SUPPORTED);
@@ -228,12 +182,42 @@ memMap_IMPL
     //
     // CPU to directly access protected memory is allowed on MODS
     //
+    // The check below is for VPR and should be skipped for Hopper CC
+    if ((pGpu != NULL) && !gpuIsCCFeatureEnabled(pGpu))
+    {
         if ((pMemoryInfo->Flags & NVOS32_ALLOC_FLAGS_PROTECTED) &&
             (pMapParams->protect != NV_PROTECT_WRITEABLE) &&
             ! RMCFG_FEATURE_PLATFORM_MODS)
         {
             return NV_ERR_NOT_SUPPORTED;
         }
+    }
+
+    if ((pGpu != NULL) && gpuIsCCFeatureEnabled(pGpu) &&
+        (pMemoryInfo->Flags & NVOS32_ALLOC_FLAGS_PROTECTED))
+    {
+        ConfidentialCompute *pCC = GPU_GET_CONF_COMPUTE(pGpu);
+        //
+        // If neither BAR1 nor PCIE as a whole is trusted, fail the mapping
+        // for allocations in CPR region. Mapping should still succeed for
+        // allocations in non-CPR region
+        // Deny BAR1 access to CPU-RM by default irrespective of prod or devtools
+        // mode. Some mappings made by CPU-RM may be allowed to go thorough in
+        // devtools mode.
+        // However, allow the mapping to go through on platforms where GSP-DMA
+        // is not present e.g. MODS. User may have also set a regkey to force
+        // BAR accesses.
+        //
+        if (((pCC != NULL) && !pCC->ccStaticInfo.bIsBar1Trusted &&
+            !pCC->ccStaticInfo.bIsPcieTrusted) ||
+            (IS_GSP_CLIENT(pGpu) && pMapParams->bKernel && !pKernelBus->bForceBarAccessOnHcc &&
+             FLD_TEST_DRF(OS33, _FLAGS, _ALLOW_MAPPING_ON_HCC, _NO, pMapParams->flags)))
+        {
+            NV_PRINTF(LEVEL_ERROR, "BAR1 mapping to CPR vidmem not supported\n");
+            NV_ASSERT(0);
+            return NV_ERR_NOT_SUPPORTED;
+        }
+    }
 
     if (!pMapParams->bKernel &&
         FLD_TEST_DRF(OS32, _ATTR2, _PROTECTION_USER, _READ_ONLY, pMemoryInfo->Attr2) &&
@@ -277,7 +261,7 @@ memMap_IMPL
         effectiveAddrSpace = ADDR_SYSMEM;
     }
 
-    bIsSysmem = (effectiveAddrSpace == ADDR_SYSMEM);
+    bIsSysmem = (effectiveAddrSpace == ADDR_SYSMEM) || (effectiveAddrSpace == ADDR_EGM);
 
     if (dynamicCast(pMemoryInfo, FlaMemory) != NULL)
     {
@@ -299,35 +283,16 @@ memMap_IMPL
         {
             if (pMapParams->bKernel)
             {
-                if (pMemDesc->_flags & MEMDESC_FLAGS_PHYSICALLY_CONTIGUOUS)
-                {
-                    NvP64 tempCpuPtr = kbusMapCoherentCpuMapping_HAL(pGpu, pKernelBus, pMemDesc);
-                    if (tempCpuPtr == NULL)
-                    {
-                        rmStatus = NV_ERR_GENERIC;
-                    }
-                    else
-                    {
-                        rmStatus = NV_OK;
-                        tempCpuPtr = NvP64_PLUS_OFFSET(tempCpuPtr, pMapParams->offset);
-                    }
-                    *pMapParams->ppCpuVirtAddr = tempCpuPtr;
-
-                    if (rmStatus != NV_OK)
-                        return rmStatus;
-                }
-                else
-                {
-                    rmStatus = osMapSystemMemory(pMemDesc,
-                                                 pMapParams->offset,
-                                                 pMapParams->length,
-                                                 pMapParams->bKernel,
-                                                 pMapParams->protect,
-                                                 pMapParams->ppCpuVirtAddr,
-                                                 &priv);
-                    if (rmStatus != NV_OK)
-                        return rmStatus;
-                }
+                rmStatus = kbusMapCoherentCpuMapping_HAL(pGpu,
+                                                         pKernelBus,
+                                                         pMemDesc,
+                                                         pMapParams->offset,
+                                                         pMapParams->length,
+                                                         pMapParams->protect,
+                                                         pMapParams->ppCpuVirtAddr,
+                                                         &priv);
+                if (rmStatus != NV_OK)
+                    return rmStatus;
             }
             else
             {
@@ -340,22 +305,11 @@ memMap_IMPL
                 //       this function will transition to storing the mapping parameters onto
                 //       the FD.  Also note: All mapping parameters are ignored (!).
                 //
-                //   For now, we're going to return the first page of the nvlink aperture
-                //   mapping of this allocation.  See nvidia_mmap_helper for establishment
-                //   of direct mapping.
-                //
+                //   TODO: refactor this to either use the new osMapMemoryArea interface or unify with kernel in osMapSystemMemory
 
-                rmStatus = osMapPciMemoryUser(pGpu->pOsGpuInfo,
-                                              ((NvUPtr)pKernelMemorySystem->coherentCpuFbBase +
-                                               (NvUPtr)memdescGetPhysAddr(pMemDesc,
-                                                AT_CPU, pMapParams->offset)),
-                                              pMapParams->length,
-                                              pMapParams->protect,
-                                              pMapParams->ppCpuVirtAddr,
-                                              &priv,
-                                              NV_MEMORY_UNCACHED);
-                if (rmStatus != NV_OK)
-                    return rmStatus;
+                rmStatus = NV_OK;
+                *((NvU64*) pMapParams->ppCpuVirtAddr) = ((NvU64) pKernelMemorySystem->coherentCpuFbBase) +
+                    ((NvU64) memdescGetPhysAddr(pMemDesc, AT_CPU, pMapParams->offset));
             }
 
             NV_PRINTF(LEVEL_INFO,
@@ -370,8 +324,8 @@ memMap_IMPL
                                                     priv,
                                                     *(pMapParams->ppCpuVirtAddr),
                                                     pMapParams->length,
-                                                    -1,
-                                                    -1,
+                                                    0,
+                                                    0,
                                                     pMapParams->flags);
             pCpuMapping->pPrivate->pGpu = pGpu;
 
@@ -396,6 +350,8 @@ memMap_IMPL
         NvBool bcState = NV_FALSE;
         NvU64 gpuVirtAddr = 0;
         NvU64 gpuMapLength = 0;
+        MemoryArea memArea;
+        NvBool bUseMemArea = NV_FALSE;
 
         //
         // MEMDESC_FLAGS_MAP_SYSCOH_OVER_BAR1 indicates a special mapping type of HW registers,
@@ -436,8 +392,13 @@ memMap_IMPL
             NV_ASSERT(pGpu->busInfo.gpuPhysFbAddr);
 
             {
+                Device *pDevice = NULL;
+
                 // Below, we only map one GPU's address for CPU access, so we can use UNICAST here
                 NvU32 busMapFbFlags = BUS_MAP_FB_FLAGS_MAP_UNICAST;
+#if defined(NV_UNIX)
+                busMapFbFlags |= pMapParams->bKernel ? 0 : BUS_MAP_FB_FLAGS_ALLOW_DISCONTIG;
+#endif
                 if(DRF_VAL(OS33, _FLAGS, _MAPPING, pMapParams->flags) == NVOS33_FLAGS_MAPPING_DIRECT)
                 {
                     busMapFbFlags |= BUS_MAP_FB_FLAGS_DISABLE_ENCRYPTION;
@@ -456,16 +417,25 @@ memMap_IMPL
                 pMemDesc = memdescGetMemDescFromGpu(pMemDesc, pGpu);
 
                 // WAR for Bug 3564398, need to allocate doorbell for windows differently
-                if (RMCFG_FEATURE_PLATFORM_WINDOWS_LDDM &&
-                    memdescGetFlag(pMemDesc, MEMDESC_FLAGS_MAP_SYSCOH_OVER_BAR1)) 
+                if (RMCFG_FEATURE_PLATFORM_WINDOWS &&
+                    memdescGetFlag(pMemDesc, MEMDESC_FLAGS_MAP_SYSCOH_OVER_BAR1))
                 {
                     busMapFbFlags |= BUS_MAP_FB_FLAGS_MAP_DOWNWARDS;
                 }
 
+                (void) deviceGetByHandle(staticCast(pClient, RsClient),
+                                         pMapParams->hDevice, &pDevice);
+                bUseMemArea = NV_TRUE;
                 rmStatus = kbusMapFbAperture_HAL(pGpu, pKernelBus,
-                                                 pMemDesc, pMapParams->offset,
-                                                 &gpuVirtAddr, &gpuMapLength,
-                                                 busMapFbFlags, pMapParams->hClient);
+                                                   pMemDesc, mrangeMake(pMapParams->offset, gpuMapLength),
+                                                   &memArea, busMapFbFlags, pDevice);
+
+                if (rmStatus == NV_OK)
+                {
+                    gpuVirtAddr = memArea.pRanges[0].start;
+                }
+
+
             }
 
             if (rmStatus != NV_OK)
@@ -490,7 +460,7 @@ memMap_IMPL
                                               pMapParams->ppCpuVirtAddr,
                                               cachingType);
         }
-        else
+        else if(!bUseMemArea)
         {
             rmStatus = osMapPciMemoryUser(pGpu->pOsGpuInfo,
                                           (kbusIsBar1PhysicalModeEnabled(pKernelBus)?
@@ -501,6 +471,29 @@ memMap_IMPL
                                           &priv,
                                           cachingType);
         }
+        else
+        {
+            NvU64 idx;
+            NvU64 barAddr = gpumgrGetGpuPhysFbAddr(pGpu);
+
+            for (idx = 0; idx < memArea.numRanges; idx++)
+            {
+                memArea.pRanges[idx].start += barAddr;
+            }
+
+            rmStatus = osMapPciMemoryAreaUser(pGpu->pOsGpuInfo,
+                                          memArea,
+                                          pMapParams->protect,
+                                          cachingType,
+                                          pMapParams->ppCpuVirtAddr,
+                                          &priv);
+
+            for (idx = 0; idx < memArea.numRanges; idx++)
+            {
+                memArea.pRanges[idx].start -= barAddr;
+            }
+
+        }
 
         //
         // It's possible that NVOS33_FLAGS_MAPPING is set to NVOS33_FLAGS_MAPPING_DIRECT
@@ -519,12 +512,17 @@ memMap_IMPL
                                                 *(pMapParams->ppCpuVirtAddr),
                                                 pMapParams->length,
                                                 kbusIsBar1PhysicalModeEnabled(pKernelBus)
-                                                    ? (NvU64)-1
+                                                    ? (NvU64)0
                                                     : gpuVirtAddr,
                                                 kbusIsBar1PhysicalModeEnabled(pKernelBus)
-                                                    ? (NvU64)-1
+                                                    ? (NvU64)0
                                                     : gpuMapLength,
                                                 pMapParams->flags);
+
+        if (bUseMemArea)
+        {
+            pCpuMapping->pPrivate->memArea = memArea;
+        }
         pCpuMapping->pPrivate->pGpu = pGpu;
 
         if (rmStatus != NV_OK)
@@ -538,11 +536,10 @@ memMap_IMPL
             if (!kbusIsBar1PhysicalModeEnabled(pKernelBus))
             {
                 kbusUnmapFbAperture_HAL(pGpu,
-                                        pKernelBus,
-                                        pMemDesc,
-                                        gpuVirtAddr,
-                                        gpuMapLength,
-                                        BUS_MAP_FB_FLAGS_MAP_UNICAST);
+                                          pKernelBus,
+                                          pMemDesc,
+                                          memArea,
+                                          BUS_MAP_FB_FLAGS_MAP_UNICAST);
     _rmMapMemory_busFail:
                 gpumgrSetBcEnabledStatus(pGpu, bcState);
             }
@@ -583,13 +580,12 @@ memMap_IMPL
     {
         RS_PRIV_LEVEL privLevel;
 
-        pRmClient = dynamicCast(pRsClient, RmClient);
-        if (pRmClient == NULL)
-            return NV_ERR_OPERATING_SYSTEM;
-
-        privLevel = rmclientGetCachedPrivilege(pRmClient);
-        if (!rmclientIsAdmin(pRmClient, privLevel) && !memdescGetFlag(pMemDesc, MEMDESC_FLAGS_SKIP_REGMEM_PRIV_CHECK))
+        privLevel = rmclientGetCachedPrivilege(pClient);
+        if (!rmclientIsAdmin(pClient, privLevel) &&
+            !memdescGetFlag(pMemDesc, MEMDESC_FLAGS_SKIP_REGMEM_PRIV_CHECK))
+        {
             return NV_ERR_PROTECTION_FAULT;
+        }
 
         if (DRF_VAL(OS33, _FLAGS, _MEM_SPACE, pMapParams->flags) == NVOS33_FLAGS_MEM_SPACE_USER)
         {
@@ -613,8 +609,8 @@ memMap_IMPL
                                                 priv,
                                                 *(pMapParams->ppCpuVirtAddr),
                                                 pMapParams->length,
-                                                -1, // gpu virtual addr
-                                                -1, // gpu map length
+                                                0, // gpu virtual addr
+                                                0, // gpu map length
                                                 pMapParams->flags);
         pCpuMapping->pPrivate->pGpu = pGpu;
 
@@ -679,19 +675,9 @@ memUnmap_IMPL
 
         if (pCpuMapping->pPrivate->bKernel)
         {
-            if(pMemDesc->_flags & MEMDESC_FLAGS_PHYSICALLY_CONTIGUOUS)
-            {
-                NV_ASSERT(pMemDesc->_flags & MEMDESC_FLAGS_PHYSICALLY_CONTIGUOUS);
-                kbusUnmapCoherentCpuMapping_HAL(pGpu, pKernelBus, pMemDesc);
-            }
-            else
-            {
-                osUnmapSystemMemory(pMemDesc,
-                                    pCpuMapping->pPrivate->bKernel,
-                                    pCpuMapping->processId,
-                                    pCpuMapping->pLinearAddress,
-                                    pCpuMapping->pPrivate->pPriv);
-            }
+            kbusUnmapCoherentCpuMapping_HAL(pGpu, pKernelBus, pMemDesc,
+                                            pCpuMapping->pLinearAddress,
+                                            pCpuMapping->pPrivate->pPriv);
         }
 
         NV_PRINTF(LEVEL_INFO,
@@ -705,7 +691,8 @@ memUnmap_IMPL
         //
     }
     // System Memory case
-    else if ((pGpu == NULL) || ((memdescGetAddressSpace(pMemDesc) == ADDR_SYSMEM) &&
+    else if ((pGpu == NULL) || (((memdescGetAddressSpace(pMemDesc) == ADDR_SYSMEM) ||
+                                  (memdescGetAddressSpace(pMemDesc) == ADDR_EGM)) &&
                                  FLD_TEST_DRF(OS33, _FLAGS, _MAPPING, _DIRECT, pCpuMapping->flags)))
     {
         if (FLD_TEST_DRF(OS33, _FLAGS, _MAPPING, _DIRECT, pCpuMapping->flags))
@@ -731,35 +718,15 @@ memUnmap_IMPL
         {
             {
                 kbusUnmapFbAperture_HAL(pGpu, pKernelBus,
-                                        pMemory->pMemDesc,
-                                        pCpuMapping->pPrivate->gpuAddress,
-                                        pCpuMapping->pPrivate->gpuMapLength,
-                                        BUS_MAP_FB_FLAGS_MAP_UNICAST);
+                                          pMemory->pMemDesc,
+                                          pCpuMapping->pPrivate->memArea,
+                                          BUS_MAP_FB_FLAGS_MAP_UNICAST);
             }
         }
     }
     else if (memdescGetAddressSpace(pMemDesc) == ADDR_VIRTUAL)
     {
-        // If the memory is tiled, then it's being mapped through BAR1
-        if( DRF_VAL(OS32, _ATTR, _TILED, pMemory->Attr) )
-        {
-            // BAR1 mapping.  Unmap it.
-            if (pCpuMapping->pPrivate->bKernel)
-            {
-                osUnmapPciMemoryKernel64(pGpu, pCpuMapping->pLinearAddress);
-            }
-            else
-            {
-                osUnmapPciMemoryUser(pGpu->pOsGpuInfo,
-                                     pCpuMapping->pLinearAddress,
-                                     pCpuMapping->length,
-                                     pCpuMapping->pPrivate->pPriv);
-            }
-        }
-        else
-        {
-            NV_ASSERT_OR_RETURN(0, NV_ERR_INVALID_STATE);
-        }
+        NV_ASSERT_OR_RETURN(0, NV_ERR_INVALID_STATE);
     }
     else if (memdescGetAddressSpace(pMemDesc) == ADDR_REGMEM)
     {
@@ -815,8 +782,7 @@ serverMap_Prologue
 )
 {
     NV_STATUS           rmStatus;
-    RsClient           *pRsClient;
-    RmClient           *pRmClient;
+    RmClient           *pClient;
     RsResourceRef      *pMemoryRef;
     NvHandle            hClient = pMapParams->hClient;
     NvHandle            hParent = hClient;
@@ -830,16 +796,15 @@ serverMap_Prologue
         return NV_ERR_INVALID_FLAGS;
 
     // Populate Resource Server information
-    NV_ASSERT_OK_OR_RETURN(serverGetClientUnderLock(&g_resServ, hClient, &pRsClient));
+    pClient = serverutilGetClientUnderLock(hClient);
+    NV_ASSERT_OR_ELSE(pClient != NULL, return NV_ERR_INVALID_CLIENT);
 
     // Validate hClient
-    pRmClient = dynamicCast(pRsClient, RmClient);
-    if (pRmClient == NULL)
-        return NV_ERR_OPERATING_SYSTEM;
-    privLevel = rmclientGetCachedPrivilege(pRmClient);
+    privLevel = rmclientGetCachedPrivilege(pClient);
 
     // RS-TODO: Assert if this fails after all objects are converted
-    NV_ASSERT_OK_OR_RETURN(clientGetResourceRef(pRsClient, pMapParams->hMemory, &pMemoryRef));
+    NV_ASSERT_OK_OR_RETURN(clientGetResourceRef(staticCast(pClient, RsClient),
+                pMapParams->hMemory, &pMemoryRef));
 
     if (pMemoryRef->pParentRef != NULL)
         hParent = pMemoryRef->pParentRef->hResource;
@@ -859,7 +824,9 @@ serverMap_Prologue
         NV_ASSERT_OR_RETURN(hParent != hClient, NV_ERR_INVALID_OBJECT_PARENT);
 
         RsResourceRef *pContextRef;
-        rmStatus = clientGetResourceRef(pRsClient, pMapParams->hDevice, &pContextRef);
+        rmStatus = clientGetResourceRef(staticCast(pClient, RsClient),
+                pMapParams->hDevice, &pContextRef);
+
         if (rmStatus != NV_OK)
             return rmStatus;
 
@@ -917,8 +884,7 @@ serverUnmap_Prologue
 {
     OBJGPU *pGpu = NULL;
     NV_STATUS rmStatus;
-    RsClient *pRsClient;
-    RmClient *pRmClient;
+    RmClient *pClient;
     RsResourceRef *pMemoryRef;
     NvHandle hClient = pUnmapParams->hClient;
     NvHandle hParent = hClient;
@@ -931,16 +897,15 @@ serverUnmap_Prologue
     void *pProcessHandle = NULL;
 
     // Populate Resource Server information
-    NV_ASSERT_OK_OR_RETURN(serverGetClientUnderLock(&g_resServ, hClient, &pRsClient));
+    pClient = serverutilGetClientUnderLock(hClient);
+    NV_ASSERT_OR_ELSE(pClient != NULL, return NV_ERR_INVALID_CLIENT);
 
     // check if we have a user or kernel RM client
-    pRmClient = dynamicCast(pRsClient, RmClient);
-    if (pRmClient == NULL)
-        return NV_ERR_OPERATING_SYSTEM;
-    privLevel = rmclientGetCachedPrivilege(pRmClient);
+    privLevel = rmclientGetCachedPrivilege(pClient);
 
     // RS-TODO: Assert if this fails after all objects are converted
-    NV_ASSERT_OK_OR_RETURN(clientGetResourceRef(pRsClient, hMemory, &pMemoryRef));
+    NV_ASSERT_OK_OR_RETURN(clientGetResourceRef(staticCast(pClient, RsClient),
+                hMemory, &pMemoryRef));
 
     if (pMemoryRef->pParentRef != NULL)
         hParent = pMemoryRef->pParentRef->hResource;
@@ -955,7 +920,9 @@ serverUnmap_Prologue
         NV_ASSERT_OR_RETURN(hParent != hClient, NV_ERR_INVALID_OBJECT_PARENT);
 
         RsResourceRef *pContextRef;
-        rmStatus = clientGetResourceRef(pRsClient, pUnmapParams->hDevice, &pContextRef);
+        rmStatus = clientGetResourceRef(staticCast(pClient, RsClient),
+                pUnmapParams->hDevice, &pContextRef);
+
         if (rmStatus != NV_OK)
             return rmStatus;
 
@@ -1129,7 +1096,23 @@ rmapiMapToCpuWithSecInfoV2
     NV_PRINTF(LEVEL_INFO, "MMU_PROFILER Nv04MapMemory 0x%x\n", *flags);
 
     portMemSet(&lockInfo, 0, sizeof(lockInfo));
-    rmapiInitLockInfo(pRmApi, hClient, &lockInfo);
+    status = rmapiInitLockInfo(pRmApi, hClient, NV01_NULL_OBJECT, &lockInfo);
+    if (status != NV_OK)
+    {
+        rmapiEpilogue(pRmApi, &rmApiContext);
+        return status;
+    }
+
+    //
+    // In the RTD3 case, the API lock isn't taken since it can be initiated
+    // from another thread that holds the API lock and because we now hold
+    // the GPU lock.
+    //
+    if (rmapiInRtd3PmPath())
+    {
+        lockInfo.flags |= RM_LOCK_FLAGS_NO_API_LOCK;
+        lockInfo.state &= ~RM_LOCK_STATES_API_LOCK_ACQUIRED;
+    }
 
     LOCK_METER_DATA(MAPMEM, flags, 0, 0);
 
@@ -1184,8 +1167,8 @@ rmapiMapToCpuWithSecInfo
     API_SECURITY_INFO *pSecInfo
 )
 {
-    return rmapiMapToCpuWithSecInfoV2(pRmApi, hClient, 
-        hDevice, hMemory, offset, length, ppCpuVirtAddr, 
+    return rmapiMapToCpuWithSecInfoV2(pRmApi, hClient,
+        hDevice, hMemory, offset, length, ppCpuVirtAddr,
         &flags, pSecInfo);
 }
 
@@ -1299,7 +1282,12 @@ rmapiUnmapFromCpuWithSecInfo
         return status;
 
     portMemSet(&lockInfo, 0, sizeof(lockInfo));
-    rmapiInitLockInfo(pRmApi, hClient, &lockInfo);
+    status = rmapiInitLockInfo(pRmApi, hClient, NV01_NULL_OBJECT, &lockInfo);
+    if (status != NV_OK)
+    {
+        rmapiEpilogue(pRmApi, &rmApiContext);
+        return NV_OK;
+    }
 
     LOCK_METER_DATA(UNMAPMEM, flags, 0, 0);
 
