@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1993-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1993-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -36,6 +36,7 @@
 #include "core/thread_state.h"
 #include "gpu_mgr/gpu_mgr.h"
 #include "resource_desc.h"
+#include "ctrl/ctrl0000/ctrl0000system.h"
 
 typedef struct
 {
@@ -46,6 +47,9 @@ typedef struct
     NvU64               tlsEntryId;
     volatile NvU32      contentionCount;
     NvU32               lowPriorityAging;
+    volatile NvU64      totalWaitTime;
+    volatile NvU64      totalRwHoldTime;
+    volatile NvU64      totalRoHoldTime;
 } RMAPI_LOCK;
 
 RsServer          g_resServ;
@@ -53,6 +57,8 @@ static RM_API     g_RmApiList[RMAPI_TYPE_MAX];
 static NvBool     g_bResServInit = NV_FALSE;
 static RMAPI_LOCK g_RmApiLock;
 
+static NvU64 g_rtd3PmPathThreadId = ~0ULL;
+
 static void _rmapiInitInterface(RM_API *pRmApi, API_SECURITY_INFO *pDefaultSecurityInfo, NvBool bTlsInternal,
                                 NvBool bApiLockInternal, NvBool bGpuLockInternal);
 static NV_STATUS _rmapiLockAlloc(void);
@@ -217,6 +223,65 @@ rmapiGetInterface
     return &g_RmApiList[rmapiType];
 }
 
+static void
+_rmapiUnrefGpuAccessNeeded
+(
+    NvU32           gpuMask
+)
+{
+    NvU32           gpuInstance = 0;
+    OBJGPU         *pGpu = NULL;
+
+    while ((pGpu = gpumgrGetNextGpu(gpuMask, &gpuInstance)) != NULL)
+    {
+        osUnrefGpuAccessNeeded(pGpu->pOsGpuInfo);
+    }
+}
+
+static NV_STATUS
+_rmapiRefGpuAccessNeeded
+(
+    NvU32          *pGpuMask
+)
+{
+    NV_STATUS       status = NV_OK;
+    NvU32           mask = 0;
+    NvU32           gpuInstance = 0;
+    OBJGPU         *pGpu = NULL;
+
+    status = gpumgrGetGpuAttachInfo(NULL, &mask);
+    if (status != NV_OK)
+    {
+        return status;
+    }
+
+    while ((pGpu = gpumgrGetNextGpu(mask, &gpuInstance)) != NULL)
+    {
+        status = osRefGpuAccessNeeded(pGpu->pOsGpuInfo);
+        if (status != NV_OK)
+        {
+            goto unref;
+        }
+
+       /*
+        *_rmapiRefGpuAccessNeeded records the gpuMask
+        * during ref up and this is used to unref exact same
+        * GPUs in _rmapiUnrefGpuAccessNeeded. This is done
+        * to protect against obtaining incorrect pGpu if the mask
+        * changes due to a RM_API called between ref/unref
+        * sequence.
+        */
+        *pGpuMask |= (1 << pGpu->gpuInstance);
+    }
+
+unref:
+    if (status != NV_OK)
+    {
+        _rmapiUnrefGpuAccessNeeded(*pGpuMask);
+    }
+    return status;
+}
+
 NV_STATUS
 rmapiPrologue
 (
@@ -225,6 +290,49 @@ rmapiPrologue
 )
 {
     NV_STATUS       status = NV_OK;
+    NvBool          bApiLockTaken = NV_FALSE;
+    NvU32           mask;
+
+    NV_ASSERT_OR_RETURN(pRmApi != NULL, NV_ERR_INVALID_ARGUMENT);
+    NV_ASSERT_OR_RETURN(pContext != NULL, NV_ERR_INVALID_ARGUMENT);
+
+    /*
+     * Check for external clients. This condition is checked here
+     * in order to avoid a check at all caller sites of
+     * rmapiPrologue. Effectively rmapiprologue is a no-op for
+     * internal clients.
+     */
+    if (!pRmApi->bTlsInternal)
+    {
+        mask = osGetDynamicPowerSupportMask();
+        if (!mask)
+            return status;
+       /*
+        * NOTE1: Callers of rmapiPro{Epi}logue function call may call
+        * it with or without API lock taken. Hence, we check here
+        * whether API lock has been taken. We take API lock if
+        * it not taken already.
+        * We obtain the pGPU by using the gpuMask in
+        * _rmapiRef{Unref}GpuAccessNeeded. This needs API lock to be
+        * safe against init/teardown of GPUs while we ref/unref
+        * the GPUs. We release the lock after we have finished
+        * with ref/unref, if we had taken it.
+        */
+       if (!rmapiLockIsOwner())
+       {
+           status = rmapiLockAcquire(RMAPI_LOCK_FLAGS_READ, RM_LOCK_MODULES_CLIENT);
+           if (status != NV_OK)
+           {
+               return status;
+           }
+           bApiLockTaken = NV_TRUE;
+       }
+       status = _rmapiRefGpuAccessNeeded(&pContext->gpuMask);
+       if (bApiLockTaken == NV_TRUE)
+       {
+           rmapiLockRelease();
+       }
+    }
     return status;
 }
 
@@ -235,35 +343,142 @@ rmapiEpilogue
     RM_API_CONTEXT *pContext
 )
 {
+    NV_STATUS       status = NV_OK;
+    NvBool          bApiLockTaken = NV_FALSE;
+    NvU32           mask;
+
+    NV_ASSERT_OR_RETURN_VOID(pRmApi != NULL);
+    NV_ASSERT_OR_RETURN_VOID(pContext != NULL);
+
+    /*
+     * Check for external clients. This condition is checked here
+     * in order to avoid a check at all caller sites of
+     * rmapiEpilogue. Effectively rmapiEpilogue is a no-op for
+     * internal clients.
+     */
+    if (!pRmApi->bTlsInternal)
+    {
+        mask = osGetDynamicPowerSupportMask();
+        if (!mask)
+            return;
+
+       /* Please see NOTE1 */
+       if (!rmapiLockIsOwner())
+       {
+           status = rmapiLockAcquire(RMAPI_LOCK_FLAGS_READ, RM_LOCK_MODULES_CLIENT);
+           if (status != NV_OK)
+           {
+               return;
+           }
+           bApiLockTaken = NV_TRUE;
+       }
+
+       _rmapiUnrefGpuAccessNeeded(pContext->gpuMask);
+
+       if (bApiLockTaken == NV_TRUE)
+       {
+           rmapiLockRelease();
+       }
+    }
 }
 
-void
+NV_STATUS
 rmapiInitLockInfo
 (
-    RM_API            *pRmApi,
-    NvHandle           hClient,
-    RS_LOCK_INFO      *pLockInfo
+    RM_API       *pRmApi,
+    NvHandle      hClient,
+    NvHandle      hSecondClient,
+    RS_LOCK_INFO *pLockInfo
 )
 {
-    NV_ASSERT_OR_RETURN_VOID(pLockInfo != NULL);
+    CALL_CONTEXT *pCallContext = resservGetTlsCallContext();
+
+    NV_ASSERT_OR_RETURN(pLockInfo != NULL, NV_ERR_INVALID_STATE);
     pLockInfo->flags = 0;
     pLockInfo->state = 0;
 
     if (hClient != 0)
     {
-        CALL_CONTEXT *pCallContext = resservGetTlsCallContext();
         if ((pCallContext != NULL) && (pCallContext->pLockInfo != NULL))
         {
             pLockInfo->state = pCallContext->pLockInfo->state;
 
-            if ((pCallContext->pLockInfo->pClient != NULL) &&
-                (pCallContext->pLockInfo->pClient->hClient == hClient))
+            if (!serverAllClientsLockIsOwner(&g_resServ))
             {
-                pLockInfo->pClient = pCallContext->pLockInfo->pClient;
-            }
-            else
-            {
-                pLockInfo->state &= ~RM_LOCK_STATES_CLIENT_LOCK_ACQUIRED;
+                // If no clients are locked, then we need to acquire client locks
+                if (pCallContext->pLockInfo->pClient == NULL)
+                    pLockInfo->state &= ~RM_LOCK_STATES_CLIENT_LOCK_ACQUIRED;
+
+                // If we only need one client locked
+                else if (hSecondClient == NV01_NULL_OBJECT)
+                {
+                    if (pCallContext->pLockInfo->pClient->hClient == hClient)
+                        pLockInfo->pClient = pCallContext->pLockInfo->pClient;
+                    else if ((pCallContext->pLockInfo->pSecondClient != NULL) &&
+                             (pCallContext->pLockInfo->pSecondClient->hClient == hClient))
+                    {
+                        pLockInfo->pClient = pCallContext->pLockInfo->pSecondClient;
+                    }
+                    else
+                        pLockInfo->state &= ~RM_LOCK_STATES_CLIENT_LOCK_ACQUIRED;
+                }
+
+                // If we only have one client locked, but we need two
+                else if (pCallContext->pLockInfo->pSecondClient == NULL)
+                {
+                    if ((pCallContext->pLockInfo->pClient->hClient == hClient) ||
+                         (pCallContext->pLockInfo->pClient->hClient == hSecondClient))
+                    {
+                        pLockInfo->pClient = pCallContext->pLockInfo->pClient;
+
+                        //
+                        // Special case: if both clients are the same -
+                        // Set both pClient's so _serverLockDualClientWithLockInfo
+                        // doesn't complain about the lock state being invalid.
+                        //
+                        if (hClient == hSecondClient)
+                            pLockInfo->pSecondClient = pCallContext->pLockInfo->pClient;
+                    }
+                    else
+                        pLockInfo->state &= ~RM_LOCK_STATES_CLIENT_LOCK_ACQUIRED;
+                }
+
+                // If we need two clients locked, and already have two
+                else
+                {
+                    //
+                    // Check whether both clients match, keep the original order of the
+                    // clients (dual client locking always locks the lower numbered client
+                    // handle first).
+                    //
+                    if (((pCallContext->pLockInfo->pClient->hClient == hClient) &&
+                         (pCallContext->pLockInfo->pSecondClient->hClient ==
+                          hSecondClient)) ||
+                        ((pCallContext->pLockInfo->pClient->hClient == hSecondClient) &&
+                         (pCallContext->pLockInfo->pSecondClient->hClient == hClient)))
+                    {
+                        pLockInfo->pClient = pCallContext->pLockInfo->pClient;
+                        pLockInfo->pSecondClient = pCallContext->pLockInfo->pSecondClient;
+                    }
+
+                    // Check whether one client handle matches
+                    else if ((pCallContext->pLockInfo->pClient->hClient == hClient) ||
+                             (pCallContext->pLockInfo->pClient->hClient == hSecondClient))
+                    {
+                        pLockInfo->pClient = pCallContext->pLockInfo->pClient;
+                        pLockInfo->state &= ~RM_LOCK_STATES_CLIENT_LOCK_ACQUIRED;
+                    }
+                    else if ((pCallContext->pLockInfo->pSecondClient->hClient ==
+                              hClient) ||
+                             (pCallContext->pLockInfo->pSecondClient->hClient ==
+                              hSecondClient))
+                    {
+                        pLockInfo->pClient = pCallContext->pLockInfo->pSecondClient;
+                        pLockInfo->state &= ~RM_LOCK_STATES_CLIENT_LOCK_ACQUIRED;
+                    }
+                    else
+                        pLockInfo->state &= ~RM_LOCK_STATES_CLIENT_LOCK_ACQUIRED;
+                }
             }
         }
     }
@@ -275,13 +490,21 @@ rmapiInitLockInfo
     {
         pLockInfo->state |= RM_LOCK_STATES_API_LOCK_ACQUIRED;
 
-        // RS-TODO: Assert that API rwlock is taken if no client is locked
-        if (pLockInfo->pClient == NULL)
+        //
+        // Don't acquire client locks if we already hold the API lock since we might've
+        // already acquired RM locks that are ordered after client locks (such as higher numbered
+        // client/GPU locks) and don't want to violate RM lock ordering.
+        //
+        if (rmapiLockIsOwner())
+        {
             pLockInfo->flags |= RM_LOCK_FLAGS_NO_CLIENT_LOCK;
+        }
     }
 
     if (pRmApi->bGpuLockInternal)
         pLockInfo->state |= RM_LOCK_STATES_ALLOW_RECURSIVE_LOCKS;
+
+    return NV_OK;
 }
 
 static NV_STATUS
@@ -327,26 +550,39 @@ _rmapiLockFree(void)
 NV_STATUS
 rmapiLockAcquire(NvU32 flags, NvU32 module)
 {
+    OBJSYS *pSys = SYS_GET_INSTANCE();
     NV_STATUS rmStatus = NV_OK;
     NvU64 threadId = portThreadGetCurrentThreadId();
 
     NvU64 myPriority = 0;
+    NvU64 startWaitTime = 0;
 
-    LOCK_ASSERT_AND_RETURN(!rmapiLockIsOwner());
+    // Make sure lock has been created
+    NV_CHECK_OR_RETURN(LEVEL_ERROR, g_RmApiLock.pLock != NULL, NV_ERR_NOT_READY);
+
+    NV_ASSERT_OR_RETURN(!rmapiLockIsOwner(), NV_ERR_INVALID_LOCK_STATE);
+
+    // Ensure that GPU locks are NEVER acquired before the API lock
+    NV_ASSERT_OR_RETURN(rmGpuLocksGetOwnedMask() == 0, NV_ERR_INVALID_LOCK_STATE);
 
     //
     // If a read-only lock was requested, check to see if the module is allowed
-    // to take read-only locks
+    // to take read-only locks or the _FORCE flag was enabled and set.
     //
     if ((flags & RMAPI_LOCK_FLAGS_READ) && (module != RM_LOCK_MODULES_NONE))
     {
-        OBJSYS *pSys = SYS_GET_INSTANCE();
-        if ((pSys->apiLockModuleMask & RM_LOCK_MODULE_GRP(module)) == 0)
+        if ((((flags & RMAPI_LOCK_FLAGS_READ_FORCE) == 0) ||
+                !pSys->getProperty(pSys, PDB_PROP_SYS_ENABLE_FORCE_SHARED_LOCK)) &&
+            ((pSys->apiLockModuleMask & RM_LOCK_MODULE_GRP(module)) == 0))
         {
             flags &= ~RMAPI_LOCK_FLAGS_READ;
         }
     }
 
+    // Get start wait time measuring lock wait times
+    if (pSys->getProperty(pSys, PDB_PROP_SYS_RM_LOCK_TIME_COLLECT))
+        startWaitTime = osGetCurrentTick();
+
     //
     // For conditional acquires and DISPATCH_LEVEL we want to exit
     // immediately without waiting.
@@ -388,6 +624,7 @@ rmapiLockAcquire(NvU32 flags, NvU32 module)
             if (flags & RMAPI_LOCK_FLAGS_LOW_PRIORITY)
             {
                 NvS32 age = g_RmApiLock.lowPriorityAging;
+
                 portSyncRwLockAcquireWrite(g_RmApiLock.pLock);
                 while ((g_RmApiLock.contentionCount > 0) && (age--))
                 {
@@ -410,7 +647,11 @@ rmapiLockAcquire(NvU32 flags, NvU32 module)
     if (rmStatus == NV_OK)
     {
         NvU64 timestamp;
-        osGetCurrentTick(&timestamp);
+        timestamp = osGetCurrentTick();
+
+        // Update total API lock wait time if measuring lock times
+        if (pSys->getProperty(pSys, PDB_PROP_SYS_RM_LOCK_TIME_COLLECT))
+            portAtomicExAddU64(&g_RmApiLock.totalWaitTime, timestamp - startWaitTime);
 
         if (g_RmApiLock.threadId == threadId)
             g_RmApiLock.timestamp = timestamp;
@@ -439,10 +680,17 @@ rmapiLockAcquire(NvU32 flags, NvU32 module)
         }
     }
 
-    NvP64 *pAcquireAddress = tlsEntryAcquire(g_RmApiLock.tlsEntryId);
-    if (pAcquireAddress != NULL)
+    NvP64 *pStartTime = tlsEntryAcquire(g_RmApiLock.tlsEntryId);
+    if (pStartTime != NULL)
     {
-        *pAcquireAddress = (NvP64)(NvUPtr)NV_RETURN_ADDRESS();
+        //
+        // Store start time to track lock hold time. This is done
+        // regardless of the value of PDB_PROP_SYS_RM_LOCK_TIME_COLLECT since
+        // the API lock can be acquired before PDB properties are initialized
+        // and released after they are which could lead to uninitialized memory
+        // being present in TLS.
+        //
+        *(NvU64*)pStartTime = osGetCurrentTick();
     }
 
     return rmStatus;
@@ -451,10 +699,16 @@ rmapiLockAcquire(NvU32 flags, NvU32 module)
 void
 rmapiLockRelease(void)
 {
+    OBJSYS *pSys = SYS_GET_INSTANCE();
     NvU64 threadId = portThreadGetCurrentThreadId();
     NvU64 timestamp;
+    NvU64 startTime = 0;
 
-    osGetCurrentTick(&timestamp);
+    // Fetch start of hold time from TLS if measuring lock times
+    if (pSys->getProperty(pSys, PDB_PROP_SYS_RM_LOCK_TIME_COLLECT))
+        startTime = (NvU64) tlsEntryGet(g_RmApiLock.tlsEntryId);
+
+    timestamp = osGetCurrentTick();
 
     RMTRACE_RMLOCK(_API_LOCK_RELEASE);
 
@@ -476,11 +730,20 @@ rmapiLockRelease(void)
         //
         g_RmApiLock.threadId  = ~0ull;
         g_RmApiLock.timestamp = timestamp;
+
+        // Update total RW API lock hold time if measuring lock times
+        if (pSys->getProperty(pSys, PDB_PROP_SYS_RM_LOCK_TIME_COLLECT))
+            portAtomicExAddU64(&g_RmApiLock.totalRwHoldTime, timestamp - startTime);
+
         portSyncRwLockReleaseWrite(g_RmApiLock.pLock);
 
     }
     else
     {
+        // Update total RO API lock hold time if measuring lock times
+        if (pSys->getProperty(pSys, PDB_PROP_SYS_RM_LOCK_TIME_COLLECT))
+            portAtomicExAddU64(&g_RmApiLock.totalRoHoldTime, timestamp - startTime);
+
         portSyncRwLockReleaseRead(g_RmApiLock.pLock);
     }
 
@@ -490,7 +753,58 @@ rmapiLockRelease(void)
 NvBool
 rmapiLockIsOwner(void)
 {
-    return tlsEntryGet(g_RmApiLock.tlsEntryId) != NvP64_NULL;
+    return tlsEntryGet(g_RmApiLock.tlsEntryId) != 0;
+}
+
+NvBool
+rmapiLockIsWriteOwner(void)
+{
+    NvU64 threadId = portThreadGetCurrentThreadId();
+
+    return (rmapiLockIsOwner() && (threadId == g_RmApiLock.threadId));
+}
+
+//
+// Retrieve total RM API lock wait and hold times
+//
+void
+rmapiLockGetTimes(NV0000_CTRL_SYSTEM_GET_LOCK_TIMES_PARAMS *pParams)
+{
+    pParams->waitApiLock   = g_RmApiLock.totalWaitTime;
+    pParams->holdRoApiLock = g_RmApiLock.totalRoHoldTime;
+    pParams->holdRwApiLock = g_RmApiLock.totalRwHoldTime;
+}
+
+//
+// Indicates current thread is in the RTD3 PM path (rm_transition_dynamic_power) which
+// means that certain locking asserts/checks must be skipped due to inability to acquire
+// the API lock in this path.
+//
+void rmapiEnterRtd3PmPath(void)
+{
+    // RTD3 path cannot be entered without the GPU lock
+    NV_ASSERT(rmGpuLockIsOwner());
+
+    NV_ASSERT(g_rtd3PmPathThreadId == ~0ULL);
+    g_rtd3PmPathThreadId = portThreadGetCurrentThreadId();
+}
+
+//
+// Signifies that current thread is leaving the RTD3 PM path, restoring lock
+// asserting/checking behavior to normal.
+//
+void rmapiLeaveRtd3PmPath(void)
+{
+    NV_ASSERT(rmapiInRtd3PmPath());
+    g_rtd3PmPathThreadId = ~0ULL;
+}
+
+//
+// Checks if current thread is currently running in the RTD3 PM path.
+//
+NvBool rmapiInRtd3PmPath(void)
+{
+    return (g_rtd3PmPathThreadId == portThreadGetCurrentThreadId());
 }
 
 //
@@ -673,108 +987,57 @@ rmapiGetClientHandlesFromOSInfo
     NvU32     *pClientHandleListSize
 )
 {
-    OBJSYS *pSys = SYS_GET_INSTANCE();
-
     NvHandle   *pClientHandleList;
     NvU32       clientHandleListSize = 0;
     NvU32       k;
 
-    RmClient **ppClient;
-    RmClient **ppFirstClient;
     RmClient  *pClient;
     RsClient  *pRsClient;
 
-    NvBool clientHandleLookup = pSys->getProperty(pSys, PDB_PROP_SYS_CLIENT_HANDLE_LOOKUP);
+    OsInfoMapSubmap *pSubmap = NULL;
+    OsInfoMapIter it;
+    NvU64 key1 = (NvUPtr)pOSInfo;
 
-    if (!clientHandleLookup)
+    *pClientHandleListSize = 0;
+    *ppClientHandleList = NULL;
+
+    pSubmap = multimapFindSubmap(&g_osInfoList, key1);
+
+    if (pSubmap == NULL)
+        return NV_WARN_NOTHING_TO_DO;
+
+    clientHandleListSize = multimapCountSubmapItems(&g_osInfoList, pSubmap);
+    NV_PRINTF(LEVEL_INFO, "*** Found %d clients for %llx\n", clientHandleListSize, key1);
+
+    if (clientHandleListSize == 0)
     {
-        ppFirstClient = NULL;
-        for (ppClient = serverutilGetFirstClientUnderLock();
-            ppClient;
-            ppClient = serverutilGetNextClientUnderLock(ppClient))
-        {
-            pClient = *ppClient;
-            if (pClient->pOSInfo != pOSInfo)
-            {
-                continue;
-            }
-            clientHandleListSize++;
-
-            if (NULL == ppFirstClient)
-                ppFirstClient = ppClient;
-        }
-
-        if (clientHandleListSize == 0)
-        {
-            *pClientHandleListSize = 0;
-            *ppClientHandleList = NULL;
-            return NV_ERR_INVALID_ARGUMENT;
-        }
-
-        pClientHandleList = portMemAllocNonPaged(clientHandleListSize * sizeof(NvU32));
-        if (pClientHandleList == NULL)
-        {
-            return NV_ERR_NO_MEMORY;
-        }
-
-        *pClientHandleListSize = clientHandleListSize;
-        *ppClientHandleList = pClientHandleList;
-
-        k = 0;
-        for (ppClient = ppFirstClient;
-            ppClient;
-            ppClient = serverutilGetNextClientUnderLock(ppClient))
-        {
-            pClient = *ppClient;
-            pRsClient = staticCast(pClient, RsClient);
-            if (pClient->pOSInfo != pOSInfo)
-            {
-                continue;
-            }
-            pClientHandleList[k++] = pRsClient->hClient;
-
-            if (clientHandleListSize <= k)
-                break;
-        }
+        NV_ASSERT_FAILED("Empty client handle submap");
+        return NV_ERR_INVALID_STATE;
     }
-    else
-    {
-        OsInfoMapSubmap *pSubmap = NULL;
-        OsInfoMapIter it;
-        NvU64 key1 = (NvUPtr)pOSInfo;
-        pSubmap = multimapFindSubmap(&g_osInfoList, key1);
-        if (pSubmap != NULL)
-        {
-            clientHandleListSize = multimapCountSubmapItems(&g_osInfoList, pSubmap);
-            NV_PRINTF(LEVEL_INFO, "*** Found %d clients for %llx\n", clientHandleListSize, key1);
-        }
-        if (clientHandleListSize == 0)
-        {
-            *pClientHandleListSize = 0;
-            *ppClientHandleList = NULL;
-            return NV_ERR_INVALID_ARGUMENT;
-        }
-        pClientHandleList = portMemAllocNonPaged(clientHandleListSize * sizeof(NvU32));
-        if (pClientHandleList == NULL)
-        {
-            return NV_ERR_NO_MEMORY;
-        }
-        *pClientHandleListSize = clientHandleListSize;
-        *ppClientHandleList = pClientHandleList;
-        k = 0;
-        it = multimapSubmapIterItems(&g_osInfoList, pSubmap);
-        while(multimapItemIterNext(&it))
-        {
-            pClient = *it.pValue;
-            pRsClient = staticCast(pClient, RsClient);
-            
-            NV_CHECK_OR_ELSE_STR(LEVEL_ERROR, pClient->pOSInfo == pOSInfo, "*** OS info mismatch", continue);
 
-            pClientHandleList[k++] = pRsClient->hClient;
-            NV_PRINTF(LEVEL_INFO, "*** Found: %x\n", pRsClient->hClient);
-            if (clientHandleListSize <= k)
-                break;
-        }
+    pClientHandleList = portMemAllocNonPaged(clientHandleListSize * sizeof(NvU32));
+
+    if (pClientHandleList == NULL)
+    {
+        return NV_ERR_NO_MEMORY;
+    }
+
+    *pClientHandleListSize = clientHandleListSize;
+    *ppClientHandleList = pClientHandleList;
+
+    k = 0;
+    it = multimapSubmapIterItems(&g_osInfoList, pSubmap);
+    while(multimapItemIterNext(&it))
+    {
+        NV_ASSERT_OR_ELSE(clientHandleListSize > k, break);
+
+        pClient = *it.pValue;
+        pRsClient = staticCast(pClient, RsClient);
+
+        NV_CHECK_OR_ELSE_STR(LEVEL_ERROR, pClient->pOSInfo == pOSInfo, "*** OS info mismatch", continue);
+
+        pClientHandleList[k++] = pRsClient->hClient;
+        NV_PRINTF(LEVEL_INFO, "*** Found: %x\n", pRsClient->hClient);
     }
 
     return NV_OK;
